from controller import controller_main as       _cm
from view import draw as                        _d
from view import frame as                       _f
from view import grammar as                     _g
from view import layer as                       _l
from view import py_form as                     _pf
from view import py_rule as                     _pr
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
import sortalgi as                              _sgi
import                                          sortal
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm

cm = _cm.ControllerMain()
d = _tu.make_hash_suite_dict()
default_layer_name = _s.Settings.default_layer_name
origin = _s.Settings.origin
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message
SortalRule = sortal.grammar.rules.rule.rule

def test_load_rules():
    method_name = 'load_rules'
    try_names = [
        'no_rules_register_empty', 
        '2_rules_sortal_empty', 
        'sortal_not_empty', 
    ]
    set_ups = [
        '_set_up_for_load_rules_no_rules()', 
        '_set_up_for_load_rules_2_rules()', 
        None, 
    ]
    expected_values = [
        0, 
        2, 
        0, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_load_rules(
            method_name, try_names[i], set_ups[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_load_rules_no_rules():
    _vm.ViewMain.clear_all()
    _sgi.sortal_setup()

def _set_up_for_load_rules_2_rules():
    rules = pr_square_to_points, pr_square_to_hash = (
        d['pr_square_to_points'], d['pr_square_to_hash'])
    for i in range(len(rules)):
        rules[i].draw()

def _try_load_rules(method_name, try_name, set_up, expected_value):
    if set_up:
        eval(set_up)
    actual_value = cm.load_rules()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_create_rule():
    method_name = 'create_rule'
    try_names = [
        'ill_formed_rule', 
        'new_rule', 
        'existing_rule_cancel', 
        'existing_rule_duplicate', 
    ]
    rr_square_empty_name = _set_up_for_create_rule()
    instructions = [
        '[Test: Select empty, square, origin]', 
        '[Test: Select square, origin, empty]', 
        '[Test: Select square, origin, empty, no]', 
        '[Test: Select square, origin, empty, yes]', 
    ]
    expected_names = [
        None, 
        rr_square_empty_name, 
        None, 
        '%s_1' % rr_square_empty_name, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_create_rule(
            method_name, 
            try_names[i], 
            instructions[i], 
            expected_names[i])
    print_test_done(method_name)

def _set_up_for_create_rule():
    _sgi.sortal_setup()
    _vm.ViewMain.clear_all()
    _sgi.sortal_setup()
    pf_name = 'pf_square'
    _l.Layer.new(pf_name)
    pf_square = d[pf_name]
    pf_square.draw(pf_name)
    pr_square_to_empty = d['pr_square_to_empty']
    rr_square_empty_name = str(pr_square_to_empty.id_)
    return rr_square_empty_name

def _try_create_rule(method_name, try_name, instruction, expected_name):
    print(instruction)
    actual_value = cm.create_rule()
    if actual_value == None:
        actual_name = None
    else:
        actual_name = actual_value.layer_name
    if not actual_name == expected_name:
        print_test_error_message(
            method_name, try_name, expected_name, actual_name)

def test_get_rule_draw_next_shapes():
    method_name = 'get_rule_draw_next_shapes'
    _sgi.sortal_setup()
    _g.Grammar.clear_all()
    try_names = [
        # 0, x = none, cancel
        # c = current = ells
        # d = current sub = L11
        # r = rule = L22_to_empty
        'bad_state_grammar_empty',              # 0
        '0x',                                   # 1
        '0c_0_0x',                              # 2
        'c_0_r',                                # 3
        'c_d_r',                                # 4
    ]
    _vm.ViewMain.clear_all()
    _sgi.sortal_setup()
    instructions = [
        '[Try 0: Do nothing]',                  # 0
        '[Try 1: 0x]',                          # 1
        '[Try 2: 0ells__0__0x]',                # 2
        '[Try 3: ells__0__L22_to_empty]',       # 3
        '[Try 4: ells__L11__L22_to_empty]',     # 4
    ]
    expected_values = [
        0, # 'bad_state_grammar_empty',         # 0
        0, # cancelled                          # 1
        0, # cancelled                          # 2
        4, # 3_ells                             # 3
        1, # 3_ells                             # 4
    ]
    n = len(try_names)
    for i in range(n):
        if i == 1:
            _set_up_get_rule_draw_next_shapes()
        _try_get_rule_draw_next_shapes(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _set_up_get_rule_draw_next_shapes():
    def _set_up_rule():
        py_rule = pr_L22_to_empty = d['pr_L22_to_empty']
        py_rule.draw()
        name = str(py_rule.id_)
        descr = 'description'
        ag_a = py_rule.a.ag
        ag_b = py_rule.b.ag
        _sgi.create_rule(name, descr, ag_a, ag_b)

    pf_names = ['pf_ells', 'pf_square', 'pf_hash']
    i = 0
    for pf_name in pf_names:
        layer_name = _l.Layer.new(pf_name)
        py_form = d[pf_name]
        rh_form = py_form.draw(layer_name)
        guids = rh_form.els
        translation = (200 * i, 0, 0)
        _rs.MoveObjects(guids, translation)
        i += 1
    _set_up_rule()

def _try_get_rule_draw_next_shapes(
    method_name, try_name, instruction, expected_value
):
    print(instruction)
    actual_value = cm.get_rule_draw_next_shapes()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_get_rule_draw_one_next_shape_repeat(): ##  2018-12-26 19:30
    method_name = 'get_rule_draw_one_next_shape_repeat'
    try_names = [
        'bad_state_grammar_empty',              ##  0
        'n_next_shape_=_0',                     ##  1
        'n_next_shapes_<_n',                    ##  2
        'n_next_shapes_=_n',                    ##  3
        'n_next_shapes_>_n',                    ##  4
    ]
    instructions = [
        "[0 Bad state: do nothing]", 
        "[1 Select square + 0 points; rule; none]", 
        "[2 Select square + 1 points; rule; none]", 
        "[3 Select square + 3 points; rule; none]", 
        "[4 Select square + 4 points; rule; none]", 
    ]
    expected_forms = [
        '0 not applicable', 
        '1 empty', 
        '2 square', 
        '3 square', 
        '4 square with 1 point', 
    ]
    none_type = type(None)
    expected_types = [
        none_type,                              ##  0
        none_type,                              ##  1
        _pf.PyForm,                             ##  2
        _pf.PyForm,                             ##  3
        _pf.PyForm,                             ##  4
    ]
    n = len(try_names)
    for i in range(n):
        print('i: %i' % i)
        if i == 0:
            _set_up_for_grd1nsr_0()
        elif _g.Grammar.is_empty():
            _set_up_for_grd1nsr_i()
        else:
            pass
        _try_get_rule_draw_one_next_shape_repeat(
            method_name, 
            try_names[i], 
            instructions[i], 
            expected_forms[i], 
            expected_types[i], 
        )
    print_test_done(method_name)

def _set_up_for_grd1nsr_0():
    _vm.ViewMain.clear_all()
    _sgi.sortal_setup()
    _l.Layer.new('Shape 0')

def _set_up_for_grd1nsr_i():
    _vm.ViewMain.clear_all()
    _sgi.sortal_setup()
    _l.Layer.new('Shape 0')
    _draw_square_with_4_points()
    _create_rule_point_to_empty()

def _draw_square_with_4_points():
    pf_square_4 = _tu.make_modular_py_form('square', ['43', '34', '23', '32'])
    rh_square_4 = pf_square_4.draw(default_layer_name)
    translation = (0, -150, 0)
    _rs.MoveObjects(rh_square_4.els, translation)

def _create_rule_point_to_empty():
    pf_point = _tu.make_modular_py_form('', ['33'])
    pf_empty = _pf.PyForm.new_empty()
    p_to_e = pr_point_to_empty = _pr.PyRule(pf_point, pf_empty)
    p_to_e.draw()
    ag_left = p_to_e.a.ag
    ag_right = p_to_e.b.ag
    name = str(p_to_e.id_)
    result = cm.create_sortal_rule(ag_left, ag_right, name)
    print('result')

def _try_get_rule_draw_one_next_shape_repeat(
        method_name, try_name, instruction, expected_form, expected_type
    ):
    print(instruction)
    actual_value = cm.get_rule_draw_one_next_shape_repeat()
    actual_type = type(actual_value)
    if not actual_type == expected_type:
        print_test_error_message(
            method_name, try_name, expected_type, actual_type)
    if actual_value:
        confirm_message = '%s %s' % (
            "Confirm final shape:", expected_form)
        _rs.GetString(confirm_message)

def test_apply_rule_draw_one_next_shape_repeat():
    method_name = 'apply_rule_draw_one_next_shape_repeat'
    try_names = [
        'shapes_with_points_0',                             # 0
        'shapes_with_points_1_0',                           # 1
        'shapes_with_points_2_1_0',                         # 2
        'shapes_with_points_3_2_1_0',                       # 3
        'shapes_with_points_4_3_2_1',                       # 4
    ]
    [   pf_square, 
        pf_square_pt_e, 
        pf_square_pt_n, 
        pf_square_pt_w, 
        pf_square_pt_s, 
        pf_square_pt_en, 
        pf_square_pt_enw, 
        pf_square_pt_enws, 
        point_to_empty_name, 
        n_reps, 
    ] = _set_up_for_ard1nsr()
    arg_lists = [
        [pf_square, point_to_empty_name, n_reps],           # 0
        [pf_square_pt_e, point_to_empty_name, n_reps],      # 1
        [pf_square_pt_en, point_to_empty_name, n_reps],     # 2
        [pf_square_pt_enw, point_to_empty_name, n_reps],    # 3
        [pf_square_pt_enws, point_to_empty_name, n_reps],   # 4
    ]
    expected_value_lists = [
        None,                                               # 0
        [pf_square],                                        # 1
        [pf_square],                                        # 2
        [pf_square],                                        # 3
        [   pf_square, 
            pf_square_pt_e, 
            pf_square_pt_n, 
            pf_square_pt_w, 
            pf_square_pt_s, 
        ],                                                  # 4
    ]
    n_try_names = len(try_names)
    for i in range(n_try_names):
        py_form_to_draw = arg_lists[i][0]
        _pf.PyForm.draw_many_nicely([py_form_to_draw])
        _try_apply_rule_draw_one_next_shape_repeat(
            method_name, try_names[i], arg_lists[i], expected_value_lists[i])
        message = "Check shape: %s" % try_names[i]
        _rs.GetString(message)
    print_test_done(method_name)

def _set_up_for_ard1nsr():
    def _create_point_to_empty():
        pr_point_to_empty = d['pr_point_to_empty']
        pr_point_to_empty.draw()
        pr_point_to_empty_name = str(pr_point_to_empty.id_)
        ag_left = pr_point_to_empty.a.ag
        ag_right = pr_point_to_empty.b.ag
        _cm.ControllerMain.create_sortal_rule(
            ag_left, ag_right, pr_point_to_empty_name)
        return pr_point_to_empty_name
        
    _sgi.sortal_setup()
    _vm.ViewMain.clear_all()
    _l.Layer.new('Shape 0')
    line_shape_name = 'square'
    e, n, w, s = '43', '34', '23', '32'
    point_name_lists = [
        [], 
        [e], 
        [n], 
        [w], 
        [s], 
        [e, n], 
        [e, n, w], 
        [e, n, w, s], 
    ]
    ard1nsr_args = []
    for point_names in point_name_lists:
        pf = _tu.make_modular_py_form(line_shape_name, point_names)
        ard1nsr_args.append(pf)

    pr_point_to_empty_name = _create_point_to_empty()
    n_reps = 3
    ard1nsr_args.extend([pr_point_to_empty_name, n_reps])
    return ard1nsr_args

def _try_apply_rule_draw_one_next_shape_repeat(
        method_name, try_name, args, possible_values
    ):
        curr_pf, layer_name, n = args
        actual_value = cm.apply_rule_draw_one_next_shape_repeat(
            curr_pf, layer_name, n)
        if (not actual_value and
            not possible_values
        ):
            pass
        else:
            if not actual_value in possible_values:
                print_test_error_message(
                    method_name, try_name, possible_values, actual_value)

def test_apply_rule_draw_one_next_shape():
    method_name = 'apply_rule_draw_one_next_shape'
    try_names = [
        'bad_state_grammar_empty',              # 0
        'bad_type_curr_pf',                     # 1
        'bad_type_layer_name',                  # 2

        'bad_value_layer_name_non_existent',    # 3
        'bad_value_curr_pf_ill_formed',         # 4
        'bad_value_layer_name_not_in_sortal',   # 5

        'no_next_shape',                        # 6
        'one_next_shape',                       # 7
        'several_next_shapes',                  # 8
    ]
    next_pf = next_py_form = d['pf_ells']
    next_pfs = next_py_forms = [
        d['pf_L11'], 
        d['pf_L12'], 
        d['pf_L21'], 
        d['pf_L22'], 
    ]
    expected_values = [
        None,                                   # 0
        None,                                   # 1
        None,                                   # 2

        None,                                   # 3
        None,                                   # 4
        None,                                   # 5

        None,                                   # 6
        next_pf,                                # 7
        next_pfs,                               # 8
    ]
    n = len(try_names)
    for i in range(n):
        curr_pf, layer_name = _set_up_for_ard1ns(i)
        _try_apply_rule_draw_one_next_shape(
            method_name, 
            try_names[i], 
            curr_pf, 
            layer_name, 
            expected_values[i], 
        )
    print_test_done(method_name)

def _set_up_for_ard1ns(i):
    _vm.ViewMain.clear_all()
    _l.Layer.new('Shape 0')
    if i == 0:
        curr_pf = 'curr_pf_kilroy'
        layer_name = 'layer_name_kilroy'
    elif i == 1:
        curr_pf = bad_type = 37
        py_rule = _draw_hash_to_empty_rule_for_ard1ns()
        layer_name = str(py_rule.id_)
    elif i == 2:
        curr_pf = _draw_hash_shape_for_ard1ns()
        layer_name = 37
    elif i == 3:
        curr_pf = _draw_hash_shape_for_ard1ns()
        _draw_hash_to_empty_rule_for_ard1ns()
        layer_name = 'layer_name_kilroy'
    elif i == 4:
        curr_pf = _pf.PyForm(37, [])
        py_rule = _draw_hash_to_empty_rule_for_ard1ns()
        layer_name = str(py_rule.id_)
    elif i == 5:
        curr_pf = _draw_hash_shape_for_ard1ns()
        py_rule = _draw_hash_to_empty_rule_for_ard1ns()
        layer_name = str(py_rule.id_)
    elif i == 6:
        curr_pf = _draw_square_shape_for_ard1ns()
        py_rule = _draw_hash_to_empty_rule_for_ard1ns()
        layer_name = str(py_rule.id_)
        _create_sortal_rule(py_rule, layer_name)
    elif i == 7:
        curr_pf = _draw_hash_shape_for_ard1ns()
        py_rule = _draw_square_to_empty_rule_for_ard1ns()
        layer_name = str(py_rule.id_)
        _create_sortal_rule(py_rule, layer_name)
    elif i == 8:
        curr_pf = _draw_ells_shape_for_ard1ns()
        py_rule = _draw_L11L12L21_to_empty_rule_for_ard1ns()
        layer_name = str(py_rule.id_)
        _create_sortal_rule(py_rule, layer_name)
    else:
        message = "index out of range"
        print(message)
    return (curr_pf, layer_name)

def _create_sortal_rule(py_rule, layer_name):
    _sgi.sortal_setup()
    ag_left = py_rule.a.ag
    ag_right = py_rule.b.ag
    cm.create_sortal_rule(ag_left, ag_right, layer_name)

def _draw_hash_shape_for_ard1ns():
    pf_hash = d['pf_hash']
    pf_hash.draw('Shape 0')
    return pf_hash

def _draw_ells_shape_for_ard1ns():
    pf_ells = d['pf_ells']
    pf_ells.draw('Shape 0')
    return pf_ells

def _draw_square_shape_for_ard1ns():
    pf_square = d['pf_square']
    pf_square.draw('Shape 0')
    return pf_square

def _draw_hash_to_empty_rule_for_ard1ns():
    pr_hash_to_empty = d['pr_hash_to_empty']
    pr_hash_to_empty.draw()
    return pr_hash_to_empty

def _draw_square_to_empty_rule_for_ard1ns():
    pr_square_to_empty = d['pr_square_to_empty']
    pr_square_to_empty.draw()
    return pr_square_to_empty

def _draw_L11L12L21_to_empty_rule_for_ard1ns():
    pr_L11L12L21_to_empty = d['pr_L11L12L21_to_empty']
    pr_L11L12L21_to_empty.draw()
    return pr_L11L12L21_to_empty

def _try_apply_rule_draw_one_next_shape(
    method_name, try_name, curr_pf, layer_name, expected_value
):
    actual_value = cm.apply_rule_draw_one_next_shape(curr_pf, layer_name)
    expected_type = type(expected_value)
    if (not expected_value or
        expected_type == _pf.PyForm
    ):
        if not actual_value == expected_value:
            print_test_error_message(
                method_name, try_name, expected_value, actual_value)
    elif expected_type == list:
        possible_values = expected_value
        if actual_value not in possible_values:
            print_test_error_message(
                method_name, try_name, expected_value, actual_value)
    else:
        message = "Shouldn't be here!"
        print(message)

def test_apply_rule_by_name():
    method_name = 'apply_rule_by_name'
    try_names = [
        'bad_type_c', 
        'bad_type_d', 
        'bad_type_id', 
        'bad_value_id_rhino_no_rule', 
        'bad_value_id_sortal_no_rule', 
        'square_to_hash_on_square', 
    ]
    subsequent_set_ups = [
        None, 
        None, 
        None, 
        None, 
        'py_rule.draw()', 
        'cm.create_sortal_rule(py_rule.a.ag, py_rule.b.ag, str(py_rule.id_))', 
    ]
    pf_square, pf_hash, pr_square_to_hash = _set_up_for_apply_rule_by_name()
    pr_square_to_hash_name = str(pr_square_to_hash.id_)
    input_tuples = [
        (37, pf_square, pr_square_to_hash_name), 
        (pf_square, 37, pr_square_to_hash_name), 
        (pf_square, pf_square, 37), 
        (pf_square, pf_square, '37'), 
        (pf_square, pf_square, pr_square_to_hash_name), 
        (pf_square, pf_square, pr_square_to_hash_name), 
    ]
    expected_values = [
        None, 
        None, 
        None, 
        None, 
        None, 
        [pf_hash], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_apply_rule_by_name(
            method_name, 
            try_names[i], 
            subsequent_set_ups[i], 
            input_tuples[i], 
            expected_values[i])
    print_test_done(method_name)

def _set_up_for_apply_rule_by_name():
    sortal.rule.cleanup()
    _sgi.sortal_setup()
    _vm.ViewMain.clear_all()
    pf_square, pf_hash, pr_square_to_hash = (
        d['pf_square'], d['pf_hash'], d['pr_square_to_hash'])
    return (pf_square, pf_hash, pr_square_to_hash)

def _try_apply_rule_by_name(
    method_name, try_name, set_up, tuple_, expected_value
):
    if set_up:
        py_rule = d['pr_square_to_hash']
        eval(set_up)
    pf_c, pf_d, pr_id = tuple_
    actual_value = cm.apply_rule_by_name(pf_c, pf_d, pr_id)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_apply_py_rule():
    method_name = 'apply_py_rule'
    try_names = [
        'bad_type_c', 
        'bad_type_d', 
        'bad_type_rule', 
        'bad_type_name', 
        'not_bad_value_c_empty', 
        'bad_value_d_!<=_c', 
        'hash_minus_ells', 
        'ells_minus_ells', 
    ]
    pf_empty, pf_square, pf_ells, pf_hash = (
        d['pf_empty'], d['pf_square'], d['pf_ells'], d['pf_hash'])
    pr_ells_to_empty = d['pr_ells_to_empty']
    rule_name = str(pr_ells_to_empty.id_)
    input_tuples = [
        (37, pf_hash, pr_ells_to_empty, rule_name), 
        (pf_hash, 37, pr_ells_to_empty, rule_name), 
        (pf_hash, pf_hash, 37, rule_name), 
        (pf_hash, pf_hash, pr_ells_to_empty, 37), 
        (pf_empty, pf_empty, pr_ells_to_empty, rule_name), 
        (pf_ells, pf_square, pr_ells_to_empty, rule_name), ##
        (pf_hash, pf_hash, pr_ells_to_empty, rule_name), 
        (pf_ells, pf_ells, pr_ells_to_empty, rule_name), 
    ]
    expected_values = [
        None, 
        None, 
        None, 
        None, 
        [], 
        [],                                     ##  I'd have thought None
        [pf_square], 
        [pf_empty], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_apply_py_rule(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_apply_py_rule(method_name, try_name, input_tuple, expected_value):
    _vm.ViewMain.clear_all()
    sortal.rule.cleanup()
    _sgi.sortal_setup()
    py_c, py_d, py_rule, rule_name = input_tuple
    actual_value = cm.apply_py_rule(py_c, py_d, py_rule, rule_name)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_create_sortal_rules():
    method_name = 'create_sortal_rules'
    try_names = [
        'different_lengths', 
        'rules_empty', 
        'names_empty', 
        '0', 
        '1_new', 
        '1_existing', 
        '1_new_1_existing', 
        '2_new', 
    ]
    sortal.rule.cleanup()
    _sgi.sortal_setup()
    name_0, name_1, name_2, name_3 = (
        'pr_points_to_square', 
        'pr_square_to_ells', 
        'pr_ells_to_hash', 
        'pr_hash_to_empty', 
    )
    py_rule_0, py_rule_1, py_rule_2, py_rule_3 = (
        d[name_0], 
        d[name_1], 
        d[name_2], 
        d[name_3], 
    )
    input_list_pairs = [
        ([py_rule_0, py_rule_1], [name_0]), 
        ([], [name_0]), 
        ([py_rule_0], []), 
        ([], []), 
        ([py_rule_0], [name_0]), 
        ([py_rule_0], [name_0]), 
        ([py_rule_1, py_rule_0], [name_1, name_0]), 
        ([py_rule_2, py_rule_3], [name_2, name_3]), 
    ]
    expected_values = [
        0, 
        0, 
        0, 
        0, 
        1, 
        0, 
        1, 
        2, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_create_sortal_rules(
            method_name, try_names[i], input_list_pairs[i], expected_values[i])
    print_test_done(method_name)

def _try_create_sortal_rules(
    method_name, try_name, list_pair, expected_value
):
    py_rules, names = list_pair
    actual_value = cm.create_sortal_rules(py_rules, names)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_create_sortal_rule():
    method_name = 'create_sortal_rule'
    ag_empty = d['ag_empty']
    ag_square = d['ag_square']
    ag_ells = d['ag_ells']
    ag_hash = d['ag_hash']
    _sgi.sortal_setup()
    _sgi.create_rule('hash_to_empty', 'delete_hash', ag_hash, ag_empty)
    try_names = [
        'bad_type_ag_left', 
        'bad_type_ag_right', 
        'bad_type_name', 
        'bad_type_descr', 
        'bad_value_ag_left__empty', 
        'bad_value_name_in_sortal_register', 
        'square_to_ells', 
    ]
    input_tuples = [
        (37, ag_ells, '37_to_ells', '<descr>'), 
        (ag_square, 37, 'square_to_37', '<descr>'), 
        (ag_square, ag_ells, 37, '<descr>'), 
        (ag_square, ag_ells, 'square_to_ells', 37), 
        (ag_empty, ag_ells, 'empty_to_ells', '<descr>'), 
        (ag_square, ag_ells, 'hash_to_empty', '<descr>'), 
        (ag_square, ag_ells, 'square_to_ells', '<descr>'), 
    ]
    expected_values = [
        None, 
        None, 
        None, 
        None, 
        None, 
        None, 
        (SortalRule, 'square_to_ells'), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_create_sortal_rule(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_create_sortal_rule(
    method_name, try_name, input_tuple, expected_value
):
    ag_left, ag_right, name, descr = input_tuple
    return_value = _cm.ControllerMain.create_sortal_rule(
        ag_left, ag_right, name, descr)
    if return_value == None:
        actual_value = return_value
    else:
        sortal_rule = return_value
        rule_name_out = _sgi.getRuleName(sortal_rule)
        actual_value = (type(sortal_rule), rule_name_out)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_is_in_sortal_register():                  ##  to be deprecated?
    method_name = 'is_in_sortal_register'
    try_names = [
        'bad_type', 
        'name_absent', 
        'name_present', 
    ]
    _make_sgi_rule_delete_square()
    names = [
        37, 
        'delete_hash', 
        'delete_square', 
    ]
    expected_values = [
        False, 
        False, 
        True, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_is_in_sortal_register(
            method_name, 
            try_names[i], 
            names[i], 
            expected_values[i], 
        )
    print_test_done(method_name)

def _make_sgi_rule_delete_square():
    _sgi.sortal_setup()
    name_in = 'delete_square'
    descr = '<descr>'
    ag_left = d['ag_square']
    ag_right = d['ag_empty']
    _sgi.create_rule(name_in, descr, ag_left, ag_right)

def _try_is_in_sortal_register(
    method_name, try_name, name, expected_value
):
    actual_value = _cm.ControllerMain.is_in_sortal_register(name)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_is_a_sortal_rule():
    method_name = 'is_a_sortal_rule'
    try_names = [
        'bad_type', 
        'rule_exists', 
        'rule_does_not_exist', 
    ]
    _set_up_for_is_a_sortal_rule()
    rule_ids = [
        37, 
        'hash_to_empty', 
        'hash_to_ells', 
    ]
    expected_values = [
        False, 
        True, 
        False, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_is_a_sortal_rule(
            method_name, try_names[i], rule_ids[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_is_a_sortal_rule():
    sortal.rule.cleanup()
    _sgi.sortal_setup()
    rule_name_in = 'hash_to_empty'
    descr = 'deletes hash'
    ag_left = d['ag_hash']
    ag_right = d['ag_empty']
    print('ag_left: %s' % ag_left)
    rule, rule_name_out = _sgi.create_rule(
        rule_name_in, descr, ag_left, ag_right)

def _try_is_a_sortal_rule(method_name, try_name, rule_id, expected_value):
    actual_value = _cm.ControllerMain.is_a_sortal_rule(rule_id)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_delete_empty_layers():
    method_name = 'delete_empty_layers'
    try_names = [
        # 'bad_state', 
        # default, occupied, empty
        '000', '002', '020', '022', 
        '100', '102', '120', '122', 
    ]
    expected_values = [
        0, 2, 0, 2, 
        0, 2, 0, 2, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_delete_empty_layers(
            method_name, try_names[i], expected_values[i])
    print_test_done(method_name)

def _try_delete_empty_layers(method_name, try_name, expected_value):
    _vm.ViewMain.clear_all()
    [d, o, e] = nn = [int(a) for a in try_name]
    [_draw_on_default_layer() for i in range(d)]
    [_make_occupied_layer(i) for i in range(o)]
    [_make_empty_layer(i) for i in range(e)]
    actual_value = cm.delete_empty_layers()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def _draw_on_default_layer():
    _d.Draw._draw_framed_object_group(origin, 'i')

def _make_occupied_layer(i):
    layer_name = 'occupied_%i' % (i + 1)
    _l.Layer.new(layer_name)
    _rs.CurrentLayer(layer_name)
    draw_point = (100 * i, 100, 0)
    _d.Draw._draw_framed_object_group(draw_point, 'i')

def _make_empty_layer(i):
    layer_name = 'empty_%i' % (i + 1)
    _l.Layer.new(layer_name)

test_load_rules()
test_create_rule()
test_get_rule_draw_next_shapes()
## _set_up_get_rule_draw_next_shapes()
test_apply_rule_by_name()
test_apply_py_rule()
test_create_sortal_rules()
test_create_sortal_rule()
test_is_a_sortal_rule()
test_delete_empty_layers()

