from view import py_form as                     _pf
import sortalgi as                              _sgi
import                                          sortal
from view.tests import test_utilities as             _tu

d = _tu.make_hash_suite_dict()
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message

def test_getRule():
    method_name = 'getRule'
    try_names = [
        'bad_type_non_str', 
        'rule_exists', 
        'rule_does_not_exist', 
    ]
    existing_rule_name = _set_up_for_getRule()
    inputs = [
        [], 
        existing_rule_name, 
        'kilroy', 
    ]
    expected_values = [
        None,               # Message box: Name entered is not a string
        sortal.grammar.rules.rule.rule, 
                            # hash_to_empty
        None,               # Message box: Rule does not exist
    ]
    n = len(try_names)
    for i in range(n):
        _try_getRule(method_name, try_names[i], inputs[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_getRule():
    _sgi.sortal_setup()
    name = 'hash_to_empty'
    descr = '<descr>'
    ag_left, ag_right = d['ag_hash'], d['ag_empty']
    rule_out, name_out = _sgi.create_rule(name, descr, ag_left, ag_right)
    return name_out

def _try_getRule(method_name, try_name, input_, expected_value):
    received_value = _sgi.getRule(input_)
    # print('%s: %s' % (try_name, str(received_value)))
    if received_value == None:
        actual_value = received_value
    else:
        rule_out = received_value
        actual_value = type(rule_out)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

    """create_rule(name, descr, ag_a, ag_b)
        Receives:
            name_in             str or num. List fatal: 'list objects are 
                                unhashable'
            descr               str, num, or list. Any type?
            ag_a                ag_shape. Int fatal: 'len() of 
                                unsized object'
            ag_b                ag_shape. List ok
        Creates a rule instance. If name_in is already in use, the associated 
        rule is overwritten. Returns:
            rule                SortalRule. The new rule object
            name_out            str or num. The name of the new rule
    """

def test_create_rule():
    method_name = 'create_rule'
    try_names = [
        # 'fatal_type_name_list', 
        # 'fatal_type_name_tuple', 
        # 'fatal_type_name_int'
        # 'fatal_type_ag_1_int', 
        # 'fatal_type_ag_1_non_empty_list', 
        # 'fatal_type_ag_1_tuple_of_str', 

        'ok_type_ag_1_str', 
        'ok_type_ag_1_empty_list', 
        'ok_type_ag_1_empty_dict', 
        'ok_type_ag_1_non_empty_dict', 
        'ok_type_descr_num', 

        'ag0_ag0', 
        'ag1_ag0', 
        'ag0_ag1', 
        'ag1_ag1', 
        'ag1_ag2', 
    ]
    descr = '<descr>'
    ag_empty, ag_square, ag_hash = d['ag_empty'], d['ag_square'], d['ag_hash']
    pf_empty, pf_square, pf_hash = d['pf_empty'], d['pf_square'], d['pf_hash']
    input_tuples = [
        ('name_0', descr, 'x', ag_square), 
        ('name_1', descr, [], ag_square), 
        ('name_2', descr, {}, ag_square), 
        ('name_3', descr, {'a': 1}, ag_square), 
        ('name_4', 37, ag_hash, ag_square), 

        ('empty_to_empty', descr, ag_empty, ag_empty), 
        ('hash_to_empty', descr, ag_hash, ag_empty), 
        ('empty_to_hash', descr, ag_empty, ag_hash), 
        ('hash_to_hash', descr, ag_hash, ag_hash), 
        ('hash_to_square', descr, ag_hash, ag_square), 
    ]
    expected_values = [
        (pf_empty, pf_square, 'name_0'),            # no lines
        (pf_empty, pf_square, 'name_1'),            # no lines
        (pf_empty, pf_square, 'name_2'),            # no lines
        (pf_empty, pf_square, 'name_3'),            # no lines
        (pf_hash, pf_square, 'name_4'),             # no lines

        (pf_empty, pf_empty, 'empty_to_empty'), 
        (pf_hash, pf_empty, 'hash_to_empty'),       # no lines
        (pf_empty, pf_hash, 'empty_to_hash'),       # no lines
        (pf_hash, pf_hash, 'hash_to_hash'),         # no lines
        (pf_hash, pf_square, 'hash_to_square'),     # no lines
    ]
    n = len(try_names)
    for i in range(n):
        _try_create_rule(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_create_rule(method_name, try_name, input_tuple, expected_value):
    _sgi.sortal_setup()
    name_in, descr, ag_a, ag_b = input_tuple
    rule, name_out = received_value = _sgi.create_rule(name_in, descr, ag_a, ag_b)
    ag_left = _sgi.getLHS(rule)
    ag_right = _sgi.getRHS(rule)
    pf_left = _pf.PyForm.new_from_agnostic(ag_left)
    pf_right = _pf.PyForm.new_from_agnostic(ag_right)
    actual_value = (pf_left, pf_right, name_out)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

"""
    name_0 has been created.
    create_rule: ok_type_ag_1_str:
        expected (
            _pf.PyForm([], []), 
            _pf.PyForm(
                [], 
                [   _pl.PyLine((170, 170, 0), (170, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 170, 0), (230, 170, 0), [], 1.0), 
                    _pl.PyLine((170, 230, 0), (230, 230, 0), [], 1.0), 
                    _pl.PyLine((230, 170, 0), (230, 230, 0), [], 1.0)]), 
            'name_0')
        got (
            _pf.PyForm([], []), 
            _pf.PyForm([], []), 
            'name_0')
    name_1 has been created.
    create_rule: ok_type_ag_1_empty_list:
        expected (
            _pf.PyForm([], []), 
            _pf.PyForm(
                [], 
                [   _pl.PyLine((170, 170, 0), (170, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 170, 0), (230, 170, 0), [], 1.0), 
                    _pl.PyLine((170, 230, 0), (230, 230, 0), [], 1.0), 
                    _pl.PyLine((230, 170, 0), (230, 230, 0), [], 1.0)]), 
            'name_1')
        got (
            _pf.PyForm([], []), 
            _pf.PyForm([], []), 
            'name_1')
    name_2 has been created.
    create_rule: ok_type_ag_1_empty_dict:
        expected (
            _pf.PyForm([], []), 
            _pf.PyForm(
                [], 
                [   _pl.PyLine((170, 170, 0), (170, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 170, 0), (230, 170, 0), [], 1.0), 
                    _pl.PyLine((170, 230, 0), (230, 230, 0), [], 1.0), 
                    _pl.PyLine((230, 170, 0), (230, 230, 0), [], 1.0)]), 
            'name_2')
        got (
            _pf.PyForm([], []), 
            _pf.PyForm([], []), 
            'name_2')
    name_3 has been created.
    create_rule: ok_type_ag_1_non_empty_dict:
        expected (
            _pf.PyForm([], []), 
            _pf.PyForm(
                [], 
                [   _pl.PyLine((170, 170, 0), (170, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 170, 0), (230, 170, 0), [], 1.0), 
                    _pl.PyLine((170, 230, 0), (230, 230, 0), [], 1.0), 
                    _pl.PyLine((230, 170, 0), (230, 230, 0), [], 1.0)]), 
            'name_3')
        got (
            _pf.PyForm([], []), 
            _pf.PyForm([], []), 
            'name_3')
    name_4 has been created.
    create_rule: ok_type_descr_num:
        expected (
            _pf.PyForm(
                [   _pp.PyPoint(150, 170, 0, ['a']), 
                    _pp.PyPoint(150, 230, 0, ['a']), 
                    _pp.PyPoint(170, 150, 0, ['a']), 
                    _pp.PyPoint(170, 250, 0, ['a']), 
                    _pp.PyPoint(230, 150, 0, ['a']), 
                    _pp.PyPoint(230, 250, 0, ['a']), 
                    _pp.PyPoint(250, 170, 0, ['a']), 
                    _pp.PyPoint(250, 230, 0, ['a'])], 
                [   _pl.PyLine((150, 170, 0), (250, 170, 0), [], 1.0), 
                    _pl.PyLine((150, 230, 0), (250, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 150, 0), (170, 250, 0), [], 1.0), 
                    _pl.PyLine((230, 150, 0), (230, 250, 0), [], 1.0)]), 
            _pf.PyForm(
                [], 
                [   _pl.PyLine((170, 170, 0), (170, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 170, 0), (230, 170, 0), [], 1.0), 
                    _pl.PyLine((170, 230, 0), (230, 230, 0), [], 1.0), 
                    _pl.PyLine((230, 170, 0), (230, 230, 0), [], 1.0)]), 
            'name_4')
        got (   
            _pf.PyForm(
                [   _pp.PyPoint(150.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(150.0, 230.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 230.0, 0.0, ['a'])], 
                []), 
            _pf.PyForm([], []), 
            'name_4')
    empty_to_empty has been created.
    hash_to_empty has been created.
    create_rule: ag1_ag0:
        expected (
            _pf.PyForm(
                [   _pp.PyPoint(150, 170, 0, ['a']), 
                    _pp.PyPoint(150, 230, 0, ['a']), 
                    _pp.PyPoint(170, 150, 0, ['a']), 
                    _pp.PyPoint(170, 250, 0, ['a']), 
                    _pp.PyPoint(230, 150, 0, ['a']), 
                    _pp.PyPoint(230, 250, 0, ['a']), 
                    _pp.PyPoint(250, 170, 0, ['a']), 
                    _pp.PyPoint(250, 230, 0, ['a'])], 
                [   _pl.PyLine((150, 170, 0), (250, 170, 0), [], 1.0), 
                    _pl.PyLine((150, 230, 0), (250, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 150, 0), (170, 250, 0), [], 1.0), 
                    _pl.PyLine((230, 150, 0), (230, 250, 0), [], 1.0)]), 
            _pf.PyForm([], []), 
            'hash_to_empty')
        got (
            _pf.PyForm(
                [   _pp.PyPoint(150.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(150.0, 230.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 230.0, 0.0, ['a'])], 
                []), 
            _pf.PyForm([], []), 
            'hash_to_empty')
    empty_to_hash has been created.
    create_rule: ag0_ag1:
        expected (
            _pf.PyForm([], []), 
            _pf.PyForm(
                [   _pp.PyPoint(150, 170, 0, ['a']), 
                    _pp.PyPoint(150, 230, 0, ['a']), 
                    _pp.PyPoint(170, 150, 0, ['a']), 
                    _pp.PyPoint(170, 250, 0, ['a']), 
                    _pp.PyPoint(230, 150, 0, ['a']), 
                    _pp.PyPoint(230, 250, 0, ['a']), 
                    _pp.PyPoint(250, 170, 0, ['a']), 
                    _pp.PyPoint(250, 230, 0, ['a'])], 
                [   _pl.PyLine((150, 170, 0), (250, 170, 0), [], 1.0), 
                    _pl.PyLine((150, 230, 0), (250, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 150, 0), (170, 250, 0), [], 1.0), 
                    _pl.PyLine((230, 150, 0), (230, 250, 0), [], 1.0)]), 
            'empty_to_hash')
        got (
            _pf.PyForm([], []), 
            _pf.PyForm(
                [   _pp.PyPoint(150.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(150.0, 230.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 230.0, 0.0, ['a'])], 
                []), 
            'empty_to_hash')
    hash_to_hash has been created.
    create_rule: ag1_ag1:
        expected (
            _pf.PyForm(
                [   _pp.PyPoint(150, 170, 0, ['a']), 
                    _pp.PyPoint(150, 230, 0, ['a']), 
                    _pp.PyPoint(170, 150, 0, ['a']), 
                    _pp.PyPoint(170, 250, 0, ['a']), 
                    _pp.PyPoint(230, 150, 0, ['a']), 
                    _pp.PyPoint(230, 250, 0, ['a']), 
                    _pp.PyPoint(250, 170, 0, ['a']), 
                    _pp.PyPoint(250, 230, 0, ['a'])], 
                [   _pl.PyLine((150, 170, 0), (250, 170, 0), [], 1.0), 
                    _pl.PyLine((150, 230, 0), (250, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 150, 0), (170, 250, 0), [], 1.0), 
                    _pl.PyLine((230, 150, 0), (230, 250, 0), [], 1.0)]), 
            _pf.PyForm(
                [   _pp.PyPoint(150, 170, 0, ['a']), 
                    _pp.PyPoint(150, 230, 0, ['a']), 
                    _pp.PyPoint(170, 150, 0, ['a']), 
                    _pp.PyPoint(170, 250, 0, ['a']), 
                    _pp.PyPoint(230, 150, 0, ['a']), 
                    _pp.PyPoint(230, 250, 0, ['a']), 
                    _pp.PyPoint(250, 170, 0, ['a']), 
                    _pp.PyPoint(250, 230, 0, ['a'])], 
                [   _pl.PyLine((150, 170, 0), (250, 170, 0), [], 1.0), 
                    _pl.PyLine((150, 230, 0), (250, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 150, 0), (170, 250, 0), [], 1.0), 
                    _pl.PyLine((230, 150, 0), (230, 250, 0), [], 1.0)]), 
            'hash_to_hash')
        got (
            _pf.PyForm(
                [   _pp.PyPoint(150.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(150.0, 230.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 230.0, 0.0, ['a'])], 
                []), 
            _pf.PyForm(
                [   _pp.PyPoint(150.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(150.0, 230.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 230.0, 0.0, ['a'])], 
                []), 
            'hash_to_hash')
    hash_to_square has been created.
    create_rule: ag1_ag2:
        expected (
            _pf.PyForm(
                [   _pp.PyPoint(150, 170, 0, ['a']), 
                    _pp.PyPoint(150, 230, 0, ['a']), 
                    _pp.PyPoint(170, 150, 0, ['a']), 
                    _pp.PyPoint(170, 250, 0, ['a']), 
                    _pp.PyPoint(230, 150, 0, ['a']), 
                    _pp.PyPoint(230, 250, 0, ['a']), 
                    _pp.PyPoint(250, 170, 0, ['a']), 
                    _pp.PyPoint(250, 230, 0, ['a'])], 
                [   _pl.PyLine((150, 170, 0), (250, 170, 0), [], 1.0), 
                    _pl.PyLine((150, 230, 0), (250, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 150, 0), (170, 250, 0), [], 1.0), 
                    _pl.PyLine((230, 150, 0), (230, 250, 0), [], 1.0)]), 
            _pf.PyForm(
                [], 
                [   _pl.PyLine((170, 170, 0), (170, 230, 0), [], 1.0), 
                    _pl.PyLine((170, 170, 0), (230, 170, 0), [], 1.0), 
                    _pl.PyLine((170, 230, 0), (230, 230, 0), [], 1.0), 
                    _pl.PyLine((230, 170, 0), (230, 230, 0), [], 1.0)]), 
            'hash_to_square')
        got (
            _pf.PyForm(
                [   _pp.PyPoint(150.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(150.0, 230.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(170.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 150.0, 0.0, ['a']), 
                    _pp.PyPoint(230.0, 250.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 170.0, 0.0, ['a']), 
                    _pp.PyPoint(250.0, 230.0, 0.0, ['a'])], []), 
            _pf.PyForm([], []), 
            'hash_to_square')
    Done: test_create_rule
"""

def test_find_rule_appns():
    method_name = 'find_rule_appns'
    try_names = [
        'register_empty_rule_instance', 
        'register_not_empty_rule_name', 
        'ok_value_ag_c_empty', 
        'bad_value_ag_d_not_subshape', 
        'no_next_shapes', 
        'next_shapes', 
        # 'fatal_type_rule_inst', 
        # 'fatal_type_ag_c', 
        # 'fatal_type_ag_d', 
        # 'fatal_value_rule_inst_non_existent', 
        # 'bad_value_ag_c_1_point', 
        # 'bad_value_ag_c_2_points', 
        # 'bad_value_ag_c_3_points_colinear', 
        # 'bad_value_ag_c_3_points_non_colinear', 
        # 'bad_value_ag_c_1_point_1_line_colinear', 
        # 'bad_value_ag_c_1_point_1_line_non_colinear', 
        # 'bad_value_ag_c_2_lines_colinear', 
        # 'bad_value_ag_c_2_lines_non_colinear', 
        # 'bad_value_ag_d_none', 
        # 'bad_value_ag_d_1_point', 
        # 'bad_value_ag_d_2_points', 
        # 'bad_value_ag_d_3_points_colinear', 
        # 'bad_value_ag_d_3_points_non_colinear', 
        # 'bad_value_ag_d_1_point_1_line_colinear', 
        # 'bad_value_ag_d_1_point_1_line_non_colinear', 
        # 'bad_value_ag_d_2_lines_colinear', 
        # 'bad_value_ag_d_2_lines_non_colinear', 
    ]
    _sgi.sortal_setup()
    ag_empty, ag_square, ag_ells, ag_hash = (
        d['ag_empty'], d['ag_square'], d['ag_ells'], d['ag_hash'])
    pf_square = d['pf_square']
    rules = _make_sortal_rules_for_find_next_appns()
    sr_ells_to_empty = rules[0]
    input_tuples = [
        (sr_ells_to_empty, ag_hash, None), 
        ('ells to empty', ag_hash, None), 
        (sr_ells_to_empty, ag_ells, ag_square), 
        (sr_ells_to_empty, ag_empty, None), 
        (sr_ells_to_empty, ag_square, None), 
        (sr_ells_to_empty, ag_hash, None), 
    ]
    expected_values = [
        [pf_square], 
        [pf_square], 
        [], 
        [], 
        [], 
        [pf_square], 
    ]
    n = len(try_names)
    for i in range(n):
        if i <= 1:
            _sgi.sortal_setup()
        _try_find_rule_appns(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _make_sortal_rules_for_find_next_appns():
    ag_empty, ag_square, ag_ells, ag_hash = (
        d['ag_empty'], d['ag_square'], d['ag_ells'], d['ag_hash'])
    sr_ells_to_empty, name = _sgi.create_rule(
        'ells to empty', '<descr>', ag_ells, ag_empty)
    return [sr_ells_to_empty]

def _try_find_rule_appns(method_name, try_name, input_tuple, expected_value):
    rule, ag_c, ag_d = input_tuple
    received_value = appns = _sgi.find_rule_appns(rule, ag_c, ag_d)
    next_py_forms = []
    for appn in appns:
        t, t_of_a, t_of_b, next_ag = appn
        next_py_form = _pf.PyForm.new_from_agnostic(next_ag)
        next_py_forms.append(next_py_form)
    actual_value = next_py_forms
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_ag_shape_format():
    method_name = 'ag_shape_format'
    try_names = [
        '_', 
        # 'l[]',      # fatal
        # 'l[x]',     # fatal
        'p[]', 
        'p[]l[]', 
        'p[]l[x]', 
        'p[x]', 
        'p[x]l[]', 
        'p[x]l[x]', 
    ]
    _sgi.sortal_setup()
    l_, lx, p_, p_l_, p_lx, px, pxl_, pxlx = (
        _make_ag_shapes_for_ag_shape_format())
    ag_b_list = [
        {}, 
        # l_,   # empty *
        # lx,   # square *
        p_,     # empty
        p_l_,   # empty
        p_lx,   # square
        px,     # points
        pxl_,   # points
        pxlx,   # hash
    ]
    pf_empty, pf_points, pf_square, pf_hash = (
        d['pf_empty'], d['pf_points'], d['pf_square'], d['pf_hash'])
    expected_values = [
        (pf_empty, pf_empty), 
        # (pf_empty, pf_empty),   # index out of range: 1
        # (pf_square, pf_empty),  # index out of range: 1
        (pf_empty, pf_empty), 
        (pf_empty, pf_empty), 
        (pf_square, pf_empty), 
        (pf_points, pf_empty), 
        (pf_points, pf_empty), 
        (pf_hash, pf_empty), 
    ]
    n = len(try_names)
    for i in range(n):
        print('i: %i' % i)
        _try_ag_shape_format(
            method_name, try_names[i], ag_b_list[i], expected_values[i])
    print_test_done(method_name)

def _make_ag_shapes_for_ag_shape_format():
    ag_points, ag_square, ag_hash = (
        d['ag_points'], d['ag_square'], d['ag_hash'])
    points_points = ag_points['points']
    square_lines = ag_square['line segments']
    l_ = {
        'line segments': []}
    lx = {
        'line segments': square_lines}
    p_ = {
        'points': []}
    p_l_ = {
        'points': [], 
        'line segments': [], }
    p_lx = {
        'points': [], 
        'line segments': square_lines, }
    px = {
        'points': points_points}
    pxl_ = {
        'points': points_points, 
        'line segments': [], }
    pxlx = ag_hash
    ag_shapes = [l_, lx, p_, p_l_, p_lx, px, pxl_, pxlx]
    return ag_shapes

def _try_ag_shape_format(method_name, try_name, ag_a_in, expected_value):
    name_in = try_name
    descr = '<descr>'
    ag_b_in = d['ag_empty']
    rule, name_out = _sgi.create_rule(name_in, descr, ag_a_in, ag_b_in)
    ag_a_out = _sgi.getLHS(rule)
    ag_b_out = _sgi.getRHS(rule)
    pf_a = _pf.PyForm.new_from_agnostic(ag_a_out)
    pf_b = _pf.PyForm.new_from_agnostic(ag_b_out)
    actual_value = (pf_a, pf_b)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

# test_getRule()
test_create_rule()
# test_find_rule_appns()
# test_ag_shape_format()


