from controller import controller_main as       _cm
from view import py_form as                     _pf
from view import py_rule as                     _pr
import sortalgi as                              _sgi
import                                          sortal
from tests import test_utilities as             _tu

d = _tu.make_hash_suite_dict()
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message
cm = _cm.ControllerMain()

def test_create_rule():
    method_name = 'create_rule'
    _sgi.sortal_setup()
    ag_empty, ag_square, ag_ells, ag_hash = (
        d['ag_empty'], d['ag_square'], d['ag_ells'], d['ag_hash'])
    _sgi.create_rule(
        'hash_to_empty', 'descr: hash to empty', ag_hash, ag_empty)
    try_names = [
        'bad_type_name', 
        'bad_type_descr', 
        'bad_type_ag_left', 
        'bad_type_ag_right', 
        'bad_value_name__in_use', 
        'bad_value_ag_left__empty', 
        'square_to_ells', 
    ]
    input_tuples = [
        (37, 'descr: 37 to ells', ag_square, ag_ells), 
        ('square_to_37', 37, ag_square, ag_ells), 
        ('37_to_ells', 'descr: 37 to ells', 37, ag_ells), 
        ('square_to_37', 'descr: square to 37', ag_square, 37), 
        ('hash_to_empty', 'descr: square to ells', ag_square, ag_ells), 
        ('empty_to_ells', 'descr: empty to ells', ag_empty, ag_ells), 
        ('square_to_ells', 'descr: square to ells', ag_square, ag_ells), 
    ]
    expected_values = [
        None,   # (sortal.grammar.rules.rule.rule, 37)
        None,   # (sortal.grammar.rules.rule.rule, 'square_to_37')
        None,   # len() of unsized object
        None,   # len() of unsized object
        None,   # message; sortal.grammar.rules.rule.rule
        None,   # (sortal.grammar.rules.rule.rule, 'empty_to_ells')
        (sortal.grammar.rules.rule.rule, 'square_to_ells'), 
    ]
    n = len(try_names)
    for i in range(n):                          ##  (2, 4) breaks
        _try_create_rule(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_create_rule(method_name, try_name, tuple_, expected_value):
    print('try_name: %s' % try_name)
    rule_name, rule_descr, ag_left, ag_right = tuple_
    return_value = _sgi.create_rule(rule_name, rule_descr, ag_left, ag_right)
    if return_value == None:
        actual_value = return_value
    else:
        sgi_rule, sgi_rule_name = return_value
    actual_value = (type(sgi_rule), sgi_rule_name)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)


def test_create_rule_0():
    method_name = 'create_rule_0'
    try_names = [
        'point_to_empty', 
        'square_to_empty', 
        'square_to_square', 
        'square_to_ells', 
        'square_to_hash', 
        'ells_to_empty', 
        'ells_to_square', 
        'ells_to_ells', 
        'ells_to_hash', 
        'hash_to_empty', 
        'hash_to_square', 
        'hash_to_ells', 
        'hash_to_hash', 
    ]
    d = _tu.make_hash_suite_ag_dict()
    input_tuples = [
        # (name, descr, left_ag, right_ag), 
        ('delete_point', 'deletes_point', ag_point, ag_empty), 
        ('square_to_empty', 'deletes square', ag_square, ag_empty), 
        ('square_to_ells', 'changes square to ells', ag_square, ag_ells), 
        ('square_to_square', 'leaves square unchanged', ag_square, ag_square), 
        ('square_to_hash', 'changes square to hash', ag_square, ag_hash), 
        ('ells_to_empty', 'deletes ells', ag_ells, ag_empty), 
        ('ells_to_square', 'changes ells to square', ag_ells, ag_square), 
        ('ells_to_ells', 'leaves ells unchanged', ag_ells, ag_ells), 
        ('ells_to_hash', 'changes ells to hash', ag_ells, ag_hash), 
        ('hash_to_empty', 'deletes hash', ag_hash, ag_empty), 
        ('hash_to_square', 'changes hash to square', ag_hash, ag_square), 
        ('hash_to_hash', 'leaves hash unchanged', ag_hash, ag_hash), 
        # (37, rule_descr, ag_ells, ag_hash), 
        # (rule_name, 37, ag_ells, ag_hash), 
        # (rule_name, rule_descr, 37, ag_hash), 
        # (rule_name, rule_descr, [], ag_hash), 
        # (rule_name, rule_descr, )
    ]
    expected_values = [
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        sortal.grammar.rules.rule.rule, 
        # 'delete_point', 
    ]
    n = len(try_names)
    for i in range(1, 2):
        _try_create_rule_0(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_create_rule_0(method_name, try_name, input_tuple, expected_value):
    name, descr, ag_left, ag_right = input_tuple
    sgi_rule, sgi_rule_name = _sgi.create_rule(
        name, descr, ag_left, ag_right)
    actual_value = type(sgi_rule)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_apply_rule_bad():                      ##  2017-11-15 09:33
    method_name = 'apply_rule_bad'
    try_names = [
        'bad_type_form', 
        'bad_type_subform', 
        'bad_type_rule', 
        'bad_value_empty_form', 
        'bad_value_empty_subform', 
        'bad_value_empty_left', 
        'hash_hash_ells_to_empty', 
    ]
    py_empty, py_square, py_ells, py_hash = _tu.make_hash_suite_py_forms()
    py_ells_to_empty = _pr.PyRule(py_ells, py_empty)
    py_empty_to_ells = _pr.PyRule(py_empty, py_ells)
    input_triples = [
        (37, py_hash, py_ells_to_empty),        # 'bad_type_form', 
        (py_hash, 37, py_ells_to_empty),        # 'bad_type_subform', 
        (py_hash, py_hash, 37),                 # 'bad_type_rule', 
        (py_empty, py_hash, py_ells_to_empty),  # 'bad_value_empty_form', 
        (py_hash, py_empty, py_ells_to_empty),  # 'bad_value_empty_subform', 
        (py_hash, py_hash, py_empty_to_ells),   # 'bad_value_empty_left', 
        (py_hash, py_hash, py_ells_to_empty),   # 'hash_hash_ells_to_empty', 
    ]
    expected_values = [
        None,                                   # 'bad_type_form', 
        None,                                   # 'bad_type_subform', 
        None,                                   # 'bad_type_rule', 
        None,                                   # Subshape is not part of main shape
        None,                                   # PyRule: no empty left form
        None,                                   # PyRule: no attrib 'name'
        [py_square],                            # 'hash_hash_ells_to_empty', 
    ]
    n = len(try_names)
    for i in range(6, 7):
        _try_apply_rule(
            method_name, try_names[i], input_triples[i], expected_values[i])
    print_test_done(method_name)

def _try_apply_rule(method_name, try_name, triple, expected_value):
    py_form, sub_py_form, py_rule = triple
    actual_value = cm.apply_rule(py_form, sub_py_form, py_rule)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__extract_ag_shapes_from_appns():
    method_name = 'extract_ag_shapes_from_appns'
    try_names = [
        'hash_minus_ells_appns', 
    ]
    hash_minus_ells_appns = _make_hash_minus_ells_appns()
    appn_lists = [
        hash_minus_ells_appns, 
    ]
    ag_empty, ag_square, ag_ells, ag_hash = _tu.make_hash_suite_ags()
    expected_ag_shape_lists = [
        [ag_square], 
    ]
    n = len(try_names)
    for i in range(n):
        _try__extract_ag_shapes_from_appns(
            method_name, 
            try_names[i], 
            appn_lists[i], 
            expected_ag_shape_lists[i])
    print_test_done(method_name)

def _try__extract_ag_shapes_from_appns(
    method_name, try_name, appns, expected_ag_shapes
):
    actual_ag_shapes = cm._extract_ag_shapes_from_appns(appns)
    actual_py_forms = _pf.PyForm.some_new_from_agnostics(actual_ag_shapes)
    actual_value = sorted(actual_py_forms)
    expected_py_forms = _pf.PyForm.some_new_from_agnostics(expected_ag_shapes)
    expected_value = sorted(expected_py_forms)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_get_rule():
    method_name = 'get_rule'
    try_names = [
        'bad_type', 
        'bad_value_no_rule', 
        'good_value', 
    ]
    existing_rule = _set_up_for_get_rule()
    names_in = [
        37, 
        'delete_empty', 
        'delete_hash', 
    ]
    expected_values = [
        None, 
        None, 
        existing_rule, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_get_rule(
            method_name, try_names[i], names_in[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_get_rule():
    _cm.ControllerMain.reset_sortal()
    name = 'delete_hash'
    descr = 'deletes hash'
    d = _tu.make_hash_suite_dict()
    left_ag = d['ag_hash']
    right_ag = d['ag_empty']
    rule_out, name_out = _sgi.create_rule(name, descr, left_ag, right_ag)
    return rule_out

def _try_get_rule(method_name, try_name, name, expected_value):
    actual_value = _sgi.getRule(name)
    if actual_value == expected_value:
    # if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)


def test_find_rule_appns_0():                   ##  2017-11-14 10:59
    method_name = 'find_rule_appns_0'
    try_names = [
        'hash_None_ells_empty', 
    ]
    ag_empty, ag_square, ag_ells, ag_hash = _tu.make_hash_suite_ags()
    rd = _tu.make_hash_suite_sgi_rule_dict()
    sr_ells_empty, name_ells_empty = _sgi.create_rule(
        'delete ells', 'deletes ells', ag_ells, ag_empty)
    tuples = [
        (ag_hash, None, sr_ells_empty), 
    ]
    expected_values = [
        _pf.PyForm.new_from_agnostic(ag_square), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_find_rule_appns_0(
            method_name, try_names[i], tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_find_rule_appns_0(method_name, try_name, tuple_, expected_value):
    ag_shape, ag_subshape, rule_inst = tuple_
    return_tuples = _sgi.find_rule_appns(rule_inst, ag_shape)
    return_tuple = return_tuples[0]
    t, t_of_a_in_d, t_of_b, ag_next = return_tuple
    actual_value = _pf.PyForm.new_from_agnostic(ag_next)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_find_rule_appns():
    method_name = 'find_rule_appns'
    try_names = [
        'square__________square_to_empty', 
        'square__________square_to_square', 
        'square__________square_to_ells', 
        'square__________square_to_hash', 
        'square__empty___square_to_empty', 
        'square__empty___square_to_square', 
        'square__empty___square_to_ells', 
        'square__empty___square_to_hash', 
        'square__square__square_to_empty', 
        'square__square__square_to_square', 
        'square__square__square_to_ells', 
        'square__square__square_to_hash', 
        'square__________ells_to_empty', 
        'square__________ells_to_square', 
        'square__________ells_to_ells', 
        'square__________ells_to_hash', 
        'square__empty___ells_to_empty', 
        'square__empty___ells_to_square', 
        'square__empty___ells_to_ells', 
        'square__empty___ells_to_hash', 
    ]
    input_tuples = [
        # (ag_shape, ag_subshape, sortal_rule), 
        (ag_hash,           ag_square_to_empty), 
        (ag_hash, ag_empty, ag_square_to_empty), 
        (ag_hash, ag_hash,  ag_square_to_empty), 
        (ag_hash,           ag_square_to_ells), 
        (ag_hash, ag_empty, ag_square_to_ells), 
        (ag_hash, ag_hash,  ag_square_to_ells), 
        (ag_ells,           ag_square_to_ells), 
        (ag_ells, ag_empty, ag_square_to_ells), 
        (ag_ells, ag_ells,  ag_square_to_ells), 
    ]
    expected_values = [
        # (xform, ag_t_of_a, ag_c), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_find_rule_appns(
            method_name, try_names[i], input_tuples[i], expected_values[i])

    ag_empty, ag_square, ag_ells, ag_hash = _tu.make_hash_suite_ags()
    ag_shapes = [
        ag_empty, 
        ag_square, 
        ag_ells, 
        ag_hash, 
    ]
    ag_shape_names = [
        'ag_empty', 
        'ag_square', 
        'ag_ells', 
        'ag_hash', 
    ]
    ag_subshapes = [
        None, 
        ag_square, 
        ag_ells, 
        ag_hash, 
    ]
    ag_subshape_names = [
        'None', 
        'ag_square', 
        'ag_ells', 
        'ag_hash', 
    ]
    n_ag_shapes = len(ag_shapes)
    n_ag_subshapes = len(ag_subshapes)
    ag_shape_subshape_pairs = []
    ag_shape_subshape_pair_names = []
    for i in range(n_ag_shapes):
        for j in range(n_ag_subshapes):
            pair = (ag_shapes[i], ag_shapes[j])
            ag_shape_subshape_pairs.append(pair)
            pair_name = '%s_%s' % (
                ag_shape_names[i], ag_shape_subshape_names[j])
    rule_insts = []
    for i in range(1, n_agshapes):
        for j in range(n_agshapes):
            rule_inst = _sgi.create_rule(
                rule_name, rule_descr, ag_left, ag_right)

    for i in range(n_ag_shape_subshape_pairs):
        for j in range(n_rule_insts):
            _try_find_rule_appns(
                method_name, try_name, input_tuple, expected_value)

    print_test_done(method_name)

def _try_find_rule_appns(method_name, try_name, input_tuple, expected_value):
    ag_shape, ag_subshape, rule_inst = input_tuple
    actual_value = _sgi.find_rule_appns(rule_inst, ag_shape, ag_subshape)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_find_rule_appns_many():     ##  2017-11-18 07:45
    method_name = 'find_rule_appns_many'
    try_names = [
        'try_hash_minus_L11', 
        'try_hash_minus_L12', 
        'try_hash_minus_L21', 
        'try_hash_minus_L22', 
    ]
    d = _tu.make_hash_suite_dict()
    tuples = [
        (d['sr_L11_to_0'], d['ag_hash'], d['ag_hash']), 
        (d['sr_L12_to_0'], d['ag_hash'], d['ag_hash']), 
        (d['sr_L21_to_0'], d['ag_hash'], d['ag_hash']), 
        (d['sr_L22_to_0'], d['ag_hash'], d['ag_hash']), 
    ]
    expected_py_forms = [
        d['pf_hash_minus_L11'], 
        d['pf_hash_minus_L12'], 
        d['pf_hash_minus_L21'], 
        d['pf_hash_minus_L22']]
    n = len(try_names)
    for i in range(n):
        _try_find_rule_appns_square_and_ells(
            method_name, try_names[i], tuples[i], sorted(expected_py_forms))
    print_test_done(method_name)
"""
find_rule_appns_many: try_hash_minus_L11:
    expected [
        _pf.PyForm(
            [   _pp.PyPoint(0, 80, 0, ['a']), 
                _pp.PyPoint(20, 100, 0, ['a']), 
                _pp.PyPoint(80, 0, 0, ['a']), 
                _pp.PyPoint(80, 100, 0, ['a']), 
                _pp.PyPoint(100, 20, 0, ['a']), 
                _pp.PyPoint(100, 80, 0, ['a'])], 
            [   _pl.PyLine((0, 80, 0), (100, 80, 0), [], 0.9), 
                _pl.PyLine((20, 20, 0), (20, 100, 0), [], 0.9), 
                _pl.PyLine((20, 20, 0), (100, 20, 0), [], 0.9), 
                _pl.PyLine((80, 0, 0), (80, 100, 0), [], 0.9)]), 
        _pf.PyForm(
            [   _pp.PyPoint(0, 0, 0, ['a']), 
                _pp.PyPoint(20, 0, 0, ['a']), 
                _pp.PyPoint(80, 0, 0, ['a']), 
                _pp.PyPoint(80, 100, 0, ['a']), 
                _pp.PyPoint(100, 20, 0, ['a']), 
                _pp.PyPoint(100, 80, 0, ['a'])], 
            [   _pl.PyLine((0, 20, 0), (100, 20, 0), [], 0.9), 
                _pl.PyLine((20, 0, 0), (20, 80, 0), [], 0.9), 
                _pl.PyLine((20, 80, 0), (100, 80, 0), [], 0.9), 
                _pl.PyLine((80, 0, 0), (80, 100, 0), [], 0.9)]), 
        _pf.PyForm(
            [   _pp.PyPoint(0, 0, 0, ['a']), 
                _pp.PyPoint(0, 80, 0, ['a']), 
                _pp.PyPoint(20, 0, 0, ['a']), 
                _pp.PyPoint(20, 100, 0, ['a']), 
                _pp.PyPoint(80, 100, 0, ['a']), 
                _pp.PyPoint(100, 80, 0, ['a'])], 
            [   _pl.PyLine((0, 20, 0), (100, 80, 0), [], 0.9), 
                _pl.PyLine((0, 80, 0), (100, 80, 0), [], 0.9), 
                _pl.PyLine((20, 0, 0), (20, 100, 0), [], 0.9), 
                _pl.PyLine((80, 20, 0), (80, 100, 0), [], 0.9)]), 
        _pf.PyForm(
            [   _pp.PyPoint(0, 0, 0, ['a']), 
                _pp.PyPoint(0, 80, 0, ['a']), 
                _pp.PyPoint(20, 0, 0, ['a']), 
                _pp.PyPoint(20, 100, 0, ['a']), 
                _pp.PyPoint(80, 0, 0, ['a']), 
                _pp.PyPoint(100, 20, 0, ['a'])], 
            [   _pl.PyLine((0, 20, 0), (100, 20, 0), [], 0.9), 
                _pl.PyLine((0, 80, 0), (80, 80, 0), [], 0.9), 
                _pl.PyLine((20, 0, 0), (20, 100, 0), [], 0.9), 
                _pl.PyLine((80, 0, 0), (80, 80, 0), [], 0.9)])]
    got [
        _pf.PyForm(
            [   _pp.PyPoint(0.0, 80.0, 0.0, ['a']), 
                _pp.PyPoint(20.0, 100.0, 0.0, ['a']), 
                _pp.PyPoint(80.0, 0.0, 0.0, ['a']), 
                _pp.PyPoint(80.0, 100.0, 0.0, ['a']), 
                _pp.PyPoint(100.0, 20.0, 0.0, ['a']), 
                _pp.PyPoint(100.0, 80.0, 0.0, ['a'])], 
            [   _pl.PyLine((0.0, 80.0, 0.0), (100.0, 80.0, 0.0), [], 0.9), 
                _pl.PyLine((20.0, 20.0, 0.0), (20.0, 100.0, 0.0), [], 0.9), 
                _pl.PyLine((20.0, 20.0, 0.0), (100.0, 20.0, 0.0), [], 0.9), 
                _pl.PyLine((80.0, 0.0, 0.0), (80.0, 100.0, 0.0), [], 0.9)]), 
        _pf.PyForm(
            [   _pp.PyPoint(0.0, 80.0, 0.0, ['a']), 
                _pp.PyPoint(20.0, 0.0, 0.0, ['a']), 
                _pp.PyPoint(20.0, 100.0, 0.0, ['a']), 
                _pp.PyPoint(80.0, 100.0, 0.0, ['a']), 
                _pp.PyPoint(100.0, 20.0, 0.0, ['a']), 
                _pp.PyPoint(100.0, 80.0, 0.0, ['a'])], 
            [   _pl.PyLine((0.0, 20.0, 0.0), (100.0, 20.0, 0.0), [], 0.9), 
                _pl.PyLine((20.0, 0.0, 0.0), (20.0, 100.0, 0.0), [], 0.9), 
                _pl.PyLine((80.0, 80.0, 0.0), (80.0, 100.0, 0.0), [], 0.9), 
                _pl.PyLine((80.0, 80.0, 0.0), (100.0, 80.0, 0.0), [], 0.9)]), 
        _pf.PyForm(
            [   _pp.PyPoint(20.0, 0.0, 0.0, ['a']), 
                _pp.PyPoint(20.0, 100.0, 0.0, ['a']), 
                _pp.PyPoint(80.0, 0.0, 0.0, ['a']), 
                _pp.PyPoint(80.0, 100.0, 0.0, ['a']), 
                _pp.PyPoint(100.0, 20.0, 0.0, ['a']), 
                _pp.PyPoint(100.0, 80.0, 0.0, ['a'])], 
            [   _pl.PyLine((0.0, 20.0, 0.0), (100.0, 20.0, 0.0), [], 0.9), 
                _pl.PyLine((20.0, 0.0, 0.0), (20.0, 100.0, 0.0), [], 0.9), 
                _pl.PyLine((80.000000000000014, 80.0, 0.0), (80.0, 100.0, 0.0), [], 0.9), 
                _pl.PyLine((80.000000000000014, 80.0, 0.0), (100.0, 80.0, 0.0), [], 0.9)])]
"""
def _try_find_rule_appns_square_and_ells(
    method_name, try_name, tuple_, expected_value
):
    sgi_rule, ag_shape, sub_ag_shape = tuple_
    actual_appns = _sgi.find_rule_appns(sgi_rule, ag_shape, sub_ag_shape)
    actual_ag_shapes = cm._extract_ag_shapes_from_appns(actual_appns)
    actual_py_forms = _pf.PyForm.some_new_from_agnostics(actual_ag_shapes)
    actual_value = sorted(actual_py_forms)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, str(expected_value), str(actual_value))

def _make_hash_minus_ells_appns():
    applications = [
        (   [[0, 0, 0, ], [0, 0, 0, ], [0, 0, 0, ]], 
            {   'line segments': [
                    (((20.0, 0.0, 0.0), (20.0, 20.0, 0.0)), {'weight': 1.7}), 
                    (((20.0, 80.0, 0.0), (20.0, 100.0, 0.0)), {'weight': 1.7}), 
                    (((80.0, 0.0, 0.0), (80.0, 20.0, 0.0)), {'weight': 1.7}), 
                    (((80.0, 80.0, 0.0), (80.0, 100.0, 0.0)), {'weight': 1.7}), 
                    (((0.0, 20.0, 0.0), (20.0, 20.0, 0.0)), {'weight': 1.7}), 
                    (((80.0, 20.0, 0.0), (100.0, 20.0, 0.0)), {'weight': 1.7}), 
                    (((0.0, 80.0, 0.0), (20.0, 80.0, 0.0)), {'weight': 1.7}), 
                    (((80.0, 80.0, 0.0), (100.0, 80.0, 0.0)), {'weight': 1.7})], 
                'points': [
                    ((0.0, 20.0, 0.0), {'labels': ['a']}), 
                    ((0.0, 80.0, 0.0), {'labels': ['a']}), 
                    ((20.0, 0.0, 0.0), {'labels': ['a']}), 
                    ((20.0, 100.0, 0.0), {'labels': ['a']}), 
                    ((80.0, 0.0, 0.0), {'labels': ['a']}), 
                    ((80.0, 100.0, 0.0), {'labels': ['a']}), 
                    ((100.0, 20.0, 0.0), {'labels': ['a']}), 
                    ((100.0, 80.0, 0.0), {'labels': ['a']})]}, 
            {}, 
            {   'line segments': [
                    (((20.0, 20.0, 0.0), (20.0, 80.0, 0.0)), {'weight': 1.7}), 
                    (((80.0, 20.0, 0.0), (80.0, 80.0, 0.0)), {'weight': 1.7}), 
                    (((20.0, 20.0, 0.0), (80.0, 20.0, 0.0)), {'weight': 1.7}), 
                    (((20.0, 80.0, 0.0), (80.0, 80.0, 0.0)), {'weight': 1.7})]})]
    return applications

test_create_rule()

# test_get_rule()

# test_import()
# test_create_rule_0()
# test_apply_rule_bad()
# test__extract_ag_shapes_from_appns()
# test_find_rule_appns_0()                        ##  successful!
# test_find_rule_appns()

# test_find_rule_appns_many()                     ##  runs but with bad results

