from System.Drawing import              Color
# import py_line as                       _pl
# import py_point as                      _pp
import rhinoscriptsyntax as             _rs

class Settings(object):
    dark_gray = Color.FromArgb(105, 105, 105)
    frame_side = 32
    frame_size = (frame_side, frame_side, frame_side)
    font_style_normal = 0
    justified_center = 2
    blue        = Color.FromArgb(  0,   0, 255)
    green       = Color.FromArgb(  0, 255,   0)
    green_blue  = Color.FromArgb(  0, 255, 255)
    red         = Color.FromArgb(255,   0,   0)
    red_blue    = Color.FromArgb(255,   0, 255)
    red_green   = Color.FromArgb(255, 255,   0)

    annotation_type = 512
    arithmetic_frame_a_offset_factor = (0, 0, 0)
    arithmetic_frame_b_offset_factor = (1.5, 0, 0)
    arithmetic_frame_sum_ab_offset_factor = (3, 0, 0)
    arithmetic_frame_diff_ab_offset_factor = (4.5, 0, 0)
    arithmetic_frame_diff_ba_offset_factor = (6, 0, 0)
    arithmetic_tag_offset_factor = (0.5, -0.4, 0)
    arithmetic_tag_text_height = 5
    arithmetic_tag_text_a = 'a'
    arithmetic_tag_text_b = 'b'
    arithmetic_tag_text_sum_ab = 'a + b'
    arithmetic_tag_text_diff_ab = 'a - b'
    arithmetic_tag_text_diff_ba = 'b - a'
    arrow_base_point = (0, 0, 0)
    arrow_color_name = dark_gray
    arrow_label_offset_from_arrow = (0, 4, 0)   ##  rethink
    arrow_length = frame_size[0] / 4
    arrow_name = 'arrow'    
    block_instance_filter = 4096    
    block_type = 4096
    color_left = red_green
    color_left_name = 'red_green'
    color_right = red
    color_right_name = 'red'
    color_current_shape = color_current = green
    color_current_shape_name = color_current_name = 'green'
    color_current_subshape = green_blue
    color_current_subshape_name = 'green_blue'
    curve_filter = 4
    curve_type = 4
    default_layer_name = 'Default'
    default_line_weight = 1.0
    double_arrow_base_point = (0, 0, 0)
    double_arrow_color_name = dark_gray
    double_arrow_name = 'double_arrow'
    double_arrow_length = frame_size[0]/4
    first_initial_shape_layer_name = 'initial_shape_1'
    first_initial_shape_frame_position = (0, -40, 0)
    first_rule_layer_name = 'rule_1'
    first_rule_left_frame_position = (60, -40, 0)
    frame_base_point = (0, 0, 0)                ##  Point3D
    frame_color_name = dark_gray
    frame_layer_name = 'frames'
    frame_name = 'frame block'
    frame_padding = 10
    frame_padding_factor = 0.2
    frame_pad_minimum = 10
    group_name_suffix_left = 'a'
    group_name_suffix_right = 'b'
    group_name_separator = '_'
    n_round_digits = 14
    # origin = [0, 0, 0]
    # origin = _rs.AddPoint(0, 0, 0)
    origin_object = _rs.AddPoint(0, 0, 0)
    origin = _rs.PointCoordinates(origin_object)
    origin_p3d = _rs.PointAdd(origin, origin)
    point_type = 1
    right_frame_offset_factor = 1.5
    rule_arrow_offset_factor_x = -0.5   # from right frame insert point
    rule_arrow_offset_factor_y = -0.5   # from right frame insert point
    rule_gap_factor = 2
    rule_left_shape_offset_factor = -2  # from right frame insert point
    rule_margin = (-100, 100, 0)        # from right frame lower right corner
    rule_display_point = rule_margin
    rule_name_text_height = 2
    rule_name_text_font = 'Arial'
    rule_name_text_font_style = font_style_normal
    rule_name_text_justification = justified_center
    rule_origin_offset_factor = (-1, 0, 0)
    text_dot_filter = 8192
    text_dot_type = 8192

    derivation_array_position = (0, 0, 0)
    derivation_cell_offset_factor_x = 1.5
    derivation_cell_offset_factor_y = 2
    derivation_layer_name = 'derivation'
    derivation_n_columns = 5
    derivation_labeled_shape_offset = (0, 0, 0)
    derivation_arrow_offset = (
        (   (   (   derivation_cell_offset_factor_x - 1) / 
                2) + 
            1) * 
            frame_size[0], 
        frame_size[1] * 0.5, 
        0)
    derivation_rule_name_offset = (
        (   (   (   derivation_cell_offset_factor_x - 1) / 
                2) + 
            1) * 
            frame_size[0], 
        frame_size[1] * 0.75, 
        0)

    def __init__(self):
        pass

    @classmethod
    def get_right_frame_position(cls, left_frame_position):
        """Receives:
            left_frame_position
                            Point3d or (num, num, num). The origin of the left 
                            frame block
        Returns:
            right_frame_position
                            Point3d. The origin of the right frame block
        """
        frame_side_x = cls.frame_size[0]
        right_frame_offset_x = (
            frame_side_x * cls.right_frame_offset_factor)
        right_frame_offset = (right_frame_offset_x, 0, 0)
        right_frame_position = _rs.PointAdd(
            left_frame_position, right_frame_offset)
        return right_frame_position

    @classmethod
    def get_arrow_position(cls, left_frame_position):       ##  rule arrow
        """Receives:
            left_frame_position
                            Point3d or (num, num, num). The position of the 
                            left frame instance
        Returns:
            arrow_position  Point3d. The center point of the arrow
        """
        frame_gap_factor_x = (cls.right_frame_offset_factor - 1) / 2
        arrow_offset_x = cls.frame_size[0] * (frame_gap_factor_x + 1)
        arrow_offset_y = cls.frame_size[1] / 2
        arrow_offset_z = 0
        arrow_offset = (arrow_offset_x, arrow_offset_y, arrow_offset_z)
        arrow_position = _rs.PointAdd(left_frame_position, arrow_offset)
        return arrow_position

    @classmethod
    def get_derivation_cell_position_triples(cls, n_shapes):##  09-21 17:52
        """Receives:
            n_shapes        int >= 2. The number of labeled shapes in the 
                            derivation
        Calculates the positions of labeled shapes, arrows, and rule names 
        arranged in an array. Returns:
            derivation_cell_position_triples
                            [(triple)]. A list of n triples, n = n_shapes, 
                            each of the form:
                                (   labeled_shape_position,
                                    arrow_position, 
                                    rule_name_position)
                            where each position is a triple of the form: 
                                (num, num, num)
        """
        labeled_shape_offset = cls.derivation_labeled_shape_offset
        arrow_offset = cls.derivation_arrow_offset
        rule_name_offset = cls.derivation_rule_name_offset
        cell_positions = (cls._get_derivation_array_cell_positions(n_shapes))
        derivation_cell_position_triples = []
        for i in range(n_shapes):
            derivation_array_cell_position = (
                cell_positions[i])
            labeled_shape_position = _rs.PointAdd(
                derivation_array_cell_position,
                labeled_shape_offset)
            arrow_position = _rs.PointAdd(
                derivation_array_cell_position,
                arrow_offset)
            rule_name_position = _rs.PointAdd(
                derivation_array_cell_position, 
                rule_name_offset)
            position_triple = (
                labeled_shape_position, 
                arrow_position, 
                rule_name_position)
            derivation_cell_position_triples.append(position_triple)
        return derivation_cell_position_triples

    @classmethod
    def _get_derivation_array_cell_positions(cls, n_shapes):
        """Receives:
            n_shapes        int >= 2
        Calculates the position of each cell in the derivation array. Returns:
            derivation_array_cell_positions
                            [(num, num, num)]. A list of n triples of numbers,
                            n = n_shapes
        """
        derivation_array_cell_positions = []
        cell_positions_x = cls._get_derivation_cell_positions_x(n_shapes)
        cell_positions_y = cls._get_derivation_cell_positions_y(n_shapes)
        for i in range(n_shapes):
            x = cell_positions_x[i]
            y = cell_positions_y[i]
            z = 0
            position = (x, y, z)
            derivation_array_cell_positions.append(position)
        return derivation_array_cell_positions

    @classmethod
    def _get_derivation_cell_positions_x(cls, n_shapes):
        """Receives:
            n_shapes        int >= 2. The number of labeled_shapes
        Returns:
            cell_positions_x
                            [num]. A list of the x-coordinates of the 
                            derivation cell positions
        """
        cell_positions_x = []
        n_columns = cls.derivation_n_columns
        frame_width_x = cls.frame_size[0]
        cell_offset_factor_x = cls.derivation_cell_offset_factor_x
        cell_offset_x = frame_width_x * cell_offset_factor_x
        for i in range(n_shapes):
            column_index = i % n_columns
            xi = cell_offset_x * column_index
            cell_positions_x.append(xi)
        return cell_positions_x

    @classmethod
    def _get_derivation_cell_positions_y(cls, n_shapes):
        """Receives:
            n_shapes        int >= 2. The number of labeled shapes
        Returns:
            cell_positions_y
                            [num]. A list of the y-coordinates of the 
                            derivation cell positions
        """
        n_columns = cls.derivation_n_columns
        n_full_rows = n_shapes // n_columns
        n_remaining_shapes = n_shapes % n_columns
        if n_remaining_shapes == 0:
            n_remaining_rows = 0
        else:
            n_remaining_rows = 1
        n_rows = n_full_rows + n_remaining_rows
        first_row_index = n_rows - 1
        n_columns = cls.derivation_n_columns
        frame_height_y = cls.frame_size[1]
        cell_offset_factor_y = cls.derivation_cell_offset_factor_y
        cell_offset_y = frame_height_y * cell_offset_factor_y
        cell_positions_y = []
        for i in range(n_shapes):
            row_index = first_row_index - (i // n_columns)
            yi = cell_offset_y * row_index
            cell_positions_y.append(yi)
        return cell_positions_y

    @classmethod
    def get_arithmetic_frame_position(cls, frame_name, local_origin):
        """Receive:
            frame_name      str. 'a' | 'b' | 'a+b' | 'a-b' | 'b-a'
            local_origin    Point3d
        Return:
            frame_position  Point3d
            tag_position  Point3d

        """
        frame_offset_factors = {
            'a': cls.arithmetic_frame_a_offset_factor, 
            'b': cls.arithmetic_frame_b_offset_factor, 
            'a+b': cls.arithmetic_frame_sum_ab_offset_factor, 
            'a-b': cls.arithmetic_frame_diff_ab_offset_factor, 
            'b-a': cls.arithmetic_frame_diff_ba_offset_factor}
        frame_offset_factor = frame_offset_factors[frame_name]
        frame_offset = _rs.PointScale(frame_offset_factor, cls.frame_side)
        frame_position = _rs.PointAdd(local_origin, frame_offset)
        tag_offset = _rs.PointScale(
            cls.arithmetic_tag_offset_factor, cls.frame_side)
        tag_position = _rs.PointAdd(frame_position, tag_offset)
        return (frame_position, tag_position)

    # @classmethod                              ##  not called?
    # def make_hash_suite_ags(cls):               ##  2017-10-21 10:01
    #     """Returns:
    #         empty_ag        {}
    #         square_ag       {'line segments': line_ags}
    #         ells_ag         {'points': point_ags, 'line segments': line_ags}
    #         hash_ag         {'points': point_ags, 'line segments': line_ags}
    #     """
    #     xyz02 = (  0,  20, 0)
    #     xyz08 = (  0,  80, 0)
    #     xyz20 = ( 20,   0, 0)
    #     xyz22 = ( 20,  20, 0)
    #     xyz28 = ( 20,  80, 0)
    #     xyz2t = ( 20, 100, 0)
    #     xyz80 = ( 80,   0, 0)
    #     xyz82 = ( 80,  20, 0)
    #     xyz88 = ( 80,  80, 0)
    #     xyz8t = ( 80, 100, 0)
    #     xyzt2 = (100,  20, 0)
    #     xyzt8 = (100,  80, 0)

    #     pag_02a = (xyz02, {'labels': ['a']})
    #     pag_08a = (xyz08, {'labels': ['a']})
    #     pag_20a = (xyz20, {'labels': ['a']})
    #     pag_2ta = (xyz2t, {'labels': ['a']})
    #     pag_80a = (xyz80, {'labels': ['a']})
    #     pag_8ta = (xyz8t, {'labels': ['a']})
    #     pag_t2a = (xyzt2, {'labels': ['a']})
    #     pag_t8a = (xyzt8, {'labels': ['a']})

    #     lag_0222 = ((xyz02, xyz22), {})
    #     lag_02t2 = ((xyz02, xyzt2), {})
    #     lag_0828 = ((xyz08, xyz28), {})
    #     lag_08t8 = ((xyz08, xyzt8), {})
    #     lag_2022 = ((xyz20, xyz22), {})
    #     lag_202t = ((xyz20, xyz2t), {})
    #     lag_2228 = ((xyz22, xyz28), {})
    #     lag_2282 = ((xyz22, xyz82), {})
    #     lag_282t = ((xyz28, xyz2t), {})
    #     lag_2888 = ((xyz28, xyz88), {})
    #     lag_8082 = ((xyz80, xyz82), {})
    #     lag_808t = ((xyz80, xyz8t), {})
    #     lag_8288 = ((xyz82, xyz88), {})
    #     lag_82t2 = ((xyz82, xyzt2), {})
    #     lag_888t = ((xyz88, xyz8t), {})
    #     lag_88t8 = ((xyz88, xyzt8), {})

    #     empty_ag = {}
    #     square_ag = {
    #         'line segments': [
    #             lag_2228, lag_2282, lag_2888, lag_8288, 
    #         ]
    #     }
    #     ells_ag = {
    #         'points': [
    #             pag_02a, pag_08a, pag_20a, pag_2ta, 
    #             pag_80a, pag_8ta, pag_t2a, pag_t8a, 
    #         ], 
    #         'line segments': [
    #             lag_0222, lag_0828, lag_2022, lag_282t, 
    #             lag_8082, lag_888t, lag_82t2, lag_88t8, 
    #         ], 
    #     }
    #     hash_ag = {
    #         'points': [
    #             pag_02a, pag_08a, pag_20a, pag_2ta, 
    #             pag_80a, pag_8ta, pag_t2a, pag_t8a, 
    #         ], 
    #         'line segments': [
    #             lag_02t2, lag_08t8, lag_202t, lag_808t, 
    #         ], 
    #     }
    #     return (empty_ag, square_ag, ells_ag, hash_ag)

