from view import agnostic as                    _a
from view.tests import test_utilities as        _tu

print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message

def test_new_ag_rule():
    method_name = 'new_ag_rule'
    try_names = [
        'hash_empty_name_descr',
    ]
    name = 'delete_hash'
    descr = 'deletes hash'
    ag_empty, ag_square, ag_ells, ag_hash = _tu.make_hash_suite_ags()
    input_tuples = [
        (ag_hash, ag_empty, name, descr), 
    ]
    expected_values = [
        (   (ag_hash, ag_empty), 
            {   'name': name, 
                'description': descr})
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_ag_rule(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_new_ag_rule(method_name, try_name, input_tuple, expected_value):
    ag_left, ag_right, name, descr = input_tuple
    actual_value = _a.Agnostic.new_ag_rule(ag_left, ag_right, name, descr)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_ag_shapes():
    method_name = 'new_ag_shapes'
    try_names = [
        'empty_list', 
        'hash', 
        'ells', 
        'ells_hash', 
        'square', 
        'square_hash', 
        'square_ells', 
        'square_ells_hash', 
    ]
    hash_point_specs, square_line_specs, ell_line_specs, hash_line_specs = (
        _tu.make_hash_suite_specs())
    square_spec = ([], square_line_specs)
    ells_spec = (hash_point_specs, ell_line_specs)
    hash_spec = (hash_point_specs, hash_line_specs)
    shape_spec_lists = [
        [], 
        [hash_spec], 
        [ells_spec], 
        [ells_spec, hash_spec], 
        [square_spec], 
        [square_spec, hash_spec], 
        [square_spec, ells_spec], 
        [square_spec, ells_spec, hash_spec], 
    ]
    empty_ag, square_ag, ells_ag, hash_ag = (
        _tu.make_hash_suite_ags())
    expected_values = [
        [], 
        [hash_ag], 
        [ells_ag], 
        [ells_ag, hash_ag], 
        [square_ag], 
        [square_ag, hash_ag], 
        [square_ag, ells_ag], 
        [square_ag, ells_ag, hash_ag], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_ag_shapes(
            method_name, 
            try_names[i], 
            shape_spec_lists[i], 
            expected_values[i])
    print_test_done(method_name)

def _try_new_ag_shapes(method_name, try_name, shape_specs, expected_value):
    actual_value = _a.Agnostic.new_ag_shapes(shape_specs)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_ag_shape():
    method_name = 'new_ag_shape'
    try_names = [
        'empty', 
        'labels', 
        'square', 
        'ells', 
        'hash', 
    ]
    hash_point_specs, square_line_specs, ell_line_specs, hash_line_specs = (
        _tu.make_hash_suite_specs())
    spec_list_pairs = [
        ([], []), 
        (hash_point_specs, []), 
        ([], square_line_specs), 
        (hash_point_specs, ell_line_specs), 
        (hash_point_specs, hash_line_specs), 
    ]
    hash_ag_points, square_ag_lines, ell_ag_lines, hash_ag_lines = (
        _tu.make_hash_suite_ag_elements())
    expected_values = [
        {   'points': [], 
            'line segments': [], 
        }, 
        {   'points': sorted(hash_ag_points), 
            'line segments': [], 
        }, 
        {   'points': [], 
            'line segments': sorted(square_ag_lines), 
        }, 
        {   'points': sorted(hash_ag_points), 
            'line segments': sorted(ell_ag_lines), 
        }, 
        {   'points': sorted(hash_ag_points), 
            'line segments': sorted(hash_ag_lines), 
        }, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_ag_shape(
            method_name, try_names[i], spec_list_pairs[i], expected_values[i])
    print_test_done(method_name)

def _try_new_ag_shape(method_name, try_name, spec_list_pair, expected_value):
    point_specs, line_specs = spec_list_pair
    actual_value = _a.Agnostic.new_ag_shape(point_specs, line_specs)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_ag_lines():
    method_name = 'new_ag_lines'
    try_names = [
        'empty_list', 
        'ag01__ag12a', 
        'ag23ab_ag34b', 
    ]
    arg_tuple_lists = [
        [], 
        [   (( 0,  0,  0), (10, 10,  0), []        , 0.1), 
            ((10, 10,  0), (20, 20,  0), 'a'       , 0.2)], 
        [   ((20, 20,  0), (30, 30,  0), ['a', 'b'], 0.3), 
            ((30, 30,  0), (40, 40,  0), 'b'       , 0.4)], 
    ]
    expected_values = [
        [], 
        [   (   (( 0,  0,  0), (10, 10,  0)), 
                {   'labels': [], 
                    'weight': 0.1, 
                }), 
            (   ((10, 10,  0), (20, 20,  0)), 
                {   'labels': ['a'], 
                    'weight': 0.2, 
                })], 
        [   (   ((20, 20,  0), (30, 30,  0)), 
                {   'labels': ['a', 'b'], 
                    'weight': 0.3, 
                }), 
            (   ((30, 30,  0), (40, 40,  0)), 
                {   'labels': ['b'], 
                    'weight': 0.4, 
                })], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_ag_lines(
            method_name, try_names[i], arg_tuple_lists[i], expected_values[i])
    print_test_done(method_name)

def _try_new_ag_lines(method_name, try_name, arg_tuples, expected_value):
    actual_value = _a.Agnostic.new_ag_lines(arg_tuples)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_ag_line():
    method_name = 'new_ag_line'
    try_names = [
        'xyz1_xyz2', 
        'xyz1_xyz2_weight', 
        'xyz1_xyz2_label', 
        'xyz1_xyz2_label_weight', 
        'xyz1_xyz2_labels_weight', 
        'xyz2_xyz1_label_weight',
    ]
    xyz1, xyz2, xyz3, xyz4, xyz5, xyz6, xyz7 = [
        (10 * i, 10 * i, 0) for i in range(7)]
    input_tuples = [
        (xyz1, xyz2                 ), 
        (xyz2, xyz3, [],         0.1), 
        (xyz3, xyz4, 'c'            ), 
        (xyz4, xyz5, 'd'       , 0.3), 
        (xyz5, xyz6, ['e', 'f'], 0.4), 
        (xyz7, xyz6, 'g'       , 0.5), 
    ]
    expected_values = [
        (   (xyz1, xyz2), 
            {   'labels': [], 
                'weight': 1, 
            }), 
        (   (xyz2, xyz3), 
            {   'labels': [], 
                'weight': 0.1, 
            }), 
        (   (xyz3, xyz4), 
            {   'labels': ['c'], 
                'weight': 1, 
            }), 
        (   (xyz4, xyz5), 
            {   'labels': ['d'], 
                'weight': 0.3, 
            }), 
        (   (xyz5, xyz6), 
            {   'labels': ['e', 'f'], 
                'weight': 0.4, 
            }), 
        (   (xyz7, xyz6), 
            {   'labels': ['g'], 
                'weight': 0.5, 
            }), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_ag_line(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_new_ag_line(method_name, try_name, input_tuple, expected_value):
    xyz1 = input_tuple[0]
    xyz2 = input_tuple[1]
    n = len(input_tuple)
    if n == 2:
        actual_value = _a.Agnostic.new_ag_line(xyz1, xyz2)
    elif n == 3:
        arg = input_tuple[2]
        actual_value = _a.Agnostic.new_ag_line(xyz1, xyz2, arg)
    elif n == 4:
        labels = input_tuple[2]
        weight = input_tuple[3]
        actual_value = _a.Agnostic.new_ag_line(xyz1, xyz2, labels, weight)
    else:
        print('too many elements in the tuple')
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_ag_points():
    method_name = 'new_ag_points'
    try_names = [
        'empty_list', 
        '000__111a', 
        '222ab_333b', 
    ]
    input_lists = [
        [], 
        [   ( 0,  0,  0      ), 
            (10, 10, 10, 'a' )], 
        [   (20, 20, 20, ['a', 'b']), 
            (30, 30, 30, 'b' )], 
    ]
    expected_values = [
        [], 
        [   (( 0,  0,  0), {'labels': []        }), 
            ((10, 10, 10), {'labels': ['a']     })], 
        [   ((20, 20, 20), {'labels': ['a', 'b']}), 
            ((30, 30, 30), {'labels': ['b']     })], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_ag_points(
            method_name, try_names[i], input_lists[i], expected_values[i])
    print_test_done(method_name)

def _try_new_ag_points(method_name, try_name, input_list, expected_value):
    actual_value = _a.Agnostic.new_ag_points(input_list)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_ag_point():
    method_name = 'new_ag_point'
    try_names = [
        '000_', 
        '111a', 
        '222ab', 
        '333b', 
    ]
    input_tuples = [
        ( 0,  0,  0), 
        (10, 10, 10, 'a'), 
        (20, 20, 20, ['a', 'b']), 
        (30, 30, 30, 'b'), 
    ]
    expected_values = [
        (( 0,  0,  0), {'labels': []        }), 
        ((10, 10, 10), {'labels': ['a']     }), 
        ((20, 20, 20), {'labels': ['a', 'b']}), 
        ((30, 30, 30), {'labels': ['b']     }), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_ag_point(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_new_ag_point(method_name, try_name, input_tuple, expected_value):
    it = input_tuple
    x, y, z = it[0], it[1], it[2]
    if len(it) == 3:
        actual_value = _a.Agnostic.new_ag_point(x, y, z)
    elif len(it) > 3:
        labels = it[3]
        actual_value = _a.Agnostic.new_ag_point(x, y, z, labels)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

test_new_ag_rule()
test_new_ag_shapes()
test_new_ag_shape()
test_new_ag_lines()
test_new_ag_line()
test_new_ag_points()
test_new_ag_point()
