from view import draw as                        _d
from view import frame as                       _fr
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm
from view import view_utilities as              _vu

origin = _s.Settings.origin
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message
get_point3d = _vu.get_point3d

def test_draw_framed_object_groups():
    method_name = 'draw_framed_object_groups'
    try_names = [
        '1_T__iso', 
        '1_T_fiso', 
        '1_F__iso', 
        '1_F_fiso', 
        '2_T__iso', 
        '2_T_fiso', 
        '2_F__iso', 
        '2_F_fiso', 
        '3_T__iso', 
        '3_T_fiso', 
        '3_F__iso', 
        '3_F_fiso', 
    ]
    arg_lists = [
        [1, 'T', '_iso'], 
        [1, 'T', 'fiso'], 
        [1, 'F', '_iso'], 
        [1, 'F', 'fiso'], 
        [2, 'T', '_iso'], 
        [2, 'T', 'fiso'], 
        [2, 'F', '_iso'], 
        [2, 'F', 'fiso'], 
        [3, 'T', '_iso'], 
        [3, 'T', 'fiso'], 
        [3, 'F', '_iso'], 
        [3, 'F', 'fiso'], 
    ]
    n = len(try_names)
    for i in range(n):
        _try__draw_framed_object_groups(method_name, try_names[i], arg_lists[i])
    print("Done: test_%s" % method_name)

def _try__draw_framed_object_groups(method_name, try_name, args):
    _vm.ViewMain.clear_all()
    n, overlap, rel_locations = args
    ref_point = origin
    framed_element_lists = _d.Draw.draw_framed_object_groups(
        n, overlap, ref_point, rel_locations)
    for element_list in framed_element_lists:
        _rs.SelectObjects(element_list)
    message = try_name
    message = "[%s: %s: %s %s]" % (
        "Test", 
        try_name, 
        "Confirm the selected objects", 
        "framed (if frame); all otherwise", 
    )
    _rs.GetString(message)

def test__draw_framed_object_group():
    method_name = '_draw_framed_object_group'
    try_names = [
        '____', '___o', '__s_', '__so', '_i__', '_i_o', '_is_', '_iso', 
        'f___', 'f__o', 'f_s_', 'f_so', 'fi__', 'fi_o', 'fis_', 'fiso', 
    ]
    n = len(try_names)
    for i in range(n):
        _try__draw_framed_object_group(method_name, try_names[i])
    print("Done: test__draw_framed_object_group")

def _try__draw_framed_object_group(method_name, try_name):
    _vm.ViewMain.clear_all()
    ref_point = _d.Draw._make_random_xyz()
    objects = try_name
    framed_ob_group = _d.Draw._draw_framed_object_group(
        ref_point, objects)
    _rs.SelectObjects(framed_ob_group)
    message = ("[%s: %s: %s: %s]" % (
            "Test", 
            try_name, 
            "Confirm the selected objects", 
            "framed (if frame); all otherwise"))
    _rs.GetString(message)

def test__make_random_xyz():
    method_name = '_make_random_xyz'
    try_names = [
        'inside', 
        'straddling', 
        'outside', 
    ]
    rel_locations = [
        'i', 
        's', 
        'o', 
    ]
    n = len(try_names)
    for i in range(n):
        _try__make_random_xyz(method_name, try_names[i], rel_locations[i])
    print("Done: test__make_random_xyz")

def _try__make_random_xyz(method_name, try_name, rel_location):
    _vm.ViewMain.clear_all()
    ref_point = origin
    current_layer = _rs.CurrentLayer()
    frame = _fr.Frame.new_instance(current_layer, ref_point)
    xyz1 = _d.Draw._make_random_xyz(ref_point, rel_location)
    xyz2 = _d.Draw._make_random_xyz(ref_point, rel_location)
    line = _rs.AddLine(list(xyz1), list(xyz2))
    _rs.SelectObject(line)
    message = try_name
    print(message)

test_draw_framed_object_groups()
test__draw_framed_object_group()
test__make_random_xyz()
