from view import draw as                        _d
from view import element as                     _e
from view import frame as                       _fr
from view import line as                        _l
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
from view import text_dot as                    _td
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm

origin = _s.Settings.origin
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message
red = _s.Settings.red

def test_is_element():
    method_name = 'is_element'
    try_names = [
        'line', 
        'text_dot', 
        'point', 
        'curve', 
        'polyline', 
        'rectangle', 
        'frame_instance', 
    ]
    test_ob_reprs = [
        '_l.Line.draw_random()', 
        '_td.TextDot.draw_random()', 
        '_d.Draw.draw_random_point()', 
        '_d.Draw.draw_random_curve()', 
        '_d.Draw.draw_random_polyline()', 
        '_d.Draw.draw_random_rectangle()', 
        '_fr.Frame.new_instance(current_layer, origin)', 
    ]
    expected_values = [
        True, 
        True, 
        False, 
        False, 
        False, 
        False, 
        False, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_is_element(
            method_name, try_names[i], test_ob_reprs[i], expected_values[i])
    print("Done: test_is_element")

def _try_is_element(method_name, try_name, test_ob_repr, expected_value):
    _d.Draw.clear_all()
    current_layer = _rs.CurrentLayer()
    test_ob = eval(test_ob_repr)
    actual_value = _e.Element.is_element(test_ob)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__extract_with_frames_from_objects():
    method_name = 'extract_with_frames_from_objects'
    try_names = [
        'bad_value__no_els', 
        'bad_value__no_frames', 
        'bad_value__1_frame', 
        'bad_value__2+_frames', 
        'els__2_frames', 
    ]
    instructions = [
        '[Test: select 0 els, 2 frames]', 
        '[Test: select n els, 0 frames]', 
        '[Test: select n els, 1 frame]', 
        '[Test: select n els, 3 frames]', 
        '[Test: select n els, 2 frames]', 
    ]
    n = len(try_names)
    for i in range(n):
        _try__extract_with_frames_from_objects(
            method_name, try_names[i], instructions[i])
    print("Done: test__extract_with_frames_from_objects")

def _try__extract_with_frames_from_objects(
    method_name, try_name, instruction
):
    _d.Draw.clear_all()
    _d.Draw.draw_framed_object_groups(3, rel_locations='fi')
    obs = _rs.GetObjects(instruction)
    els, frames = _e.Element.extract_with_frames_from_objects(obs)
    _rs.ObjectColor(els, red)
    _rs.Redraw()
    _rs.SelectObjects(frames)
    _rs.GetString("Check selection and colors")

def test_find_elements():
    method_name = 'find_elements'
    try_names = [
        'none', 
        'others', 
        'lines_text_dots_others', 
    ]
    setup_type_lists = [
        [], 
        ['others'], 
        ['lines', 'text_dots', 'others'], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_find_elements(
            method_name, 
            try_names[i], 
            setup_type_lists[i], 
        )
    print("Done: test_find_elements")

def _try_find_elements(method_name, try_name, setup_types):
    _vm.ViewMain.clear_all()
    expected_elements, expected_non_elements = (
        _d.Draw.draw_objects(setup_types))
    expected_value = set(expected_elements)
    actual_elements = _e.Element.find_elements()
    actual_value = set(actual_elements)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

test_is_element()
test__extract_with_frames_from_objects()
test_find_elements()

