import                                          copy
from view import draw as                        _d
from view import frame as                       _fr
from view import layer as                       _l
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm
from view import view_utilities as              _vu

d = _tu.make_hash_suite_dict()
origin = _s.Settings.origin
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message

def test_new_instance():
    method_name = 'new_instance'
    try_names = [
        'definition_no_scale_121', 
        'definition_yes_scale_212', 
    ]
    _vm.ViewMain.clear_all()
    layer_name_1 = _l.Layer.new('layer 1')
    layer_name_2 = _l.Layer.new('layer 2')
    input_tuples = [
        (layer_name_1, ( 0, 0, 0), [1, 4, 1]), 
        (layer_name_2, (10, 0, 0), [4, 1, 1]), 
    ]
    expected_values = [
        _s.Settings.block_instance_filter, 
        _s.Settings.block_instance_filter, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_instance(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_new_instance(method_name, try_name, input_tuple, expected_value):
    layer_name, draw_point, scale = input_tuple
    ob = _fr.Frame.new_instance(layer_name, draw_point, scale)
    actual_value = _rs.ObjectType(ob)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_find_padded_frame_specs():
    method_name = 'find_padded_frame_specs'
    try_names = [
        'none', 
        'empty', 
        'point', 
        'line_x', 
        'line_y', 
        'line_z', 
        'line_xy', 
        'line_xz', 
        'line_yz', 
        'line_xyz', 
    ]
    p111, l211, l121, l112, l221, l212, l122, l222 = (
        _make_obs_for_find_padded_frame_specs())
    box_pp_lists = [
        None, 
        [],                         # 'empty', 
        _rs.BoundingBox([p111]),    # 'point', 
        _rs.BoundingBox([l211]),    # 'line_x', 
        _rs.BoundingBox([l121]),    # 'line_y', 
        _rs.BoundingBox([l112]),    # 'line_z', 
        _rs.BoundingBox([l221]),    # 'line_xy', 
        _rs.BoundingBox([l212]),    # 'line_xz', 
        _rs.BoundingBox([l122]),    # 'line_yz', 
        _rs.BoundingBox([l222]),    # 'line_xyz', 
    ]
    expected_values = [
        None, 
        (_vu.get_point3d((-10, -10, 0)), ( 20,  20,   1)),   # 'empty', 
        (_vu.get_point3d((90, 90, 100)), ( 20,  20,   1)),   # 'point', 
        (_vu.get_point3d((80, 90, 100)), (140,  20,   1)),   # 'line_x', 
        (_vu.get_point3d((90, 80, 100)), ( 20, 140,   1)),   # 'line_y', 
        (_vu.get_point3d((90, 90, 100)), ( 20,  20, 120)),   # 'line_z', 
        (_vu.get_point3d((80, 80, 100)), (140, 140,   1)),   # 'line_xy', 
        (_vu.get_point3d((80, 90, 100)), (140, 20,  120)),   # 'line_xz', 
        (_vu.get_point3d((90, 80, 100)), ( 20, 140, 120)),   # 'line_yz', 
        (_vu.get_point3d((80, 80, 100)), (140, 140, 120)),   # 'line_xyz', 
    ]
    n = len(try_names)
    for i in range(n):
        _try_find_padded_frame_specs(
            method_name, 
            try_names[i], 
            box_pp_lists[i], 
            expected_values[i])
    print_test_done(method_name)

def _make_obs_for_find_padded_frame_specs():
    a = 100
    b = 200
    aaa = (a, a, a)
    p111 = _rs.AddPoint(aaa)
    l112 = _rs.AddLine(aaa, (a, a, b))
    l121 = _rs.AddLine(aaa, (a, b, a))
    l122 = _rs.AddLine(aaa, (a, b, b))
    l211 = _rs.AddLine(aaa, (b, a, a))
    l212 = _rs.AddLine(aaa, (b, a, b))
    l221 = _rs.AddLine(aaa, (b, b, a))
    l222 = _rs.AddLine(aaa, (b, b, b))
    return [p111, l211, l121, l112, l221, l212, l122, l222]

def _try_find_padded_frame_specs(
    method_name, try_name, box_pp, expected_value
):
    actual_value = _fr.Frame.find_padded_frame_specs(box_pp)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_find_pad():
    method_name = 'find_pad'
    try_names = [
        'bad_value_negative', 
        '0', 
        'minimal', 
        'minimal_proportional_border', 
        'proportional', 
    ]
    lengths = [
        -17, 
        0, 
        20, 
        50, 
        80, 
    ]
    expected_values = [
        None, 
        10, 
        10, 
        10, 
        16, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_find_pad(
            method_name, try_names[i], lengths[i], expected_values[i])
    print_test_done(method_name)

def _try_find_pad(method_name, try_name, length, expected_value):
    actual_value = _fr.Frame.find_pad(length)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_extract_from_elements():
    _vm.ViewMain.clear_all()
    method_name = 'extract_from_elements'
    try_name = 'all_items'
    expected_value = __make_frames_for_extract_frames()
    other_items = __make_other_items_for_extract_frames()
    all_items = copy.copy(expected_value)
    all_items.extend(other_items)
    actual_value = _fr.Frame.extract_from_elements(all_items)
    if not set(actual_value) == set(expected_value):
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)
    print('Done: test_extract_from_elements')

def __make_frames_for_extract_frames():
    layer_name = 'Default'
    frame_positions = [
        ( 0, 0, 0), 
        (10, 0, 0)]
    frames = [_fr.Frame.new_instance(layer_name, position) \
        for position in frame_positions]
    return frames

def test_order_pair():
    method_name = 'order_pair'
    try_names = [
        '[a, b]', 
        '[b, a]', 
    ]
    expected_value_reprs = [
        '[a, b]', 
        '[a, b]', 
    ]
    n = len(try_names)
    for i in range(n):
        _try_order(method_name, try_names[i], expected_value_reprs[i])
    print("Done: test_order_pair")

def _try_order(method_name, try_name, expected_value_repr):
    _d.Draw.clear_all()
    layer = _rs.CurrentLayer()
    p0 = origin
    p1 = [10, 0, 0]
    a = _fr.Frame.new_instance(layer, p0)
    b = _fr.Frame.new_instance(layer, p1)
    frames = eval(try_name)
    actual_value = _fr.Frame.order_pair(frames)
    expected_value = eval(expected_value_repr)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def __make_other_items_for_extract_frames():
    lines = []
    text_dots = []
    xyz_list = [
        ( 1,  1,  2), 
        ( 1,  2,  3), 
        ( 2,  3,  5), 
        ( 3,  5,  8), 
        ( 5,  8, 13), 
        ( 8, 13, 21), 
        (13, 21, 34), 
        (21, 34, 55), 
    ]
    points = [_rs.AddPoint(xyz[0], xyz[1], xyz[2]) for xyz in xyz_list]
    n = int(len(points) / 2)
    for i in range(n):
        j = i * 2
        p0 = points[j]
        p1 = points[j + 1]
        l = _rs.AddLine(p0, p1)
        lines.append(l)
    for p in points:
        d = _rs.AddTextDot('x', p)
        text_dots.append(d)
    block_instances = ___make_block_instances_for_make_other_items_()
    items = []
    items.extend(points)
    items.extend(lines)
    items.extend(text_dots)
    items.extend(block_instances)
    return items

def ___make_block_instances_for_make_other_items_():
    xyz_list = [0, 10]
    points = []
    n = len(xyz_list)
    for i in range(n):
        for j in range(n):
            point = _rs.AddPoint(xyz_list[i], xyz_list[j], 0)
            _rs.SelectObject(point)
            points.append(point)
    point_pairs = []
    n = len(points)
    for i in range(n - 1):
        for j in range(i + 1, n):
            p0 = points[i]
            p1 = points[j]
            pair = (p0, p1)
            point_pairs.append(pair)
            p0_xyz = _rs.PointCoordinates(p0)
            p1_xyz = _rs.PointCoordinates(p1)
    lines = []
    for pair in point_pairs:
        l = _rs.AddLine(pair[0], pair[1])
        lines.append(l)
    block_name = 'x_in_square'
    p000 = (0, 0, 0)
    _rs.AddBlock(lines, p000, block_name, True)
    instances = [_rs.InsertBlock(block_name, p) for p in [
        (10, 10, 0), 
        (30, 30, 0)]]
    return instances

test_new_instance()
test_find_padded_frame_specs()
test_find_pad()
test_extract_from_elements()
test_order_pair()

