from view import grammar as                     _g
from view import layer as                       _l
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm

default_layer_name = _s.Settings.default_layer_name
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message

def test_make_default():
    method_name = 'make_default'
    try_names = [
        'does_not_exist', 
        'exists', 
    ]
    _set_up_for_make_default()
    expected_values = [
        default_layer_name, 
        default_layer_name, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_make_default(
            method_name, 
            try_names[i], 
            expected_values[i], 
        )
    print_test_done(method_name)
    
def _set_up_for_make_default():
    _vm.ViewMain.clear_all()
    _rs.DeleteLayer(default_layer_name)
    _l.Layer.new('kilroy')

def _try_make_default(method_name, try_name, expected_value):
    actual_value = _l.Layer.make_default()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_find_next_layer_name():
    method_name = 'find_next_layer_name'
    try_names = [
        'no_names', 
        'shape_0', 
        'shape_0_2', 
        'shape_0_2..10', 
        'shape_0..1_5', 
        'shape_0..2_5', 
    ]
    index_lists = [
        [], 
        [0], 
        [0, 2], 
        [0, 2, 3, 4, 5, 6, 7, 8, 9, 10], 
        [0, 1, 5], 
        [0, 1, 2, 5], 
    ]
    expected_values = [
        'Shape_1', 
        'Shape_1', 
        'Shape_1', 
        'Shape_1', 
        'Shape_2', 
        'Shape_3', 
    ]
    n = len(try_names)
    for i in range(n):
        _try_find_next_layer_name(
            method_name, 
            try_names[i], 
            index_lists[i], 
            expected_values[i], 
        )
    print_test_done(method_name)

def _try_find_next_layer_name(
    method_name, try_name, index_list, expected_value
):
    _vm.ViewMain.clear_all()
    for i in index_list:
        name_i = 'Shape_%i' % i
        _l.Layer.new(name_i)
    actual_value = _l.Layer.find_next_layer_name()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_make_layer_name():
    method_name = 'make_layer_name'
    try_names = [
        'no_stem', 
        'stem_1', 
        'stem_1_stem_3', 
        # 'stem_1_stem_3', 
        'no_stem_1', 
    ]
    setup_layer_name_lists = [
        [], 
        ['kilroy 1'], 
        ['kilroy 1', 'kilroy 3'], 
        ['kilroy 1'], 
    ]
    stems_in = [
        'kilroy', 
        'kilroy', 
        'kilroy', 
        'kilroy 1', 
    ]
    expected_values = [
        'kilroy 1', 
        'kilroy 2', 
        'kilroy 2', 
        'kilroy 1 1', 
    ]
    n = len(try_names)
    for i in range(n):
        _try_make_layer_name(
            method_name, 
            try_names[i], 
            setup_layer_name_lists[i], 
            stems_in[i], 
            expected_values[i], 
        )
    print_test_done(method_name)

def _try_make_layer_name(
    method_name, try_name, layer_names, stem_in, expected_value
):
    _g.Grammar.clear_all()
    for name in layer_names:
        _l.Layer.new(name)
    actual_value = _l.Layer.make_layer_name(stem_in)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

test_make_default()
test_find_next_layer_name()
test_make_layer_name()

