import                                          copy
from view import layer as                       _l
from view import py_form as                     _pf
from view import py_line as                     _pl
from view import py_point as                    _pp
from view import rh_form as                     _rf
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm
from view import view_utilities as              _vu

d = _tu.make_hash_suite_dict()
e = _tu.make_element_dict()
eeq = _vu.eeq
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message

def test___init__():
    method_name = '__init__'
    try_names = [
        'py_points, bad_type_container', 
        'bad_type_container, py_lines', 
        '[], []', 
        '[], bad_type_els', 
        '[], py_lines', 

        'py_points, []', 
        'py_points, bad_type_els', 
        'py_points, py_lines', 
        'bad_type_els, []', 
        'bad_type_els, bad_type_els', 

        'bad_type_els, py_lines', 
    ]
    py_points, py_lines = _make_py_points_and_py_lines()
    input_pairs = [
        (py_points, 37), 
        ('x', py_lines), 
        ([], []), 
        ([], py_points), 
        ([], py_lines), 

        (py_points, []), 
        (py_points, py_points), 
        (py_points, py_lines), 
        (py_lines, []), 
        (py_lines, py_points), 

        (py_lines, py_lines), 
    ]
    expected_values = [
        False, 
        False, 
        {   'points': [], 
            'line segments': [], 
        }, 
        False, 
        {   'points': [], 
            'line segments': [
                (   (( 0,  0,  0), (10, 10, 10)), 
                    {   'labels': ['a'], 
                        'weight': 0.01}), 
                (   ((20, 20, 20), (30, 30, 30)), 
                    {   'labels': ['b'], 
                        'weight': 0.03}), 
            ], 
        }, 

        {   'points': [
                (   (10, 10, 10), 
                    {'labels': ['a']}), 
                (   (20, 20, 20), 
                    {'labels': ['b']}), 
            ], 
            'line segments': [], 
        }, 
        False, 
        {   'points': [
                (   (10, 10, 10), 
                    {'labels': ['a']}), 
                (   (20, 20, 20), 
                    {'labels': ['b']}), 
            ], 
            'line segments': [
                (   (( 0,  0,  0), (10, 10, 10)), 
                    {   'labels': ['a'], 
                        'weight': 0.01}), 
                (   ((20, 20, 20), (30, 30, 30)), 
                    {   'labels': ['b'], 
                        'weight': 0.03}), 
            ], 
        }, 
        False, 
        False, 

        False, 
    ]
    n = len(try_names)
    for i in range(n):
        _try___init__(
            method_name, try_names[i], input_pairs[i], expected_values[i])
    print_test_done(method_name)

def _make_py_points_and_py_lines():
    n0, n1, n2, n3 = 0, 10, 20, 30
    pp1a = _pp.PyPoint(n1, n1, n1, 'a')
    pp2b = _pp.PyPoint(n2, n2, n2, 'b')
    pl01a1 = _pl.PyLine((n0, n0, n0), (n1, n1, n1), 'a', 0.01)
    pl23b3 = _pl.PyLine((n2, n2, n2), (n3, n3, n3), 'b', 0.03)
    py_points = [pp1a, pp2b]
    py_lines = [pl01a1, pl23b3]
    return (py_points, py_lines)

def _try___init__(method_name, try_name, input_pair, expected_value):
    input_0, input_1 = input_pair
    output_ = _pf.PyForm(input_0, input_1)
    if output_.is_well_formed == False:
        actual_value = output_.is_well_formed
    else:
        py_form = output_
        actual_value = py_form.agnostic
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_from_objects_and_ref_point():
    method_name = 'new_from_objects_and_ref_point'
    try_names = [
        'labeled_hash', 
        'labeled_ells', 
        'labeled_points', 
        'square', 
        'empty', 
    ]
    guids, py_forms, ref_point = _make_guids_and_py_forms()
    (   labeled_hash_guids, 
        labeled_ell_guids, 
        labeled_point_guids, 
        square_guids, 
    ) = guids
    (   labeled_hash_py_form, 
        labeled_ells_py_form, 
        labeled_points_py_form, 
        square_py_form, 
        empty_py_form, 
    ) = py_forms
    input_pairs = [
        (labeled_hash_guids, ref_point), 
        (labeled_ell_guids, ref_point), 
        (labeled_point_guids, ref_point), 
        (square_guids, ref_point), 
        ([], ref_point), 
    ]
    expected_py_forms = [
        labeled_hash_py_form, 
        labeled_ells_py_form, 
        labeled_points_py_form, 
        square_py_form, 
        empty_py_form, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_from_objects_and_ref_point(
            method_name, try_names[i], input_pairs[i], expected_py_forms[i])
    print_test_done(method_name)
        
def _make_guids_and_py_forms():
    """
     h
         |    |
    80 --+----+--
         |    |
    20 --+----+--
         |    |
    00
      0  2    8  h  = 100
      0  0    0   i = 110
    """
    _vm.ViewMain.clear_all()

    xyz_020 = (  0,  20, 0)
    xyz_080 = (  0,  80, 0)
    xyz_200 = ( 20,   0, 0)
    xyz_220 = ( 20,  20, 0)
    xyz_280 = ( 20,  80, 0)
    xyz_2h0 = ( 20, 100, 0)
    xyz_800 = ( 80,   0, 0)
    xyz_820 = ( 80,  20, 0)
    xyz_880 = ( 80,  80, 0)
    xyz_8h0 = ( 80, 100, 0)
    xyz_h20 = (100,  20, 0)
    xyz_h80 = (100,  80, 0)

    rh_point_02a = _rs.AddTextDot('a', xyz_020)
    rh_point_08a = _rs.AddTextDot('a', xyz_080)
    rh_point_20a = _rs.AddTextDot('a', xyz_200)
    rh_point_2ha = _rs.AddTextDot('a', xyz_2h0)
    rh_point_80a = _rs.AddTextDot('a', xyz_800)
    rh_point_8ha = _rs.AddTextDot('a', xyz_8h0)
    rh_point_h2a = _rs.AddTextDot('a', xyz_h20)
    rh_point_h8a = _rs.AddTextDot('a', xyz_h80)
    rh_line_02_22 = _rs.AddLine(xyz_020, xyz_220)
    rh_line_02_h2 = _rs.AddLine(xyz_020, xyz_h20)
    rh_line_08_28 = _rs.AddLine(xyz_080, xyz_280)
    rh_line_08_h8 = _rs.AddLine(xyz_080, xyz_h80)
    rh_line_20_22 = _rs.AddLine(xyz_200, xyz_220)
    rh_line_20_2h = _rs.AddLine(xyz_200, xyz_2h0)
    rh_line_22_28 = _rs.AddLine(xyz_220, xyz_280)
    rh_line_22_82 = _rs.AddLine(xyz_220, xyz_820)
    rh_line_28_2h = _rs.AddLine(xyz_280, xyz_2h0)
    rh_line_28_88 = _rs.AddLine(xyz_280, xyz_880)
    rh_line_80_82 = _rs.AddLine(xyz_800, xyz_820)
    rh_line_80_8h = _rs.AddLine(xyz_800, xyz_8h0)
    rh_line_82_88 = _rs.AddLine(xyz_820, xyz_880)
    rh_line_82_h2 = _rs.AddLine(xyz_820, xyz_h20)
    rh_line_88_8h = _rs.AddLine(xyz_880, xyz_8h0)
    rh_line_88_h8 = _rs.AddLine(xyz_880, xyz_h80)
    labeled_hash_guids = [
        rh_point_02a, 
        rh_point_08a, 
        rh_point_20a, 
        rh_point_2ha, 
        rh_point_80a, 
        rh_point_8ha, 
        rh_point_h2a, 
        rh_point_h8a, 
        rh_line_02_h2, 
        rh_line_08_h8, 
        rh_line_20_2h, 
        rh_line_80_8h, 
    ]
    labeled_ell_guids = [
        rh_point_02a, 
        rh_point_08a, 
        rh_point_20a, 
        rh_point_2ha, 
        rh_point_80a, 
        rh_point_8ha, 
        rh_point_h2a, 
        rh_point_h8a, 
        rh_line_02_22, 
        rh_line_08_28, 
        rh_line_20_22, 
        rh_line_28_2h, 
        rh_line_80_82, 
        rh_line_82_h2, 
        rh_line_88_8h, 
        rh_line_88_h8, 
    ]
    square_guids = [
        rh_line_22_28, 
        rh_line_22_82, 
        rh_line_28_88, 
        rh_line_82_88, 
    ]
    labeled_point_guids = [
        rh_point_02a, 
        rh_point_08a, 
        rh_point_20a, 
        rh_point_2ha, 
        rh_point_80a, 
        rh_point_8ha, 
        rh_point_h2a, 
        rh_point_h8a, 
    ]
    guid_lists = [
        labeled_hash_guids, 
        labeled_ell_guids, 
        square_guids, 
        labeled_point_guids, 
    ]

    ref_point = (-10, -10, 0)

    xyz_130 = ( 10,  30, 0)
    xyz_190 = ( 10,  90, 0)
    xyz_310 = ( 30,  10, 0)
    xyz_330 = ( 30,  30, 0)
    xyz_390 = ( 30,  90, 0)
    xyz_3i0 = ( 30, 110, 0)
    xyz_910 = ( 90,  10, 0)
    xyz_930 = ( 90,  30, 0)
    xyz_990 = ( 90,  90, 0)
    xyz_9i0 = ( 90, 110, 0)
    xyz_i30 = (110,  30, 0)
    xyz_i90 = (110,  90, 0)

    py_point_13a = _pp.PyPoint.new_from_xyz_and_labels(xyz_130, 'a')
    py_point_19a = _pp.PyPoint.new_from_xyz_and_labels(xyz_190, 'a')
    py_point_31a = _pp.PyPoint.new_from_xyz_and_labels(xyz_310, 'a')
    py_point_3ia = _pp.PyPoint.new_from_xyz_and_labels(xyz_3i0, 'a')
    py_point_91a = _pp.PyPoint.new_from_xyz_and_labels(xyz_910, 'a')
    py_point_9ia = _pp.PyPoint.new_from_xyz_and_labels(xyz_9i0, 'a')
    py_point_i3a = _pp.PyPoint.new_from_xyz_and_labels(xyz_i30, 'a')
    py_point_i9a = _pp.PyPoint.new_from_xyz_and_labels(xyz_i90, 'a')
    py_line_13_33 = _pl.PyLine(xyz_130, xyz_330)
    py_line_13_i3 = _pl.PyLine(xyz_130, xyz_i30)
    py_line_19_39 = _pl.PyLine(xyz_190, xyz_390)
    py_line_19_i9 = _pl.PyLine(xyz_190, xyz_i90)
    py_line_31_33 = _pl.PyLine(xyz_310, xyz_330)
    py_line_31_3i = _pl.PyLine(xyz_310, xyz_3i0)
    py_line_33_39 = _pl.PyLine(xyz_330, xyz_390)
    py_line_33_93 = _pl.PyLine(xyz_330, xyz_930)
    py_line_39_3i = _pl.PyLine(xyz_390, xyz_3i0)
    py_line_39_99 = _pl.PyLine(xyz_390, xyz_990)
    py_line_91_93 = _pl.PyLine(xyz_910, xyz_930)
    py_line_91_9i = _pl.PyLine(xyz_910, xyz_9i0)
    py_line_93_99 = _pl.PyLine(xyz_930, xyz_990)
    py_line_93_i3 = _pl.PyLine(xyz_930, xyz_i30)
    py_line_99_9i = _pl.PyLine(xyz_990, xyz_9i0)
    py_line_99_i9 = _pl.PyLine(xyz_990, xyz_i90)

    labeled_hash_py_form = _pf.PyForm(
        [   py_point_13a, 
            py_point_19a, 
            py_point_31a, 
            py_point_3ia, 
            py_point_91a, 
            py_point_9ia, 
            py_point_i3a, 
            py_point_i9a, 
        ], 
        [   py_line_13_i3, 
            py_line_19_i9, 
            py_line_31_3i, 
            py_line_91_9i, 
        ]
    )
    labeled_ells_py_form = _pf.PyForm(
        [   py_point_13a, 
            py_point_19a, 
            py_point_31a, 
            py_point_3ia, 
            py_point_91a, 
            py_point_9ia, 
            py_point_i3a, 
            py_point_i9a, 
        ], 
        [   py_line_13_33, 
            py_line_19_39, 
            py_line_31_33, 
            py_line_39_3i, 
            py_line_91_93, 
            py_line_93_i3, 
            py_line_99_9i, 
            py_line_99_i9, 
        ], 
    )
    square_py_form = _pf.PyForm(
        [], 
        [   py_line_33_39, 
            py_line_33_93, 
            py_line_39_99, 
            py_line_93_99, 
        ], 
    )
    labeled_points_py_form = _pf.PyForm(
        [   py_point_13a, 
            py_point_19a, 
            py_point_31a, 
            py_point_3ia, 
            py_point_91a, 
            py_point_9ia, 
            py_point_i3a, 
            py_point_i9a, 
        ], 
        [], 
    )
    empty_py_form = _pf.PyForm()
    py_forms = [
        labeled_hash_py_form, 
        labeled_ells_py_form, 
        square_py_form, 
        labeled_points_py_form, 
        empty_py_form, 
    ]
    return (guid_lists, py_forms, ref_point)

def _try_new_from_objects_and_ref_point(
    method_name, try_names, input_pair, expected_py_form
):
    obs, ref_point = input_pair
    actual_value = _pf.PyForm.new_from_objects_and_ref_point(obs, ref_point)
    expected_value = expected_py_form
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_names, expected_value, actual_value)

def test__get_agnostic():
    method_name = '_get_agnostic'
    try_names = [
        'empty_empty', 
        'points_lines', 
    ]
    xyz1 = (1, 1, 1)
    xyz2 = (2, 2, 2)
    xyz3 = (3, 3, 3)
    input_list_pairs = [
        ([], []), 
        (   [   _pp.PyPoint(2, 2, 2, 'b'), 
                _pp.PyPoint(2, 2, 2, 'a'), 
                _pp.PyPoint(1, 1, 1, ['b', 'a']), 
                _pp.PyPoint(1, 1, 1, 'b'), 
                _pp.PyPoint(1, 1, 1, ['a', 'b']), 
                _pp.PyPoint(1, 1, 1, 'a'), 
                _pp.PyPoint(1, 1, 1, []), 
                _pp.PyPoint(2, 2, 2, 'b'), 
            ], 
            [   _pl.PyLine(xyz1, xyz3, 'a', 0.3), 
                _pl.PyLine(xyz1, xyz3, 'a', 0.2), 
                _pl.PyLine(xyz1, xyz3,  [], 0.3), 
                _pl.PyLine(xyz1, xyz3,  [], 0.2), 
                _pl.PyLine(xyz1, xyz2, 'a', 0.3), 
                _pl.PyLine(xyz1, xyz2, 'a', 0.2), 
                _pl.PyLine(xyz1, xyz2,  [], 0.3), 
                _pl.PyLine(xyz1, xyz2,  [], 0.2), 
                _pl.PyLine(xyz1, xyz3, 'a', 0.3), 
            ]), 
    ]
    expected_values = [
        {   'points': [], 
            'line segments': []}, 
        {   'points': [
                (   (1, 1, 1), 
                    {'labels': []}), 
                (   (1, 1, 1), 
                    {'labels': ['a']}), 
                (   (1, 1, 1), 
                    {'labels': ['a', 'b']}), 
                (   (1, 1, 1), 
                    {'labels': ['a', 'b']}), 
                (   (1, 1, 1), 
                    {'labels': ['b']}), 
                (   (2, 2, 2), 
                    {'labels': ['a']}), 
                (   (2, 2, 2), 
                    {'labels': ['b']}), 
                (   (2, 2, 2), 
                    {'labels': ['b']}), 
            ], 
            'line segments': [
                (   ((1, 1, 1), (2, 2, 2)), 
                    {   'labels': [], 
                        'weight': 0.2}), 
                (   ((1, 1, 1), (2, 2, 2)), 
                    {   'labels': [], 
                        'weight': 0.3}), 
                (   ((1, 1, 1), (2, 2, 2)), 
                    {   'labels': ['a'], 
                        'weight': 0.2}), 
                (   ((1, 1, 1), (2, 2, 2)), 
                    {   'labels': ['a'], 
                        'weight': 0.3}), 
                (   ((1, 1, 1), (3, 3, 3)), 
                    {   'labels': [], 
                        'weight': 0.2}), 
                (   ((1, 1, 1), (3, 3, 3)), 
                    {   'labels': [], 
                        'weight': 0.3}), 
                (   ((1, 1, 1), (3, 3, 3)), 
                    {   'labels': ['a'], 
                        'weight': 0.2}), 
                (   ((1, 1, 1), (3, 3, 3)), 
                    {   'labels': ['a'], 
                        'weight': 0.3}), 
                (   ((1, 1, 1), (3, 3, 3)), 
                    {   'labels': ['a'], 
                        'weight': 0.3}), 
            ], 
        }, 
    ]
    n = len(try_names)
    for i in range(n):
        _try__get_agnostic(
            method_name, 
            try_names[i], 
            input_list_pairs[i], 
            expected_values[i])
    print_test_done(method_name)

def _try__get_agnostic(
    method_name, try_name, input_list_pair, expected_value
):
    py_points, py_lines = input_list_pair
    actual_value = _pf.PyForm._get_agnostic(py_points, py_lines)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_some_new_from_agnostics():
    method_name = 'some_new_from_agnostics'
    try_names = [
        'empty_list', 
        'ag_empty_ag_empty', 
        'ag_empty_ag_hash', 
        'ag_hash_ag_empty', 
        'ag_hash_ag_hash', 
    ]
    ag_empty, ag_square, ag_ells, ag_hash = _tu.make_hash_suite_ags()
    ag_shape_lists = [
        [], 
        [ag_empty, ag_empty], 
        [ag_empty, ag_hash], 
        [ag_hash, ag_empty], 
        [ag_hash, ag_hash], 
    ]
    py_empty, py_square, py_ells, py_hash = _tu.make_hash_suite_py_forms()
    expected_values = [
        [], 
        [py_empty, py_empty], 
        [py_empty, py_hash], 
        [py_hash, py_empty], 
        [py_hash, py_hash], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_some_new_from_agnostics(
            method_name, try_names[i], ag_shape_lists[i], expected_values[i])
    print_test_done(method_name)

def _try_some_new_from_agnostics(
    method_name, try_name, ag_shapes, expected_value
):
    actual_value = _pf.PyForm.some_new_from_agnostics(ag_shapes)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_from_agnostic():
    method_name = 'new_from_agnostic'
    try_names = [
        'ag_empty', 
        'ag_square', 
        'ag_ells', 
        'ag_hash', 
    ]
    ag_empty, ag_square, ag_ells, ag_hash = _tu.make_hash_suite_ags()
    ags = [
        ag_empty, 
        ag_square, 
        ag_ells, 
        ag_hash, 
    ]
    pf_empty, pf_square, pf_ells, pf_hash = _tu.make_hash_suite_py_forms()
    expected_values = [
        pf_empty, 
        pf_square, 
        pf_ells, 
        pf_hash, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_from_agnostic(
            method_name, try_names[i], ags[i], expected_values[i])
    print_test_done(method_name)

def _try_new_from_agnostic(method_name, try_name, ag_shape, expected_value):
    actual_value = _pf.PyForm.new_from_agnostic(ag_shape)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_draw_many_nicely():
    method_name = 'draw_many_nicely'
    try_names = [
        '1_rule_0_old_1_new', 
        '1_rule_1_old_1_new_non_empty', 
        '1_rule_1_old_1_new_empty', 
    ]
    _set_up_for_draw_many_nicely()
    new_py_forms = [
        d['pf_points'], 
        d['pf_square'], 
        d['pf_ells'], 
        d['pf_hash']
    ]
    pf_empty = d['pf_empty']
    py_form_lists = [
        new_py_forms, 
        new_py_forms, 
        [pf_empty], 
    ]
    instructions = [
        "[Test: Confirm new forms shifted]", 
        "[Test: Confirm new forms shifted]", 
        "[Test: Confirm new forms not shifted]", 
    ]
    expected_values = [
        (4, True), 
        (8, True), 
        (0, True), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_draw_many_nicely(
            method_name, 
            try_names[i], 
            py_form_lists[i], 
            instructions[i], 
            expected_values[i])
    print_test_done(method_name)

def _set_up_for_draw_many_nicely():
    _vm.ViewMain.clear_all()
    _draw_rule_for_draw_many_nicely()
    _draw_on_default_for_draw_many_nicely()

def _draw_rule_for_draw_many_nicely():
    py_rule = d['pr_points_to_square']
    draw_point = (-300, 0, 0)
    py_rule.draw()

def _draw_on_default_for_draw_many_nicely():
    py_form_names = ['pf_square', 'pf_hash']
    for i in range(len(py_form_names)):
        pf_name = py_form_names[i]
        py_form = d[pf_name]
        default_layer_name = _s.Settings.default_layer_name
        rh_form = py_form.draw(default_layer_name)
        guids = rh_form.els
        translation = (200 * i, 0, 0)
        _rs.MoveObjects(guids, translation)

def _try_draw_many_nicely(
    method_name, try_name, py_forms, instruction, expected_value
):
    current_layer_before = _rs.CurrentLayer('Default')
    actual_result = _pf.PyForm.draw_many_nicely(py_forms)
    current_layer_after = _rs.CurrentLayer()
    _rs.GetString(instruction)
    actual_length = len(actual_result)
    current_layer_same = (current_layer_before == current_layer_after)
    actual_value = (actual_length, current_layer_same)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__shift_forms():
    method_name = '_shift_forms'
    try_names = [
        '0_forms', 
        '2_forms', 
    ]
    forms = _set_up_for__shift_forms()
    translation = (0, 200, 0)
    input_pairs = [
        [[], translation], 
        [forms, translation], 
    ]
    instructions = [
        "[Test: Confirm that forms were not shifted]", 
        "[Test: Confirm that forms were shifted]", 
    ]
    expected_values = [
        0, 
        2, 
    ]
    n = len(try_names)
    for i in range(n):
        _try__shift_forms(
            method_name, 
            try_names[i], 
            input_pairs[i], 
            instructions[i], 
            expected_values[i])
    print_test_done(method_name)

def _set_up_for__shift_forms():
    _vm.ViewMain.clear_all()
    default = _s.Settings.default_layer_name
    py_forms = [d['pf_square'], d['pf_hash']]
    n = len(py_forms)
    forms_positioned = []
    for i in range(n):
        py_form = py_forms[i]
        rh_form = py_form.draw(default)
        guids = rh_form.els
        translation = (200 * i, 0, 0)
        form_positioned = _rs.MoveObjects(guids, translation)
        forms_positioned.append(form_positioned)
    return forms_positioned

def _try__shift_forms(
    method_name, try_name, input_pair, instruction, expected_value
):
    forms, translation = input_pair
    actual_result = _pf.PyForm._shift_forms(forms, translation)
    actual_value = len(actual_result)
    _rs.GetString(instruction)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_draw_many():
    method_name = 'draw_many'
    try_names = [
        'empty_list', 
        'pff_empty_square_ells_hash', 
    ]
    _vm.ViewMain.clear_all()
    pf_empty, pf_square, pf_ells, pf_hash = (
        d['pf_empty'], d['pf_square'], d['pf_ells'], d['pf_hash'])
    py_form_lists = (
        [], 
        [pf_empty, pf_square, pf_ells, pf_hash], 
    )
    expected_values = [
        (0, 0), 
        (4, 4 + 16 + 12), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_draw_many(
            method_name, try_names[i], py_form_lists[i], expected_values[i])
    message = "[Test: Check that each form is on its layer]"
    _rs.GetObjects(message)
    print_test_done(method_name)

def _try_draw_many(method_name, try_name, py_forms, expected_value):
    forms = _pf.PyForm.draw_many(py_forms)
    n_forms = len(forms)
    n_objects = 0
    for form in forms:
        n_objects += len(form)
    actual_value = (n_forms, n_objects)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_draw_as_group():
    method_name = 'draw_as_group'
    try_names = [
        '00_bad_type_layer_name', 
        '01_bad_type_group_name', 
        '02_pf_empty', 
        '03_pf_square', 
        '04_pf_hash', 
    ]
    _vm.ViewMain.clear_all()
    n = len(try_names)
    for i in range(n):
        layer_name = 'Layer 0%i' % i
        group_name = 'Group 0%i' % i
        _l.Layer.new(layer_name)
        _rs.AddGroup(group_name)
    pf_empty, pf_square, pf_hash = d['pf_empty'], d['pf_square'], d['pf_hash']
    input_tuples = [
        (pf_hash, 37, 'Group 00'), 
        (pf_hash, 'Layer 01', 37), 
        (pf_empty, 'Layer 02', 'Group 02'), 
        (pf_square, 'Layer 03', 'Group 03'), 
        (pf_hash, 'Layer 04', 'Group 04'), 
    ]
    expected_values = [
        None, 
        None, 
        0, 
        4, 
        12, 
    ]
    instructions = [
        None, 
        None, 
        "[Test: Select the group]", 
        "[Test: Select the group]", 
        "[Test: Select the group]", 
    ]
    for i in range(n):
        _try_draw_as_group(
            method_name, 
            try_names[i], 
            input_tuples[i], 
            instructions[i], 
            expected_values[i],  
        )
    print_test_done(method_name)

def _try_draw_as_group(
    method_name, try_name, input_tuple, instruction, expected_value
):
    py_form, layer_name, group_name = input_tuple
    result = py_form.draw_as_group(layer_name, group_name)
    if result == None:
        actual_value = result
    else:
        objects = result
        actual_value = len(objects)
        _rs.GetObjects(instruction)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_draw():
    method_name = 'draw'
    try_names = [
        'bad_type', 
        'bad_value_no_layer', 
        'empty', 
        'point', 
        'xyz', 
    ]
    _vm.ViewMain.clear_all()
    pf_empty, pf_point, pf_xyz = _set_up_for_draw()
    input_pairs = [
        (pf_xyz, 37), 
        (pf_xyz, 'kilroy'), 
        (pf_empty, 'empty'), 
        (pf_point, 'point'), 
        (pf_xyz, 'xyz'), 
    ]
    instructions = [
        None, 
        None, 
        "Check for layer and empty shape", 
        "Check for layer and point", 
        "Check for layer and xyz line", 
    ]
    expected_values = [
        type(None), 
        type(None), 
        _rf.RhForm, 
        _rf.RhForm, 
        _rf.RhForm, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_draw(
            method_name, 
            try_names[i], 
            input_pairs[i], 
            instructions[i], 
            expected_values[i])
    print_test_done(method_name)

def _set_up_for_draw():
    names = [
        'empty', 
        'point', 
        'xyz', 
    ]
    py_forms = []
    for name in names:
        pf_name = 'pf_%s' % name
        py_form = e[pf_name]
        py_forms.append(py_form)
        _rs.AddLayer(name)
    return py_forms

def _try_draw(
    method_name, 
    try_name, 
    input_pair, 
    instruction, 
    expected_value
):
    py_form, layer_name = input_pair
    actual_output = py_form.draw(layer_name)
    actual_value = type(actual_output)
    if not expected_value == type(None):
        message = "[Test: %s]" % instruction
        _rs.GetString(message)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___hash__():
    method_name = '__hash__'
    # pp: py_points, pl: py_lines
    try_names = [
        '_____', 
        '___pl', 
        'pp___', 
        'pp_pl', 
    ]
    py_points = [_pp.PyPoint((50 * i), 0, 0, 'x') for i in range(2)]
    py_lines = [_pl.PyLine(
        ((50 * i), 20, 0), ((50 * i + 30), 20, 0)) for i in range(2)]
    py_forms = [
        _pf.PyForm([]       , []      ), 
        _pf.PyForm([]       , py_lines), 
        _pf.PyForm(py_points, []      ), 
        _pf.PyForm(py_points, py_lines), 
    ]
    expected_values = [
        [True , False, False, False], 
        [False, True , False, False], 
        [False, False, True , False], 
        [False, False, False, True ], 
    ]
    n = len(try_names)
    for i in range(n):
        for j in range(n):
            try_name = '%s, %s' % (try_names[i], try_names[j])
            _try___hash__(
                method_name, 
                try_name, 
                py_forms[i], 
                py_forms[j], 
                expected_values[i][j])
    print("Done: test_%s" % method_name)

def _try___hash__(
    method_name, try_name, py_form_1, py_form_2, expected_value
):
    hash_1, hash_2 = hash(py_form_1), hash(py_form_2)
    actual_value = (hash_1 == hash_2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)
        print('%s\n%s' % (hash_1, hash_2))

def test___eq__():
    method_name = '__eq__'
    py_forms, names = _make_py_forms_and_names_for_eq_ne()
    expected_values = [
        #   _____________ _______01____ _______0102__ _______010202 
        #   0____________ 0_1a_________ 0_1a1a_______ 0_1a01_010202 
        # pp______pl______
        [   True , False, False, False, 
            False, False, False, False, 
        ], 
        # pp______pl01____
        [   False, True , False, False, 
            False, False, False, False, 
        ], 
        # pp______pl0102__
        [   False, False, True , False, 
            False, False, False, False, 
        ], 
        # pp______pl010202
        [   False, False, False, True , 
            False, False, False, False, 
        ], 
        # pp0_____pl______
        [   False, False, False, False, 
            True , False, False, False, 
        ], 
        # pp0_1a__pl______
        [   False, False, False, False, 
            False, True , False, False, 
        ], 
        # pp0_1a1apl______
        [   False, False, False, False, 
            False, False, True , False, 
        ], 
        # pp0_1a01pl010202
        [   False, False, False, False, 
            False, False, False, True , 
        ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            try_name = '%s, %s' % (names[i], names[j])
            _try___eq__(
                method_name, 
                try_name, 
                py_forms[i], 
                py_forms[j], 
                expected_values[i][j])
    print("Done: test_%s" % method_name)

def _try___eq__(method_name, try_name, pf1, pf2, expected_value):
    actual_value = (pf1 == pf2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___ne__():
    method_name = '__ne__'
    py_forms, names = _make_py_forms_and_names_for_eq_ne()
    expected_values = [
        #   _____________ _______01____ _______0102__ _______010202 
        #   0____________ 0_1a_________ 0_1a1a_______ 0_1a01_010202 
        # pp______pl______
        [   False, True , True , True , 
            True , True , True , True , 
        ], 
        # pp______pl01____
        [   True , False, True , True , 
            True , True , True , True , 
        ], 
        # pp______pl0102__
        [   True , True , False, True , 
            True , True , True , True , 
        ], 
        # pp______pl010202
        [   True , True , True , False, 
            True , True , True , True , 
        ], 
        # pp0_____pl______
        [   True , True , True , True , 
            False, True , True , True , 
        ], 
        # pp0_1a__pl______
        [   True , True , True , True , 
            True , False, True , True , 
        ], 
        # pp0_1a1apl______
        [   True , True , True , True , 
            True , True , False, True , 
        ], 
        # pp0_1a01pl010202
        [   True , True , True , True , 
            True , True , True , False, 
        ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            try_name = '%s, %s' % (names[i], names[j])
            _try___ne__(
                method_name, 
                try_name, 
                py_forms[i], 
                py_forms[j], 
                expected_values[i][j])
    print("Done: test_%s" % method_name)

def _try___ne__(method_name, try_name, pf1, pf2, expected_value):
    actual_value = (pf1 != pf2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def _make_py_forms_and_names_for_eq_ne():
    xyz0 = (0.0, 0.0, 0.0)
    xyz1 = (1.1, 1.1, 1.1)
    xyz2 = (2.2, 2.2, 2.2)
    pp0_ = _pp.PyPoint.new_from_xyz_and_labels(xyz0, '' )
    pp1a = _pp.PyPoint.new_from_xyz_and_labels(xyz1, 'a')
    pl01 = _pl.PyLine(xyz0, xyz1)
    pl02 = _pl.PyLine(xyz0, xyz2)
    pp______ = []
    pp0_____ = [pp0_]
    pp0_1a__ = [pp0_, pp1a]
    pp0_1a1a = [pp0_, pp1a, pp1a]
    pl______ = []
    pl01____ = [pl01]
    pl0102__ = [pl01, pl02]
    pl010202 = [pl01, pl02, pl02]
    py_point_lists = [
        pp0_1a__, 
        pp0_____, 
        pp______, 
        pp0_1a1a, 
    ]
    py_line_lists = [
        pl0102__, 
        pl01____, 
        pl______, 
        pl010202, 
    ]
    py_forms = [
        _pf.PyForm(pp______, pl______), 
        _pf.PyForm(pp______, pl01____), 
        _pf.PyForm(pp______, pl0102__), 
        _pf.PyForm(pp______, pl010202), 
        _pf.PyForm(pp0_____, pl______), 
        _pf.PyForm(pp0_1a__, pl______), 
        _pf.PyForm(pp0_1a1a, pl______), 
        _pf.PyForm(pp0_1a1a, pl010202), 
    ]
    names = [
        'pp______pl______', 
        'pp______pl01____', 
        'pp______pl0102__', 
        'pp______pl010202', 
        'pp0_____pl______', 
        'pp0_1a__pl______', 
        'pp0_1a1apl______', 
        'pp0_1a01pl010202', 
    ]
    return (py_forms, names)

def test___str__():
    method_name = '__str__'
    try_names = [
        '[], []', 
        '[], py_lines', 
        'py_points, []', 
        'py_points, py_lines', 
    ]
    xyz0 = ( 0,  0,  0)
    xyz1 = (10, 10, 20)
    xyz2 = (10, 20, 30)
    xyz3 = (20, 30, 50)
    pp0_ = _pp.PyPoint.new_from_xyz_and_labels(xyz0, '' )
    pp1a = _pp.PyPoint.new_from_xyz_and_labels(xyz1, 'a')
    pp2b = _pp.PyPoint.new_from_xyz_and_labels(xyz2, 'b')
    pl01 = _pl.PyLine(xyz0, xyz1)
    pl12 = _pl.PyLine(xyz1, xyz2)
    pl23 = _pl.PyLine(xyz2, xyz3)
    pps = py_points = [
        pp0_, 
        pp1a, 
        pp2b, 
    ]
    pls = py_lines = [
        pl01, 
        pl12, 
        pl23, 
    ]
    input_pairs = [
        ([] , [] ), 
        ([] , pls), 
        (pps, [] ), 
        (pps, pls), 
    ]
    expected_values = [
        "([], [])", 
        "(%s, %s)" % (
            "[]", 
            "[%s, %s, %s]" % (
                "((0, 0, 0), (10, 10, 20), [], 1.0)", 
                "((10, 10, 20), (10, 20, 30), [], 1.0)", 
                "((10, 20, 30), (20, 30, 50), [], 1.0)")),
        "(%s, %s)" % (
            "[%s, %s, %s]" % (
                "((0, 0, 0), [])", 
                "((10, 10, 20), ['a'])", 
                "((10, 20, 30), ['b'])"),
            "[]"), 
        "(%s, %s)" % (
            "[%s, %s, %s]" % (
                "((0, 0, 0), [])", 
                "((10, 10, 20), ['a'])", 
                "((10, 20, 30), ['b'])"),
            "[%s, %s, %s]" % (
                "((0, 0, 0), (10, 10, 20), [], 1.0)", 
                "((10, 10, 20), (10, 20, 30), [], 1.0)", 
                "((10, 20, 30), (20, 30, 50), [], 1.0)")),
    ]
    n = len(try_names)
    for i in range(n):
        _try___str__(
            method_name, try_names[i], input_pairs[i], expected_values[i])
    print("Done: test_%s" % method_name)

def _try___str__(
    method_name, try_name, input_pair, expected_value
):
    pps, pls = input_pair
    py_form = _pf.PyForm(pps, pls)
    actual_value = str(py_form)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___repr__():
    method_name = '__repr__'
    try_names = [
        'py_form', 
    ]
    xyz0 = (0.0, 0.0, 0.0)
    xyz1 = (1.1, 1.1, 1.1)
    xyz2 = (2.2, 2.2, 2.2)
    p0_ = _pp.PyPoint.new_from_xyz_and_labels(xyz0, '' )
    p0a = _pp.PyPoint.new_from_xyz_and_labels(xyz0, 'a')
    p1_ = _pp.PyPoint.new_from_xyz_and_labels(xyz1, '' )
    p1a = _pp.PyPoint.new_from_xyz_and_labels(xyz1, 'a')
    l01 = _pl.PyLine(xyz0, xyz1)
    l02 = _pl.PyLine(xyz0, xyz2)
    l12 = _pl.PyLine(xyz1, xyz2)
    py_points = [p0_, p0a, p1_, p1a]
    py_lines = [l01, l02, l12]
    py_form = _pf.PyForm(py_points, py_lines)
    inputs = [
        py_form, 
    ]
    expected_values = [
        py_form, 
    ]
    n = len(try_names)
    for i in range(n):
        _try___repr__(method_name, try_names[i], inputs[i], expected_values[i])
    print("Done: test_%s" % method_name)

def _try___repr__(method_name, try_name, input_, expected_value):
    actual_value = eval(repr(input_))
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

test___init__()
test_new_from_objects_and_ref_point()
test__get_agnostic()
test_some_new_from_agnostics()
test_new_from_agnostic()
test_draw_many_nicely()
test__shift_forms()
test_draw_many()
test_draw_as_group()
test_draw()
test___hash__()
test___eq__()
test___ne__()
test___str__()
test___repr__()

