import                                          copy
import                                          math
from view import agnostic as                    _a
from view import draw as                        _d
from view import layer as                       _l
from view import py_point as                    _pp
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm

print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message
TAU = math.pi * 2
c = cos30 = math.cos(TAU / 12)
s = sin60 = math.sin(TAU /  6)

def test__init__():
    method_name = '__init__'
    input_lists = [
        ['a', 'b', 'c'            ], 
        ['a', 'b', 30             ], 
        ['a', 20 , 'c'            ], 
        ['a', 20 , 30             ], 
        [10 , 'b', 30             ], 
        [10 , 20 , 'c'            ], 
        [10 , 'b', 30             ], 
        [10 , 20 , 30 , 'd'       ], 
        [10 , 20 , 30 , 40        ], 
        [10 , 20 , 30 , []        ], 
        [10 , 20 , 30 , ['d']     ], 
        [10 , 20 , 30 , ['d', 40] ], 
        [10 , 20 , 30 , ['d', 'd']], 
        [10 , 20 , 30 , ['e', 'd']], 
        [10 , 20 , 30 , [40]      ], 
    ]
    expected_values = [
        False, 
        False, 
        False, 
        False, 
        False, 
        False, 
        False, 
        (   (10, 20, 30), 
            ['d'], 
            (   (10, 20, 30), 
                {'labels': ['d']})), 
        False, 
        (   (10, 20, 30), 
            [], 
            (   (10, 20, 30), 
                {'labels': []})), 
        (   (10, 20, 30), 
            ['d'], 
            (   (10, 20, 30), 
                {'labels': ['d']})), 
        False, 
        (   (10, 20, 30), 
            ['d'], 
            (   (10, 20, 30), 
                {'labels': ['d']})), 
        (   (10, 20, 30), 
            ['d', 'e'], 
            (   (10, 20, 30), 
                {'labels': ['d', 'e']})), 
        False, 
    ]
    n = len(input_lists)
    for i in range(n):
        try_name = str(input_lists[i])
        _try___init__(
            method_name, try_name, input_lists[i], expected_values[i])
    print_test_done(method_name)

def _try___init__(method_name, try_name, inputs, expected_value):
    if len(inputs) == 3:
        x, y, z = inputs
        output_ = _pp.PyPoint(x, y, z)
    elif len(inputs) == 4:
        x, y, z, labels = inputs
        output_ = _pp.PyPoint(x, y, z, labels)
    else:
        pass
    if output_.is_well_formed:
        actual_py_point = output_
        actual_value = (
            actual_py_point.xyz, 
            actual_py_point.labels, 
            actual_py_point.agnostic)
    else:
        actual_value = output_.is_well_formed
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_some_new_from_xyz_label_pairs():
    method_name = 'some_new_from_xyz_label_pairs'

def test_new_from_xyz_and_labels():
    method_name = 'new_from_xyz_and_labels'
    
def test_some_new_from_text_dots_and_ref_point():
    method_name = 'some_new_from_text_dots_and_ref_point'
    try_names = [
        'none, p0', 
        'text_dots, p0', 
        'text_dots, p1', 
        'text_dots + others, p0', 
        'text_dots + others, p1', 
    ]
    text_dots = [
        _rs.AddTextDot('x', (10, 10, 10)), 
        _rs.AddTextDot('x', (20, 20, 20)), 
        _rs.AddTextDot('x', (30, 30, 30)), 
    ]
    others = [
        _rs.AddLine((20, 30, 40), (50, 60, 70)), 
        17, 
    ]
    text_dots_and_others = copy.copy(text_dots)
    text_dots_and_others.extend(others)
    p0, p1 = (0, 0, 0), (10, 10, 10)
    input_pairs = [
        ([], p0), 
        (text_dots, p0), 
        (text_dots, p1), 
        (text_dots_and_others, p0), 
        (text_dots_and_others, p1), 
    ]
    pp0, pp1, pp2, pp3 = [
        _pp.PyPoint(10 * i, 10 * i, 10 * i, 'x') for i in range(4)]
    expected_values = [
        [], 
        [pp1, pp2, pp3], 
        [pp0, pp1, pp2], 
        [pp1, pp2, pp3], 
        [pp0, pp1, pp2], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_some_new_from_text_dots_and_ref_point(
            method_name, try_names[i], input_pairs[i], expected_values[i])
    print("Done: test_%s" % method_name)

def _try_some_new_from_text_dots_and_ref_point(
    method_name, try_name, input_pair, expected_value
):
    text_dots, ref_point = input_pair
    actual_value = _pp.PyPoint.some_new_from_text_dots_and_ref_point(
        text_dots, ref_point)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)


def test_new_from_text_dot_and_ref_point():
    method_name = 'new_from_text_dot_and_ref_point'
    try_names = [
        # td: text dot; 0/1: origin/non-origin; _/x: no label/label
        # _, x: no label, label
        # p   : point
        # 0, 1: origin, non-origin
        "(line, p0)", 
        "(td_0, line)", 
        "(td_0, p0)", 
        "(td_0, p1)", 
        "(tdx0, p0)", 
        "(tdx0, p1)", 
        "(td_1, p0)", 
        "(td_1, p1)", 
        "(tdx1, p0)", 
        "(tdx1, p1)", 
    ]
    line_repr = "_rs.AddLine((0, 0, 0), (10, 20, 30))"
    td_0_repr = "_rs.AddTextDot('' , ( 0,  0,  0))"
    tdx0_repr = "_rs.AddTextDot('x', ( 0,  0,  0))"
    td_1_repr = "_rs.AddTextDot('' , (10, 20, 30))"
    tdx1_repr = "_rs.AddTextDot('x', (10, 20, 30))"
    p0_repr   = "( 0,  0,  0)"
    p1_repr   = "(10, 20, 30)"
    input_repr_pairs = [
        (line_repr, p0_repr), 
        (td_0_repr, line_repr), 
        (td_0_repr, p0_repr), 
        (td_0_repr, p1_repr), 
        (tdx0_repr, p0_repr), 
        (tdx0_repr, p1_repr), 
        (td_1_repr, p0_repr), 
        (td_1_repr, p1_repr), 
        (tdx1_repr, p0_repr), 
        (tdx1_repr, p1_repr), 
    ]
    expected_values = [
        None, 
        None, 
        _pp.PyPoint(  0,   0,   0, '' ), 
        _pp.PyPoint(-10, -20, -30, '' ), 
        _pp.PyPoint(  0,   0,   0, 'x'), 
        _pp.PyPoint(-10, -20, -30, 'x'), 
        _pp.PyPoint( 10,  20,  30, '' ), 
        _pp.PyPoint(  0,   0,   0, '' ), 
        _pp.PyPoint( 10,  20,  30, 'x'), 
        _pp.PyPoint(  0,   0,   0, 'x'), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_from_text_dot_and_ref_point(
            method_name, try_names[i], input_repr_pairs[i], expected_values[i])
    print("Done: test_%s" % method_name)

def _try_new_from_text_dot_and_ref_point(
    method_name, try_name, input_repr_pair, expected_value
):
    _d.Draw.clear_all()
    td_repr, p_repr = input_repr_pair
    td = eval(td_repr)
    p = eval(p_repr)
    pp = _pp.PyPoint.new_from_text_dot_and_ref_point(td, p)
    actual_value = pp
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_some_new_from_agnostics():
    method_name = 'some_new_from_agnostics'
    try_names = [
        'empty_list', 
        'ag000__ag222ab', 
        'ag111a_ag333b', 
    ]
    arg_tuples = [
        ( 0,  0,  0, []), 
        (10, 10, 10, 'a'), 
        (20, 20, 20, 'ab'), 
        (30, 30, 30, 'b'), 
    ]
    ag000_, ag111a, ag222ab, ag333b = (
        _a.Agnostic.new_ag_points(arg_tuples))
    ag_point_lists = [
        [], 
        [ag000_, ag222ab], 
        [ag111a, ag333b]
    ]
    pp000_, pp111a, pp222ab, pp333b = [
        _pp.PyPoint(tup[0], tup[1], tup[2], tup[3]) for tup in arg_tuples]
    expected_values = [
        [], 
        [pp000_, pp222ab], 
        [pp111a, pp333b], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_some_new_from_agnostics(
            method_name, try_names[i], ag_point_lists[i], expected_values[i])
    print_test_done(method_name)

def _try_some_new_from_agnostics(
    method_name, try_name, ag_points, expected_value
):
    actual_value = _pp.PyPoint.some_new_from_agnostics(ag_points)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_from_agnostic():
    method_name = 'new_from_agnostic'
    try_names = [
        'ag000_', 
        'ag000a', 
        'ag000ab', 
        'ag000b', 
        'ag222_', 
        'ag222a', 
        'ag222ab', 
        'ag222b', 
    ]
    arg_pairs = [
        (( 0,  0,  0), []), 
        (( 0,  0,  0), 'a'), 
        (( 0,  0,  0), ['a', 'b']), 
        (( 0,  0,  0), 'b'), 
        ((20, 20, 20), []), 
        ((20, 20, 20), 'a'), 
        ((20, 20, 20), ['a', 'b']), 
        ((20, 20, 20), 'b'), 
    ]
    ag000_, ag000a, ag000ab, ag000b, ag222_, ag222a, ag222ab, ag222b = (
        _a.Agnostic.new_ag_points_from_xyzs(arg_pairs))
    ag_points = [
        ag000_, 
        ag000a, 
        ag000ab, 
        ag000b, 
        ag222_, 
        ag222a, 
        ag222ab, 
        ag222b, 
    ]
    pp000_, pp000a, pp000ab, pp000b, pp222_, pp222a, pp222ab, pp222b = (
        _pp.PyPoint.some_new_from_xyz_label_pairs(arg_pairs))
    expected_values = [
        pp000_, 
        pp000a, 
        pp000ab, 
        pp000b, 
        pp222_, 
        pp222a, 
        pp222ab, 
        pp222b, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_from_agnostic(
            method_name, try_names[i], ag_points[i], expected_values[i])
    print_test_done(method_name)

def _try_new_from_agnostic(
    method_name, try_name, ag_point, expected_value
):
    actual_value = _pp.PyPoint.new_from_agnostic(ag_point)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_draw():
    method_name = 'draw'
    _vm.ViewMain.clear_all()
    _l.Layer.new('layer_0')
    _l.Layer.new('layer_1')
    _l.Layer.new('layer_2')
    try_names = [
        '000_', 
        '111a',
        '222bc',
    ]
    pp_at_000 = _pp.PyPoint(0, 0, 0, [])
    pp_at_111 = _pp.PyPoint(0, 0, 0, 'a')
    pp_at_222 = _pp.PyPoint(0, 0, 0, ['b', 'c'])
    input_tuples = [
        (pp_at_000, ( 0,  0,  0), 'layer_0'), 
        (pp_at_111, (10, 10, 10), 'layer_1'), 
        (pp_at_222, (20, 20, 20), 'layer_2'), 
    ]
    expected_values = [
        True, 
        True, 
        True, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_draw(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _try_draw(method_name, try_name, input_tuple, expected_value):
    py_point, ref_point, layer_name = input_tuple
    guid = py_point.draw(ref_point, layer_name)
    actual_value = _rs.IsTextDot(guid)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___eq__():
    method_name = '__eq__'
    points, names = _make_py_points_and_names_for_relations()
    expected_values = [
        # pp000_ 
        [   True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp000a 
        [   False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp00c_ 
        [   False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp00ca 
        [   False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp00s_ 
        [   False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp00sa
        [   False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0c0_ 
        [   False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0c0a 
        [   False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0cc_ 
        [   False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0cca 
        [   False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0cs_ 
        [   False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0csa
        [   False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0s0_ 
        [   False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0s0a 
        [   False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0sc_ 
        [   False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0sca 
        [   False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0ss_ 
        [   False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0ssa
        [   False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc00_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc00a 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc0c_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc0ca 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc0s_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc0sa
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppcc0_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
        ], 
        # ppcc0a 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
        ], 
        # ppccc_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
        ], 
        # ppccca 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
        ], 
        # ppccs_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
        ], 
        # ppccsa
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
        ], 
        # ppcs0_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
        ], 
        # ppcs0a 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
        ], 
        # ppcsc_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
        ], 
        # ppcsca 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
        ], 
        # ppcss_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
        ], 
        # ppcssa
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
        ], 
        # pps00_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps00a 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps0c_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps0ca 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps0s_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps0sa
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppsc0_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
        ], 
        # ppsc0a 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
        ], 
        # ppscc_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
        ], 
        # ppscca 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
        ], 
        # ppscs_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
        ], 
        # ppscsa
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
        ], 
        # ppss0_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
        ], 
        # ppss0a 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
            False, False, False, False, False, False,
            False, True , False, False, False, False,
            False, True , False, False, False, False,
        ], 
        # ppssc_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
        ], 
        # ppssca 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
        ], 
        # ppsss_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
            False, False, False, False, False, False,
            False, False, True , False, True , False,
            False, False, True , False, True , False,
        ], 
        # ppsssa
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
            False, False, False, False, False, False,
            False, False, False, True , False, True ,
            False, False, False, True , False, True ,
        ], 
    ]
    n = len(points)
    for i in range(n):
        for j in range(n):
            pp1 = points[i]
            pp2 = points[j]
            try_name = '%s == %s' % (names[i], names[j])
            _try___eq__(
                method_name, 
                try_name, 
                pp1, 
                pp2, 
                expected_values[i][j])
    print_test_done(method_name)

def _try___eq__(method_name, try_name, pp1, pp2, expected_value):
    actual_value = (pp1 == pp2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___ne__():
    method_name = '__ne__'
    points, names = _make_py_points_and_names_for_relations()
    expected_values = [
        # pp000_ 
        [   False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp000a 
        [   True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp00c_ 
        [   True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp00ca 
        [   True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp00s_ 
        [   True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp00sa
        [   True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0c0_ 
        [   True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0c0a 
        [   True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0cc_ 
        [   True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0cca 
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0cs_ 
        [   True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0csa
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0s0_ 
        [   True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0s0a 
        [   True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0sc_ 
        [   True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0sca 
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0ss_ 
        [   True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pp0ssa
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # ppc00_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # ppc00a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # ppc0c_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # ppc0ca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # ppc0s_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # ppc0sa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # ppcc0_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
        ], 
        # ppcc0a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
        ], 
        # ppccc_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
        ], 
        # ppccca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppccs_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
        ], 
        # ppccsa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppcs0_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
        ], 
        # ppcs0a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
        ], 
        # ppcsc_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
        ], 
        # ppcsca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppcss_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
        ], 
        # ppcssa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # pps00_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pps00a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pps0c_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pps0ca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pps0s_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # pps0sa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
        ], 
        # ppsc0_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
        ], 
        # ppsc0a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
        ], 
        # ppscc_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
        ], 
        # ppscca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppscs_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
        ], 
        # ppscsa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppss0_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, True , True , True , True , True ,
            False, True , True , True , True , True ,
        ], 
        # ppss0a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, True , True , True , True ,
            True , False, True , True , True , True ,
        ], 
        # ppssc_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
        ], 
        # ppssca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppsss_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
            True , True , True , True , True , True ,
            True , True , False, True , False, True ,
            True , True , False, True , False, True ,
        ], 
        # ppsssa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
    ]
    n = len(points)
    for i in range(n):
        for j in range(n):
            pp1 = points[i]
            pp2 = points[j]
            try_name = '%s != %s' % (names[i], names[j])
            _try___ne__(
                method_name, 
                try_name, 
                pp1, 
                pp2, 
                expected_values[i][j])
    print_test_done(method_name)

def _try___ne__(method_name, try_name, pp1, pp2, expected_value):
    actual_value = (pp1 != pp2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___ge__():
    method_name = '__ge__'
    points, names = _make_py_points_and_names_for_relations()
    expected_values = [
        # ooo_
        [   True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # oooa
        [   True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ooc_
        [   True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ooca
        [   True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # oos_
        [   True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # oosa
        [   True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # oco_
        [   True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False, 
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ocoa
        [   True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # occ_
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # occa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ocs_
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ocsa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # oso_
        [   True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            True , False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # osoa
        [   True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            True , True , False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # osc_
        [   True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # osca
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # oss_
        [   True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # ossa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # coo_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # cooa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # coc_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # coca
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # cos_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # cosa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # cco_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            True , False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            True , False, False, False, False, False, 
        ], 
        # ccoa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            True , True , False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            True , True , False, False, False, False, 
        ], 
        # ccc_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
        ], 
        # ccca
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ccs_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
        ], 
        # ccsa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # cso_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            True , False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            True , False, False, False, False, False, 
        ], 
        # csoa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            True , True , False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            True , True , False, False, False, False, 
        ], 
        # csc_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
        ], 
        # csca
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # css_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
        ], 
        # cssa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # soo_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # sooa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # soc_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # soca
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # sos_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , False, True , False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # sosa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # sco_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            True , False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            True , False, False, False, False, False, 
        ], 
        # scoa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            True , True , False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            True , True , False, False, False, False, 
        ], 
        # scc_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
        ], 
        # scca
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # scs_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
        ], 
        # scsa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sso_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            True , False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , False, False, False, False, False, 
            True , False, False, False, False, False, 
        ], 
        # ssoa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            True , True , False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , False, False, False, False, 
            True , True , False, False, False, False, 
        ], 
        # ssc_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
        ], 
        # ssca
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sss_ 
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
            True , True , True , True , True , True , 
            True , True , True , False, True , False, 
            True , True , True , False, True , False, 
        ], 
        # sssa
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
    ]
    n = len(points)
    for i in range(n):
        for j in range(n):
            try_name = '%s >= %s' % (names[i], names[j])
            pp1 = points[i]
            pp2 = points[j]
            expected_value = expected_values[i][j]
            _try___ge__(
                method_name, try_name, pp1, pp2, expected_value)
    print_test_done(method_name)

def _try___ge__(method_name, try_name, pp1, pp2, expected_value):
    actual_value = (pp1 >= pp2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___gt__():
    method_name = '__gt__'
    points, names = _make_py_points_and_names_for_relations()
    expected_values = [
        # pp000_ 
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp000a 
        [   True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp00c_ 
        [   True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp00ca 
        [   True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp00s_ 
        [   True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp00sa
        [   True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0c0_ 
        [   True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0c0a 
        [   True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0cc_ 
        [   True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0cca 
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0cs_ 
        [   True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0csa
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0s0_ 
        [   True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0s0a 
        [   True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0sc_ 
        [   True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0sca 
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0ss_ 
        [   True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pp0ssa
        [   True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc00_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc00a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc0c_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc0ca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc0s_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppc0sa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppcc0_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppcc0a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
        ], 
        # ppccc_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
        ], 
        # ppccca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppccs_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
        ], 
        # ppccsa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppcs0_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppcs0a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
        ], 
        # ppcsc_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
        ], 
        # ppcsca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppcss_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
        ], 
        # ppcssa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # pps00_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps00a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps0c_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps0ca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps0s_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # pps0sa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , True , False, True , False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppsc0_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppsc0a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
        ], 
        # ppscc_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
        ], 
        # ppscca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppscs_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
        ], 
        # ppscsa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppss0_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            True , True , True , True , True , True ,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ppss0a 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
            True , True , True , True , True , True ,
            True , False, False, False, False, False,
            True , False, False, False, False, False,
        ], 
        # ppssc_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
        ], 
        # ppssca 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
        # ppsss_ 
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
            True , True , True , True , True , True ,
            True , True , False, False, False, False,
            True , True , False, False, False, False,
        ], 
        # ppsssa
        [   True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
            True , True , True , True , True , True ,
            True , True , True , False, True , False,
            True , True , True , False, True , False,
        ], 
    ]
    n = len(points)
    for i in range(n):
        for j in range(n):
            pp1 = points[i]
            pp2 = points[j]
            try_name = '%s > %s' % (names[i], names[j])
            expected_value = expected_values[i][j]
            _try___gt__(
                method_name, try_name, pp1, pp2, expected_value)
    print_test_done(method_name)

def _try___gt__(method_name, try_name, pp1, pp2, expected_value):
    actual_value = (pp1 > pp2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___le__():
    method_name = '__le__'
    points, names = _make_py_points_and_names_for_relations()
    """
        [   ooo_, oooa, ooc_, ooca, oos_, oosa, 
            oco_, ocoa, occ_, occa, ocs_, ocsa, 
            oso_, osoa, osc_, osca, oss_, ossa, 
            coo_, cooa, coc_, coca, cos_, cosa, 
            cco_, ccoa, ccc_, ccca, ccs_, ccsa, 
            cso_, csoa, csc_, csca, css_, cssa, 
            soo_, sooa, soc_, soca, sos_, sosa, 
            sco_, scoa, scc_, scca, scs_, scsa, 
            sso_, ssoa, ssc_, ssca, sss_, sssa, 
        ], 
    """
    expected_values = [
        # ooo_
        [   True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oooa
        [   False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ooc_
        [   False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ooca
        [   False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oos_
        [   False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oosa
        [   False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oco_
        [   False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ocoa
        [   False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # occ_
        [   False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # occa
        [   False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ocs_
        [   False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ocsa
        [   False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oso_
        [   False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # osoa
        [   False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # osc_
        [   False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # osca
        [   False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oss_
        [   False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ossa
        [   False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # coo_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # cooa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # coc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # coca
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # cos_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # cosa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # cco_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ccoa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
        ], 
        # ccc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # ccca
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # ccs_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # ccsa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # cso_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # csoa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
        ], 
        # csc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # csca
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # css_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # cssa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # soo_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sooa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # soc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # soca
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sos_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sosa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, True , False, True, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sco_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # scoa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
        ], 
        # scc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # scca
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # scs_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # scsa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # sso_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ssoa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
        ], 
        # ssc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # ssca
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # sss_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # sssa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
    ]
    n = len(points)
    for i in range(n):
        for j in range(n):
            try_name = '%s <= %s' % (names[i], names[j])
            pp1 = points[i]
            pp2 = points[j]
            expected_value = expected_values[i][j]
            _try___le__(
                method_name, try_name, pp1, pp2, expected_value)
    print_test_done(method_name)

def _try___le__(method_name, try_name, pp1, pp2, expected_value):
    actual_value = (pp1 <= pp2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___lt__():
    method_name = '__lt__'
    points, names = _make_py_points_and_names_for_relations()
    expected_values = [
        # ooo_
        [   False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oooa 
        [   False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ooc_
        [   False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ooca 
        [   False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oos_
        [   False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oosa
        [   False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oco_
        [   False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ocoa
        [   False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # occ_
        [   False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # occa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ocs_
        [   False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ocsa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oso_
        [   False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # osoa
        [   False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # osc_
        [   False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # osca
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # oss_
        [   False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # ossa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # coo_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # cooa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # coc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # coca
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # cos_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # cosa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # cco_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
        ], 
        # ccoa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # ccc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # ccca
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # ccs_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # ccsa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
        ], 
        # cso_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
        ], 
        # csoa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # csc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # csca
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # css_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # cssa
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # soo_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, True , True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sooa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, True , True , True , True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # soc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # soca
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sos_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, True , False, True , 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sosa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
            False, False, False, False, False, False, 
            True , True , True , True , True , True , 
            True , True , True , True , True , True , 
        ], 
        # sco_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
        ], 
        # scoa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # scc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # scca
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # scs_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # scsa
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # sso_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
            False, False, False, False, False, False, 
            False, True , True , True , True , True , 
            False, True , True , True , True , True , 
        ], 
        # ssoa
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
            False, False, False, False, False, False, 
            False, False, True , True , True , True , 
            False, False, True , True , True , True , 
        ], 
        # ssc_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # ssca
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,

            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,

            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
        # sss_
        [   False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
            False, False, False, False, False, False, 
            False, False, False, True , False, True , 
            False, False, False, True , False, True , 
        ], 
        # sssa
        [   False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
            False, False, False, False, False, False,
        ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            try_name = '%s < %s' % (names[i], names[j])
            pp1 = points[i]
            pp2 = points[j]
            expected_value = expected_values[i][j]
            _try___lt__(
                method_name, try_name, pp1, pp2, expected_value)
    print_test_done(method_name)

def _try___lt__(method_name, try_name, pp1, pp2, expected_value):
    actual_value = (pp1 < pp2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def _make_py_points_and_names_for_relations():
    values = [0, cos30, sin60]
    points, names = [], []
    bit_values = [0, cos30, sin60]
    bit_names = ['o', 'c', 's']
    label_values = ['', 'a']
    label_names = ['_', 'a']
    n = len(bit_values)
    m = len(label_values)
    for i in range(n):
        for j in range(n):
            for k in range(n):
                for l in range(m):
                    point = _pp.PyPoint(
                        bit_values[i], 
                        bit_values[j], 
                        bit_values[k], 
                        label_values[l])
                    points.append(point)
                    name = '%s%s%s%s' % (
                        bit_names[i], 
                        bit_names[j], 
                        bit_names[k], 
                        label_names[l])
                    names.append(name)
    return (points, names)

def _test_make_py_points_and_names_for_relations():
    points, names = _make_py_points_and_names_for_relations()
    print(names)
    """
    [   ooo_, oooa, ooc_, ooca, oos_, oosa, 
        oco_, ocoa, occ_, occa, ocs_, ocsa, 
        oso_, osoa, osc_, osca, oss_, ossa, 
        coo_, cooa, coc_, coca, cos_, cosa, 
        cco_, ccoa, ccc_, ccca, ccs_, ccsa, 
        cso_, csoa, csc_, csca, css_, cssa, 
        soo_, sooa, soc_, soca, sos_, sosa, 
        sco_, scoa, scc_, scca, scs_, scsa, 
        sso_, ssoa, ssc_, ssca, sss_, sssa, 
    ]
    """

def test___str__():
    method_name = '__str__'
    pps = py_points = [
        _pp.PyPoint(1, 2, 3), 
        _pp.PyPoint(1, 2, 3, 'a'), 
        _pp.PyPoint(1, 2, 3, ['a', 'b']), 
    ]
    expected_values = [
        "((1, 2, 3), [])", 
        "((1, 2, 3), ['a'])", 
        "((1, 2, 3), ['a', 'b'])", 
    ]
    n = len(pps)
    for i in range(n):
        _try___str__(method_name, pps[i], expected_values[i])
    print_test_done(method_name)

def _try___str__(method_name, pp, expected_value):
    try_name = expected_value
    actual_value = str(pp)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___repr__():
    method_name = '__repr__'
    pps = py_points = [
        _pp.PyPoint(1, 2, 3), 
        _pp.PyPoint(1, 2, 3, 'a'), 
        _pp.PyPoint(1, 2, 3, ['a', 'b']), 
    ]
    expected_values = pps
    n = len(pps)
    for i in range(n):
        _try___repr__(method_name, pps[i], expected_values[i])
    print_test_done(method_name)

def _try___repr__(method_name, pp, expected_value):
    try_name = expected_value
    actual_value = eval(repr(pp))
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

test__init__()
test_some_new_from_text_dots_and_ref_point()
test_new_from_text_dot_and_ref_point()
test_some_new_from_agnostics()
test_new_from_agnostic()
test_draw()
test___eq__()
test___ne__()
test___ge__()
test___gt__()
test___le__()
test___lt__()
test___str__()
test___repr__()

# _test_make_py_points_and_names_for_relations()



