from controller import controller_main as       _cm
from view import draw as                        _d
from view import grammar as                     _g
from view import layer as                       _l
from view import py_form as                     _pf
from view import py_line as                     _pl
from view import py_point as                    _pp
from view import py_rule as                     _pr
from view import rh_rule as                     _rr
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
import sortalgi as                              _sgi
import                                          sortal
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm
from view import view_utilities as              _vu

d = _tu.make_hash_suite_dict()
e = _tu.make_element_dict()
origin_p3d = _s.Settings.origin_p3d
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message

def test___init__():
    method_name = '__init__'
    try_names = [
        'bad_type_a', 
        'bad_type_b', 
        'bad_type_descr', 
        'bad_value_a', 
        'hash_square_descr', 
        'hash_empty_descr', 
    ]
    sortal.rule.cleanup()
    _sgi.sortal_setup()
    pf_empty, pf_square, pf_ells, pf_hash = _tu.make_hash_suite_py_forms()
    descr = '<description>'
    tuples = [
        (37, pf_square, descr), 
        (pf_hash, 37, descr), 
        (pf_hash, pf_square, 37), 
        (pf_empty, pf_square, descr), 
        (pf_hash, pf_square), 
        (pf_hash, pf_empty, descr), 
    ]
    empty_hash, square_hash, hash_hash = (
        hash(pf_empty), 
        hash(pf_square), 
        hash(pf_hash))
    hash_to_square_id, hash_to_empty_id = (
        hash((hash_hash, square_hash)), 
        hash((hash_hash, empty_hash)))
    expected_values = [
        False, 
        False, 
        False, 
        False, 
        (pf_hash, pf_square, hash_to_square_id, ''), 
        (pf_hash, pf_empty, hash_to_empty_id, descr), 
    ]
    n = len(try_names)
    for i in range(n):
        _try___init__(
            method_name, try_names[i], tuples[i], expected_values[i])
    print_test_done(method_name)

def _try___init__(method_name, try_name, tuple_, expected_value):
    if len(tuple_) == 2:
        py_a, py_b = tuple_
        py_rule = _pr.PyRule(py_a, py_b)
    elif len(tuple_) == 3:
        py_a, py_b, descr = tuple_
        py_rule = _pr.PyRule(py_a, py_b, descr)
    else:
        print('bad tuple')
    if not py_rule.is_well_formed:
        actual_value = py_rule.is_well_formed
    else:
        actual_value = (py_rule.a, py_rule.b, py_rule.id_, py_rule.descr)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def _try__get_available_name(method_name, try_name, input_, expected_value):
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_many_new_from_rule_groups():           ##  no test
    method_name = 'many_new_from_rule_groups'

def test_new_from_rule_group():
    method_name = 'new_from_rule_group'
    try_names = [
        'bad_type', 
        'bad_value_no_rule', 
        'points_to_square', 
    ]
    pr_points_to_square = _set_up_for_new_from_rule_group()
    layer_names = [
        37, 
        'no_rule', 
        str(pr_points_to_square.id_), 
    ]
    instructions = [
        None, 
        None, 
        "[Test: Confirm rules look the same]", 
    ]
    expected_values = [
        None, 
        None, 
        _pr.PyRule,                             ##  How to test pr == pr
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_from_rule_group(
            method_name, 
            try_names[i], 
            layer_names[i], 
            instructions[i], 
            expected_values[i])
    print_test_done(method_name)

def _set_up_for_new_from_rule_group():
    _vm.ViewMain.clear_all()
    _l.Layer.new('no_rule')
    pf_points, pf_square = d['pf_points'], d['pf_square']
    pr_points_to_square = _pr.PyRule(pf_points, pf_square)
    pr_points_to_square.draw()
    return pr_points_to_square

def _try_new_from_rule_group(
    method_name, try_name, layer_name, instruction, expected_value
):
    received_value = _pr.PyRule.new_from_rule_group(layer_name)
    if received_value:
        py_rule = received_value
        py_rule.draw()
        _rs.GetString(instruction)
        actual_value = type(py_rule)
    else:
        actual_value = received_value
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_exists():
    method_name = 'exists'
    try_names = [
        'does_not_exist', 
        'same_stem_no_extension', 
        'same_stem_with_extension', 
    ]
    py_rules = _set_up_for_exists()
    expected_values = [
        False, 
        True, 
        True, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_exists(
            method_name, try_names[i], py_rules[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_exists():
    _vm.ViewMain.clear_all()
    pr_a = d['pr_points_to_square']
    pr_b = d['pr_square_to_empty']
    pr_c = d['pr_points_to_empty']
    prs_to_draw = [pr_b, pr_c, pr_c]
    for pr in prs_to_draw:
        pr.draw()
    return [pr_a, pr_b, pr_c]

def _try_exists(method_name, try_name, py_rule, expected_value):
    actual_value = _pr.PyRule.exists(py_rule)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__get_stem():
    method_name = '_get_stem'
    try_names = [
        '', 
        '_', 
        '_ext', 
        'stem', 
        'stem_', 
        'stem_ext', 
    ]
    expected_values = [
        '', 
        '', 
        '', 
        'stem', 
        'stem', 
        'stem', 
    ]
    n = len(try_names)
    for i in range(n):
        _try__get_stem(method_name, try_names[i], expected_values[i])
    print_test_done(method_name)

def _try__get_stem(method_name, try_name, expected_value):
    actual_value = _pr.PyRule._get_stem(try_name)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_draw():
    method_name = 'draw'
    _vm.ViewMain.clear_all()
    try_names = [
        'pr_points_to_square', 
        'pr_square_to_points', 
    ]
    pr_points_to_square, pr_square_to_points = (
        d['pr_points_to_square'], d['pr_square_to_points'])
    py_rules = [
        pr_points_to_square, 
        pr_square_to_points, 
    ]
    expected_values = [
        _rr.RhRule, 
        _rr.RhRule, 
    ]
    instructions = [
        '[Test: Check frame, group]', 
        '[Test: Check frame, group]',  
    ]
    n = len(try_names)
    for i in range(n):
        _try_draw(
            method_name, 
            try_names[i], 
            py_rules[i], 
            expected_values[i], 
            instructions[i], 
        )
    print_test_done(method_name)

def _try_draw(method_name, try_name, py_rule, expected_value, instruction):
    _vm.ViewMain.clear_all()
    rh_rule = py_rule.draw()
    actual_value = type(rh_rule)
    _rs.GetString(instruction)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__draw_grouped_forms():
    method_name = '_draw_grouped_forms'
    try_names = [
        # 'bad_value_no_such_layer', 
        'hash_to_square', 
        'ells_to_empty', 
    ]
    py_rules = [
        # d['pr_square_to_ells'], 
        d['pr_hash_to_square'], 
        d['pr_ells_to_empty'], 
    ]
    layer_names = [
        # 'kilroy', 
        'layer_hash_to_square', 
        'layer_ells_to_empty', 
    ]
    expected_values = [
        # True, 
        True, 
        True, 
    ]
    n = len(try_names)
    for i in range(n):
        _try__draw_grouped_forms(
            method_name, 
            try_names[i], 
            py_rules[i], 
            layer_names[i], 
            expected_values[i])
    print_test_done(method_name)

def _try__draw_grouped_forms(
    method_name, try_name, py_rule, layer_name, expected_value
):
    _vm.ViewMain.clear_all()
    _rs.AddLayer(layer_name)
    actual_form_group_names = (
        py_rule._draw_grouped_forms(layer_name))
    actual_ob_sets = []
    for name in actual_form_group_names:
        obs = _rs.ObjectsByGroup(name)
        actual_ob_set = set(obs)
        actual_ob_sets.append(actual_ob_set)
    left_right = ['left', 'right']
    expected_ob_sets = []
    n = len(left_right)
    for i in range(n):
        message_1 = "[Test: If the %s shape is empty, enter 0]" % (
            left_right[i])
        string = _rs.GetString(message_1)
        if string == '0':
            obs = []
        else:
            message_2 = "[Test: Select %s group]" % left_right[i]
            obs = _rs.GetObjects(message_2)
        expected_ob_sets.append(set(obs))
    actual_value = (actual_ob_sets == expected_ob_sets)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__find_common_bounding_points():
    method_name = '_find_common_bounding_points'
    try_names = [
        'empty_empty', 
        'empty_ells', 
        'ells_empty', 
        'ells_ells', 
        'square_ells', 
    ]
    obs_lists, bounding_point_lists = (
        _set_up_for__find_common_bounding_points())
    obs_empty, obs_ells, obs_square, obs_hash = obs_lists
    ob_list_pairs = [
        (obs_empty, obs_empty), 
        (obs_empty, obs_ells), 
        (obs_ells, []),  
        (obs_ells, obs_ells), 
        (obs_square, obs_ells), 
    ]
    pp_empty, pp_ells, pp_square, pp_hash = bounding_point_lists
    expected_values = [
        [], 
        pp_ells, 
        pp_ells, 
        pp_ells, 
        pp_hash, 
    ]
    n = len(try_names)
    for i in range(n):
        _try__find_common_bounding_points(
            method_name, 
            try_names[i], 
            ob_list_pairs[i], 
            expected_values[i], 
        )
    print_test_done(method_name)

def _set_up_for__find_common_bounding_points():
    """Set up bounding point lists. Draw grouped forms"""
    _vm.ViewMain.clear_all()
    layer_name = _s.Settings.default_layer_name
    pf_names = [
        'pf_empty', 
        'pf_ells', 
        'pf_square', 
        'pf_hash', 
    ]
    obs_lists = []
    bounding_point_lists = []
    for pf_name in pf_names:
        _rs.AddGroup(pf_name)
        py_form = d[pf_name]
        rh_form = py_form.draw(layer_name)
        obs = rh_form.els
        obs_lists.append(obs)
        _rs.AddObjectsToGroup(obs, pf_name)
        box = _rs.BoundingBox(obs)
        if box == None:
            pp = []
        else:
            pp = box
        bounding_point_lists.append(pp)
    return (obs_lists, bounding_point_lists)

def _try__find_common_bounding_points(
    method_name, try_name, ob_list_pairs, expected_value
):
    obs_a, obs_b = ob_list_pairs
    py_rule = d['pr_points_to_empty']
    actual_value = py_rule._find_common_bounding_points(obs_a, obs_b)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__find_translation_to_staging_area():
    method_name = '_find_translation_to_staging_area'
    try_names = [
        'x00', 
        '0y0', 
        '00z', 
        'xy0', 
        'x0z', 
        '0yz', 
        'xyz', 
    ]
    p = insert_point = _vu.get_point3d((100, 100, 100))
    input_pairs = [
        (p, (100,   0,   0)), 
        (p, (  0, 100,   0)), 
        (p, (  0,   0, 100)), 
        (p, (100, 100,   0)), 
        (p, (100,   0, 100)), 
        (p, (  0, 100, 100)), 
        (p, (100, 100, 100)), 
    ]
    origin = _s.Settings.origin
    expected_values = [
        _rs.VectorCreate((-300, 0, -100), origin), 
        _rs.VectorCreate((-200, 0, -100), origin), 
        _rs.VectorCreate((-200, 0, -100), origin), 
        _rs.VectorCreate((-300, 0, -100), origin), 
        _rs.VectorCreate((-300, 0, -100), origin), 
        _rs.VectorCreate((-200, 0, -100), origin), 
        _rs.VectorCreate((-300, 0, -100), origin), 
    ]
    n = len(try_names)
    for i in range(n):
        _try__find_translation_to_staging_area(
            method_name, try_names[i], input_pairs[i], expected_values[i])
    print_test_done(method_name)

def _try__find_translation_to_staging_area(
    method_name, try_name, input_pair, expected_value
):
    insert_point, scale = input_pair
    pr = d['pr_points_to_empty']
    actual_value = pr._find_translation_to_staging_area(insert_point, scale)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__lay_out_rule():
    method_name = '_lay_out_rule'

def test__find_form_translations():
    method_name = '_find_form_translations'
    try_names = [
        'empty', 
        'point', 
        'xy', 
        'xz', 
        'yz', 
        'xyz', 
    ]
    bpp_lists = [
        [], 
        # bpp_point, 
        # bpp_xy, 
        # bpp_xz, 
        # bpp_yz, 
        # bpp_xyz, 
    ]
    expected_values = [
        #   (   ta, 
        #       tb, 
        #       p), 
        # bpp_empty, 
        (   (   0 - 100 - 20 - 40, 
                0 - 100 - 20, 
                0),                             # ta
            (   0 - 100 - 20, 
                0 + 100, 
                0),                             # tb
            (   0 - 100 - 10, 
                0 + 10, 
                0)),                            # p
        # bpp_point, 
        (   (   -90 - 100 - 20 - 40, 
                -90 - 100 - 20, 
                -90),                           # ta
            (   -90 - 100 - 20, 
                -90 + 100, 
                -90),                           # tb
            (   -90 - 100 - 10, 
                -90 + 10, 
                -90)),                          # p
        # bpp_xy, 
        (   (   -10 - 100 - (96 * 2), 
                -10 + 100, 
                -10),                           # ta
            (   -10 - 100 - 16, 
                -10 + 100, 
                -10),                           # tb
            (   -10 - 100 - (96 / 2), 
                -10 + 100 + 10, 
                -10)),                          # p
        # bpp_xz, 
        # bpp_yz, 
        # bpp_xyz, 
    ]
    n = len(try_names)
    for i in range(1):
        _try__find_form_translations(
            method_name, try_names[i], bpp_lists[i], expected_values[i])
    print_test_done(method_name)

def _try__find_form_translations(method_name, try_name, bpp, expected_value):
    pr = e['pr_point_empty']
    actual_value = pr._find_form_translations(bpp)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__make_unique_name():
    method_name = '_make_unique_name'
    try_names = [
        'names_in_use____', 
        'names_in_use___2', 
        'names_in_use__1_', 
        'names_in_use__12', 
        'names_in_use_0__', 
        'names_in_use_0_2', 
        'names_in_use_01_', 
        'names_in_use_012', 
    ]
    py_rule = e['pr_xyz_point']
    id_, id_1, id_2, id_3 = [
        '%i' % py_rule.id_, 
        '%i_1' % py_rule.id_, 
        '%i_2' % py_rule.id_,
        '%i_3' % py_rule.id_,
    ]
    name_lists = [
        [], 
        [id_2], 
        [id_1], 
        [id_1, id_2], 
        [id_], 
        [id_, id_2], 
        [id_, id_1], 
        [id_, id_1, id_2], 
    ]
    expected_values = [
        id_, 
        id_, 
        id_, 
        id_, 
        id_1, 
        id_1, 
        id_2, 
        id_3, 
    ]
    n = len(try_names)
    for i in range(n):
        _try__make_unique_name(
            method_name, 
            try_names[i], 
            py_rule, 
            name_lists[i], 
            expected_values[i], 
        )
    print_test_done(method_name)

def _try__make_unique_name(
    method_name, try_name, py_rule, names, expected_value
):
    _set_up_layers_for__make_unique_name(names)
    actual_value = py_rule._make_unique_name()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def _set_up_layers_for__make_unique_name(names):
    _vm.ViewMain.clear_all()
    for name in names:
        _l.Layer.new(name)

def test___hash__():
    method_name = '__hash__'
    py_rules, names = _make_py_rules_and_names_for_hash_eq_ne()
    try_names = names
    expected_types = [
        int, 
        int, 
        int, 
        int, 
        int, 
        int, 
    ]
    n = len(try_names)
    for i in range(n):
        _try___hash__(
            method_name, try_names[i], py_rules[i], expected_types[i])
    print_test_done(method_name)

def _try___hash__(method_name, try_name, py_rule, expected_type):
    repr_str = 'py_rule.%s()' % method_name
    actual_hash = eval(repr_str)
    actual_type = type(actual_hash)
    if not actual_type == expected_type:
        print_test_error_message(
            method_name, try_name, expected_type, actual_type)

def test___eq__():
    method_name = '__eq__'
    py_rules, names = _make_py_rules_and_names_for_hash_eq_ne()
    expected_values = [
        #ao     aa     ab     bo     ba     bb
        [True , False, False, False, False, False], 
        [False, True , False, False, False, False], 
        [False, False, True , False, False, False], 
        [False, False, False, True , False, False], 
        [False, False, False, False, True , False], 
        [False, False, False, False, False, True ], 
    ]
    n = len(py_rules)
    for i in range(n):
        for j in range(n):
            try_name = '%s, %s' % (names[i], names[j])
            _try___eq__(
                method_name, 
                py_rules[i], 
                py_rules[j], 
                try_name, 
                expected_values[i][j])
    print_test_done(method_name)

def _try___eq__(method_name, pr1, pr2, try_name, expected_value):
    actual_value = (pr1 == pr2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___ne__():
    method_name = '__ne__'
    py_rules, names = _make_py_rules_and_names_for_hash_eq_ne()
    expected_values = [
        #ao     aa     ab     bo     ba     bb
        [False, True , True , True , True , True ], 
        [True , False, True , True , True , True ], 
        [True , True , False, True , True , True ], 
        [True , True , True , False, True , True ], 
        [True , True , True , True , False, True ], 
        [True , True , True , True , True , False], 
    ]
    n = len(py_rules)
    for i in range(n):
        for j in range(n):
            try_name = '%s, %s' % (names[i], names[j])
            _try___ne__(
                method_name, 
                py_rules[i], 
                py_rules[j], 
                try_name, 
                expected_values[i][j])
    print_test_done(method_name)

def _try___ne__(method_name, pr1, pr2, try_name, expected_value):
    actual_value = (pr1 != pr2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def _make_py_rules_and_names_for_hash_eq_ne():
    _vm.ViewMain.clear_all()
    p00 = origin_p3d
    p50 = _vu.get_point3d((50, 0, 0))
    obs1 = _d.Draw._draw_framed_object_group(p00)
    obs2 = _d.Draw._draw_framed_object_group(p50)
    o = _pf.PyForm.new_empty()
    a = _pf.PyForm.new_from_objects_and_ref_point(obs1, p00)
    b = _pf.PyForm.new_from_objects_and_ref_point(obs2, p50)
    pf1, pf2 = [a, b], [o, a, b]
    names1, names2 = ['a', 'b'], ['o', 'a', 'b']
    len1 = len(pf1)
    len2 = len(pf2)
    py_rules = []
    names = []
    for i in range(len1):
        for j in range(len2):
            py_rule = _pr.PyRule(pf1[i], pf2[j])
            py_rules.append(py_rule)
            name = '%s%s' % (names1[i], names2[j])
            names.append(name)
    return (py_rules, names)

def test___repr__():
    method_name = '__repr__'
    try_names = [
        'hash_to_empty', 
        'hash_to_square', 
    ]
    py_empty, py_square, py_ells, py_hash = _tu.make_hash_suite_py_forms()
    py_rules = [
        _pr.PyRule(py_hash, py_empty), 
        _pr.PyRule(py_hash, py_square), 
    ]
    expected_values = [
        True, 
        True, 
    ]
    n = len(try_names)
    for i in range(n):
        _try___repr__(
            method_name, try_names[i], py_rules[i], expected_values[i])
    print_test_done(method_name)

def _try___repr__(method_name, try_name, py_form, expected_value):
    actual_value = (eval(repr(py_form)) == py_form)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

# def test__make_py_rules_and_names_for_hash_eq_ne():
    # py_rules, names = _make_py_rules_and_names_for_hash_eq_ne()
    # n = len(py_rules)
    # for i in range(n):
    #     print('%i: %s: %s' % (i, names[i], py_rules[i]))

test___init__()
# test_many_new_from_rule_groups()              ##  no test
test_new_from_rule_group()

test__draw_grouped_forms()
test__find_common_bounding_points()
test__find_translation_to_staging_area()
test__lay_out_rule()

test_exists()
# _set_up_for_exists()
test__get_stem()
# test_draw_many()                              ##  no test
test_draw()
test__make_unique_name()

test___hash__()
test___eq__()
test___ne__()
test___repr__()

# test__make_py_rules_and_names_for_hash_eq_ne()
