from controller import controller_main as       _cm
from view import draw as                        _d
from view import frame as                       _fr
from view import layer as                       _l
from view import py_form as                     _pf
from view import rh_form as                     _rf
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm
from view import view_utilities as              _vu

# _cm.ControllerMain.reset_sortal()
d = _tu.make_hash_suite_dict()
e = _tu.make_element_dict()
default_layer_name = _s.Settings.default_layer_name
origin = _s.Settings.origin
origin_p3d = _s.Settings.origin_p3d
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message

def test___init__():
    method_name = '__init__'
    _d.Draw.clear_all()
    try_names = [
        'bad_type_obs', 
        'bad_type_ref_point', 
        'nil_p0', 
        'nil_p1', 
        'obs_p0', 
        'obs_p1', 
    ]
    p0 = ( 0,  0,  0)
    p1 = (10, 20, 30)
    p2 = (60, 50, 40)
    td1 = text_dot_1 = _rs.AddTextDot('x', p1)
    l01 = line_01 = _rs.AddLine(p0, p1)
    l12 = line_12 = _rs.AddLine(p1, p2)
    c012 = curve_012 = _rs.AddCurve([p0, p1, p2])
    els = [td1, l01, l12]
    obs = [td1, l01, l12, c012]
    input_pairs = [
        (37, p0), 
        (obs, 37), 
        ([], p0), 
        ([], p1), 
        (obs, p0), 
        (obs, p1), 
    ]
    expected_values = [
        False, 
        False, 
        ([], origin_p3d), 
        ([], origin_p3d), 
        (sorted(els), p0), 
        (sorted(els), p1), 
    ]
    n = len(try_names)
    for i in range(n):
        _try___init__(
            method_name, try_names[i], input_pairs[i], expected_values[i])
    print_test_done(method_name)

def _try___init__(method_name, try_name, input_pair, expected_value):
    obs, ref_point = input_pair
    form = _rf.RhForm(obs, ref_point)
    if form.is_well_formed == False:
        actual_value = form.is_well_formed
    else:
        actual_value = (form.els, form.ref_point)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_is_a_form():
    method_name = 'is_a_form'
    try_names = [
        'empty_list', 
        'yes_objects_0_elements', 
        'yes_objects_1_element', 
        'yes_objects_2_elements', 
    ]
    _set_up_is_a_form()
    instructions = [
        None, 
        '[Test: select obs, 0 elements]', 
        '[Test: select obs, 1 element]', 
        '[Test: select obs, 2 elements]', 
    ]
    expected_values = [
        False, 
        False, 
        True, 
        True, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_is_a_form(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _try_is_a_form(method_name, try_name, instruction, expected_value):
    if instruction == None:
        obs = []
    else:
        obs = _rs.GetObjects(instruction)
    actual_value = _rf.RhForm.is_a_form(obs)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def _set_up_is_a_form():
    _vm.ViewMain.clear_all()
    pf_hash = d['pf_hash']
    pf_hash.draw('Default')
    _fr.Frame.new_instance('Default', (0, 150, 0), [100, 100, 1])

def test_get_form_pair_from_rule_objects():
    method_name = 'get_form_pair_from_rule_objects'
    try_names = [
        'bad_value_empty', 
        'bad_value_0_frames', 
        'bad_value_1_frame', 
        'bad_value_2_frames_overlapping', 
        'bad_value_3_frames', 
        'points_to_square', 
    ]
    points_a_els, square_b_els, frames = (
        _set_up_for_get_form_pair_from_rule_objects())
    instructions = [
        "[Test: Select nothing]", 
        "[Test: Select els]", 
        "[Test: Select els, 1 frame]", 
        "[Test: Select els, 2 frames overlapping]", 
        "[Test: Select els and square, 3 frames]", 
        "[Test: Select els and square, 2 frames disjoint]", 
    ]
    expected_values = [
        None, 
        None, 
        None, 
        None, 
        None, 
        (points_a_els, square_b_els), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_get_form_pair_from_rule_objects(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_get_form_pair_from_rule_objects():
    _vm.ViewMain.clear_all()
    left, right, overlapping = (
        (  0,  0, 0), 
        (150,  0, 0), 
        ( 80, 80, 0), )
    points_a, square_b = _draw_forms([
        ('pf_points', left), 
        ('pf_square', right), 
    ])
    frames = _draw_frames([left, right, overlapping])
    return (points_a, square_b, frames)

def _draw_forms(pairs):
    forms = [_draw_form(pair[0], pair[1]) for pair in pairs]
    return forms

def _draw_form(name, ref_point):
    py_form = d[name]
    rh_form = py_form.draw('Default')
    start_to_origin = _rs.VectorSubtract(origin, (150, 150, 0))
    origin_to_finish = _rs.VectorSubtract(ref_point, origin)
    t = translation = _rs.VectorAdd(start_to_origin, origin_to_finish)
    els = _rs.MoveObjects(rh_form.els, t)
    return els

def _draw_frames(ref_points):
    frames = []
    for p in ref_points:
        layer_name = 'Default'
        scale = (100, 100, 1)
        f = _fr.Frame.new_instance(layer_name, p, scale)
        frames.append(f)
    return frames

def _try_get_form_pair_from_rule_objects(
    method_name, try_name, instruction, expected_value
):
    obs = _rs.GetObjects(instruction)
    received_value = (
        _rf.RhForm.get_form_pair_from_rule_objects(obs))
    if received_value == None:
        actual_value = None
    else:
        act_form_a, act_form_b = received_value
        ref_point_a = act_form_a.ref_point
        ref_point_b = act_form_b.ref_point
        act_py_form_a = _pf.PyForm.new_from_objects_and_ref_point(
            act_form_a.els, ref_point_a)
        act_py_form_b = _pf.PyForm.new_from_objects_and_ref_point(
            act_form_b.els, ref_point_b)
        actual_value = (act_py_form_a, act_py_form_b)
        exp_form_a_els, exp_form_b_els = expected_value
        exp_py_form_a = _pf.PyForm.new_from_objects_and_ref_point(
            exp_form_a_els, ref_point_a)
        exp_py_form_b = _pf.PyForm.new_from_objects_and_ref_point(
            exp_form_b_els, ref_point_b)
        expected_value = (exp_py_form_a, exp_py_form_b)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_find_least_bounding_box():
    method_name = 'find_least_bounding_box'
    try_names = [
        'empty_list', 
        'pf_empty', 
        'pf_square', 
        'pf_hash', 
        'pff_empty_square_ells_hash', 
    ]
    pf_empty, pf_square, pf_ells, pf_hash = (
        d['pf_empty'], d['pf_square'], d['pf_ells'], d['pf_hash'])
    input_lists = [
        [], 
        [pf_empty], 
        [pf_square], 
        [pf_hash], 
        [pf_empty, pf_square, pf_ells, pf_hash], 
    ]
    expected_values = [
        None, 
        None, 
        [   (170, 170, 0), (230, 170, 0), (230, 230, 0), (170, 230, 0), 
            (170, 170, 0), (230, 170, 0), (230, 230, 0), (170, 230, 0), ], 
        [   (150, 150, 0), (250, 150, 0), (250, 250, 0), (150, 250, 0), 
            (150, 150, 0), (250, 150, 0), (250, 250, 0), (150, 250, 0), ], 
        [   (150, 150, 0), (250, 150, 0), (250, 250, 0), (150, 250, 0), 
            (150, 150, 0), (250, 150, 0), (250, 250, 0), (150, 250, 0), ], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_find_least_bounding_box(
            method_name, try_names[i], input_lists[i], expected_values[i])
    print_test_done(method_name)

def _try_find_least_bounding_box(
    method_name, try_name, py_forms, expected_value
):
    _vm.ViewMain.clear_all()
    els_lists = []
    # forms = []
    for py_form in py_forms:
        rh_form = py_form.draw('Default')
        els_list = rh_form.els
        els_lists.append(els_list)
    result = _rf.RhForm.find_least_bounding_box(els_lists)
    if result == None:
        actual_value = None
    else:
        point_obs = result
        xyzs = [tuple(p) for p in point_obs]
        actual_value = xyzs
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_array():
    method_name = 'array'
    try_names = [
        'bad_value_empty_bounding_points', 
        'empty_list_forms', 
        'square_empty_hash_at_-220', 
    ]
    forms, draw_point, bounding_points = _set_up_array()
    tuples = [
        (forms, draw_point, []), 
        ([], draw_point, bounding_points), 
        (forms, draw_point, bounding_points), 
    ]
    instructions = [
        "[Test: Forms not arrayed?]", 
        "[Test: Forms not arrayed?]", 
        "[Test: Forms arrayed at %s?]" % str(draw_point), 
    ]
    expected_values = [
        None, 
        [], 
        forms, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_array(
            method_name, 
            try_names[i], 
            tuples[i], 
            instructions[i], 
            expected_values[i])
    print_test_done(method_name)

def _set_up_array():
    _vm.ViewMain.clear_all()
    py_forms = [d['pf_ells'], d['pf_empty'], d['pf_square'], d['pf_hash']]
    rh_forms = [pf.draw('Default') for pf in py_forms]
    forms = [rh_form.els for rh_form in rh_forms]
    draw_point = (-200, -200, 0)
    all_obs = []
    for form in forms:
        obs = form
        all_obs.extend(obs)
    bounding_points = _rs.BoundingBox(all_obs)
    return(forms, draw_point, bounding_points)

def _try_array(method_name, try_name, tuple_, instruction, expected_value):
    forms, draw_point, bounding_points = tuple_
    actual_value = _rf.RhForm.array(forms, draw_point, bounding_points)
    _rs.GetString(instruction)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_get_all():
    method_name = 'get_all'
    try_names = [
        '1_default', 
        '1_default_1_rule_0_forms_1_other', 
        '1_default_1_rule_2_forms_1_other', 
    ]
    set_up_methods = [
        '_set_up_default_objects', 
        '_set_up_rule_and_other_objects', 
        '_set_up_forms', 
    ]
    instructions = [
        "[Test: Confirm that nothing is highlighted]", 
        "[Test: Confirm that nothing is highlighted]", 
        "[Test: Confirm that only forms are highlighted]", 
    ]
    expected_values = [
        0, 
        0, 
        2, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_get_all(
            method_name, 
            try_names[i], 
            set_up_methods[i], 
            instructions[i], 
            expected_values[i])
    print_test_done(method_name)

def _set_up_default_objects():
    _vm.ViewMain.clear_all()
    pf_square = d['pf_square']
    pf_square.draw(default_layer_name)

def _set_up_rule_and_other_objects():
    pr_ells_to_square = d['pr_ells_to_square']
    pr_ells_to_square.draw()
    _l.Layer.new('new')
    pf_hash = d['pf_hash']
    rh_hash = pf_hash.draw('new')
    translation = (0, 200, 0)
    _rs.MoveObjects(rh_hash.els, translation)

def _set_up_forms():
    py_form_names = [
        'pf_points', 
        'pf_ells', 
    ]
    i = 1
    for pf_name in py_form_names:
        pf = d[pf_name]
        layer_name = _l.Layer.new_shape_layer()
        rf = pf.draw(layer_name)
        translation = (0, -200 * i, 0)
        _rs.MoveObjects(rf.els, translation)
        i += 1

def _try_get_all(
    method_name, try_name, set_up_method, instruction, expected_value
):
    eval_str = '%s()' % set_up_method
    eval(eval_str)
    actual_result = _rf.RhForm.get_all()
    _rs.GetString(instruction)
    actual_value = len(actual_result)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_enframe():
    method_name = 'enframe'
    try_names = [
        'bad_value_rh_form_ill_formed', 
        'bad_value_diag_x_lt_0', 
        'bad_value_diag_x_eq_0', 
        'bad_value_diag_y_lt_0', 
        'bad_value_diag_y_eq_0', 
        'bad_value_diag_z_lt_0', 
        'bad_value_diag_z_eq_0', 
        'w0_w0_w0', 
        'w0_w0_w1', 
        'w0_w1_w0', 
        'w0_w1_w1', 
        'w1_w0_w0', 
        'w1_w0_w1', 
        'w1_w1_w0', 
        'w1_w1_w1', 
    ]
    rf_empty = _rf.RhForm.new_empty()
    rf_ill_formed = _rf.RhForm(37)
    rh_forms = [
        rf_ill_formed,                  # 'bad_value_rh_form_ill_formed', 
        rf_empty,                       # 'bad_value_diag_x_lt_0', 
        rf_empty,                       # 'bad_value_diag_x_eq_0', 
        rf_empty,                       # 'bad_value_diag_y_lt_0', 
        rf_empty,                       # 'bad_value_diag_y_eq_0', 
        rf_empty,                       # 'bad_value_diag_z_lt_0', 
        rf_empty,                       # 'bad_value_diag_z_eq_0', 
        rf_empty,                       # 'w0_w0_w0', 
        rf_empty,                       # 'w0_w0_w1', 
        rf_empty,                       # 'w0_w1_w0', 
        rf_empty,                       # 'w0_w1_w1', 
        rf_empty,                       # 'w1_w0_w0', 
        rf_empty,                       # 'w1_w0_w1', 
        rf_empty,                       # 'w1_w1_w0', 
        rf_empty,                       # 'w1_w1_w1', 
    ]
    origin = _s.Settings.origin_p3d
    frame_points = [
        origin,                         # 'bad_value_rh_form_ill_formed', 
        origin,                         # 'bad_value_diag_x_lt_0', 
        origin,                         # 'bad_value_diag_x_eq_0', 
        origin,                         # 'bad_value_diag_y_lt_0', 
        origin,                         # 'bad_value_diag_y_eq_0', 
        origin,                         # 'bad_value_diag_z_lt_0', 
        origin,                         # 'bad_value_diag_z_eq_0', 
        origin,                         # 'w0_w0_w0', 
        origin,                         # 'w0_w0_w1', 
        origin,                         # 'w0_w1_w0', 
        origin,                         # 'w0_w1_w1', 
        origin,                         # 'w1_w0_w0', 
        origin,                         # 'w1_w0_w1', 
        origin,                         # 'w1_w1_w0', 
        origin,                         # 'w1_w1_w1', 
    ]
    w0 = 10
    w1 = 90
    frame_diags = [
        _vu.get_point3d((w0, w0, w0)),  # 'bad_value_rh_form_ill_formed', 
        _vu.get_point3d((-7, w0, w0)),  # 'bad_value_diag_x_lt_0', 
        _vu.get_point3d(( 0, w0, w0)),  # 'bad_value_diag_x_eq_0', 
        _vu.get_point3d((w0, -7, w0)),  # 'bad_value_diag_y_lt_0', 
        _vu.get_point3d((w0,  0, w0)),  # 'bad_value_diag_y_eq_0', 
        _vu.get_point3d((w0, w0, -7)),  # 'bad_value_diag_z_lt_0', 
        _vu.get_point3d((w0, w0,  0)),  # 'bad_value_diag_z_eq_0', 
        _vu.get_point3d((w0, w0, w0)),  # 'w0_w0_w0', 
        _vu.get_point3d((w0, w0, w1)),  # 'w0_w0_w1', 
        _vu.get_point3d((w0, w1, w0)),  # 'w0_w1_w0', 
        _vu.get_point3d((w0, w1, w1)),  # 'w0_w1_w1', 
        _vu.get_point3d((w1, w0, w0)),  # 'w1_w0_w0', 
        _vu.get_point3d((w1, w0, w1)),  # 'w1_w0_w1', 
        _vu.get_point3d((w1, w1, w0)),  # 'w1_w1_w0', 
        _vu.get_point3d((w1, w1, w1)),  # 'w1_w1_w1', 
    ]
    layer_name = _s.Settings.default_layer_name
    expected_values = [
        None,                           # 'bad_value_rh_form_ill_formed', 
        None,                           # 'bad_value_diag_x_lt_0', 
        None,                           # 'bad_value_diag_x_eq_0', 
        None,                           # 'bad_value_diag_y_lt_0', 
        None,                           # 'bad_value_diag_y_eq_0', 
        None,                           # 'bad_value_diag_z_lt_0', 
        None,                           # 'bad_value_diag_z_eq_0', 
        True,                           # 'w0_w0_w0', 
        True,                           # 'w0_w0_w1', 
        True,                           # 'w0_w1_w0', 
        True,                           # 'w0_w1_w1', 
        True,                           # 'w1_w0_w0', 
        True,                           # 'w1_w0_w1', 
        True,                           # 'w1_w1_w0', 
        True,                           # 'w1_w1_w1', 
    ]
    n = len(try_names)
    for i in range(n):
        _try_enframe(
            method_name, 
            try_names[i], 
            rh_forms[i], 
            frame_points[i], 
            frame_diags[i], 
            layer_name, 
            expected_values[i])
    print_test_done(method_name)

def _try_enframe(
    method_name, 
    try_name, 
    rh_form, 
    frame_point, 
    frame_diag, 
    layer_name, 
    expected_value
):
    _vm.ViewMain.clear_all()
    received_value = rh_form.enframe(frame_point, frame_diag, layer_name)
    if received_value == None:
        actual_value = received_value
    elif _rs.ObjectType(received_value) == _s.Settings.block_type:
        frame_instance = received_value
        _rs.SelectObject(frame_instance)
        message = "Check that the frame instance was added"
        _rs.GetString(message)
        actual_value = True
    else:
        message = "Shouldn't have gotten here"
        print(message)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_move_to():
    method_name = 'move_to'
    try_names = [
        'bad_value_ill_formed', 
        'empty', 
        'point', 
        'xy', 
        'xz', 
        'yz', 
        'xyz', 
    ]
    pf_point = e['pf_point']
    pf_xy = e['pf_xy']
    pf_xz = e['pf_xz']
    pf_yz = e['pf_yz']
    pf_xyz = e['pf_xyz']
    _vm.ViewMain.clear_all()
    ob_lists = [
        37, 
        [], 
        pf_point.draw(default_layer_name).els, 
        pf_xy.draw(default_layer_name).els, 
        pf_xz.draw(default_layer_name).els, 
        pf_yz.draw(default_layer_name).els, 
        pf_xyz.draw(default_layer_name).els, 
    ]
    rh_forms = []
    # frame_instance = _fr.Frame.new_instance(
    #     default_layer_name, origin, (100, 100, 100))
    for obs in ob_lists:
        rh_form = _rf.RhForm(obs)
        rh_form.frame = _fr.Frame.new_instance(
            default_layer_name, origin, (100, 100, 100))
        rh_forms.append(rh_form)
    origin_p3d = _s.Settings.origin_p3d
    target_points = [
        _vu.get_point3d((-10, -10, -10)), 
        _vu.get_point3d((-20, -20, -20)), 
        _vu.get_point3d((-30, -30, -30)), 
        _vu.get_point3d((-40, -40, -40)), 
        _vu.get_point3d((-50, -50, -50)), 
        _vu.get_point3d((-60, -60, -60)), 
        _vu.get_point3d((-70, -70, -70)), 
    ]
    expected_values = [
        None, 
        target_points[1], 
        target_points[2], 
        target_points[3], 
        target_points[4], 
        target_points[5], 
        target_points[6], 
    ]
    n = len(try_names)
    for i in range(n):
        _try_move_to(
            method_name, 
            try_names[i], 
            rh_forms[i], 
            target_points[i], 
            expected_values[i])
    print_test_done(method_name)

def _try_move_to(
    method_name, try_name, rh_form, target_point, expected_value
):
    actual_value = rh_form.move_to(target_point)
    if not expected_value == None:
        message = "[Test: Confirm that the form was moved]"
        _rs.GetString(message)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_get_frame_position():
    method_name = 'get_frame_position'
    try_names = [
        'bad_value_self_ill_formed', 
        'no_frame', 
        'empty', 
        'empty_moved', 
        'point', 
        'point_moved', 
        'xyz_line', 
        'xyz_line_moved', 
    ]
    rh_forms = _make_rh_forms_for_get_frame_position(try_names)
    expected_values = [
        None,                                   # 'bad_value_self_ill_formed', 
        None,                                   # 'no_frame', 
        origin_p3d,                             # 'empty', 
        _vu.get_point3d((-50, -100, -150)),     # 'empty_moved', 
        origin_p3d,                             # 'point', 
        _vu.get_point3d((-50, -100, -150)),     # 'p_moved', 
        origin_p3d,                             # 'line', 
        _vu.get_point3d((-50, -100, -150)),     # 'line_moved', 
    ]
    n = len(try_names)
    for i in range(n):
        _try_get_frame_position(
            method_name, 
            try_names[i], 
            rh_forms[i], 
            expected_values[i])
    print_test_done(method_name)

def _make_rh_forms_for_get_frame_position(try_names):
    _vm.ViewMain.clear_all()
    frame_point = origin_p3d
    frame_diag = _vu.get_point3d((100, 100, 100))
    translation = (-50, -100, -150)
    for name in try_names:
        _rs.AddLayer(name)

    _rs.CurrentLayer(try_names[0])
    rf_ill_formed = _rf.RhForm(37)
    
    _rs.CurrentLayer(try_names[1])
    p_no_frame = _rs.AddTextDot('p_no_frame', (90, 90, 90))
    rf_no_frame = _rf.RhForm([p_no_frame])

    _rs.CurrentLayer(try_names[2])
    rf_empty = _rf.RhForm([])
    rf_empty.enframe(frame_point, frame_diag, _rs.CurrentLayer())

    _rs.CurrentLayer(try_names[3])
    rf_empty_moved = _rf.RhForm([])
    rf_empty_moved.enframe(frame_point, frame_diag, _rs.CurrentLayer())
    _rs.MoveObject(rf_empty_moved.frame, translation)

    _rs.CurrentLayer(try_names[4])
    point = _rs.AddTextDot('p', (90, 90, 90))
    rf_point = _rf.RhForm([point])
    rf_point.enframe(frame_point, frame_diag, _rs.CurrentLayer())
    
    _rs.CurrentLayer(try_names[5])
    p_moved = _rs.AddTextDot('p_moved', (90, 90, 90))
    rf_p_moved = _rf.RhForm([p_moved])
    rf_p_moved.enframe(frame_point, frame_diag, _rs.CurrentLayer())
    _rs.MoveObject(rf_p_moved.frame, translation)
    _rs.MoveObjects(rf_p_moved.els, translation)
    
    _rs.CurrentLayer(try_names[6])
    line = _rs.AddLine((10, 10, 10), (90, 90, 90))
    rf_line = _rf.RhForm([line])
    rf_line.enframe(frame_point, frame_diag, _rs.CurrentLayer())
    
    _rs.CurrentLayer(try_names[7])
    line_moved = _rs.AddLine((10, 10, 10), (90, 90, 90))
    rf_line_moved = _rf.RhForm([line_moved])
    rf_line_moved.enframe(frame_point, frame_diag, _rs.CurrentLayer())
    _rs.MoveObject(rf_line_moved.frame, translation)
    _rs.MoveObjects(rf_line_moved.els, translation)
    
    _rs.CurrentLayer(default_layer_name)

    rh_forms = [
        rf_ill_formed,                          # 'bad_value_self_ill_formed', 
        rf_no_frame,                            # 'no_frame', 
        rf_empty,                               # 'empty', 
        rf_empty_moved,                         # 'empty_moved', 
        rf_point,                               # 'point', 
        rf_p_moved,                             # 'p_moved', 
        rf_line,                                # 'line', 
        rf_line_moved,                          # 'line_moved', 
    ]
    return rh_forms

def _try_get_frame_position(method_name, try_name, rh_form, expected_value):
    actual_value = rh_form.get_frame_position()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___eq__():
    method_name = '__eq__'
    names = [
        # l: line; d: text dot; p: ref point
        '__p0', 
        '__p1', 
        '_dp0', 
        '_dp1', 
        'l_p0', 
        'l_p1', 
        'ldp0', 
        'ldp1', 
    ]
    forms, names = _make_forms_and_names_for_eq_and_ne()
    expected_values = [
        # __p0
        [True , True , False, False, False, False, False, False], 
        # __p1
        [True , True , False, False, False, False, False, False], 
        # _dp0
        [False, False, True , False, False, False, False, False], 
        # _dp1
        [False, False, False, True , False, False, False, False], 
        # l_p0
        [False, False, False, False, True , False, False, False], 
        # l_p1
        [False, False, False, False, False, True , False, False], 
        # ldp0
        [False, False, False, False, False, False, True , False], 
        # ldp1
        [False, False, False, False, False, False, False, True ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            name_pair = '%s, %s' % (names[i], names[j])
            form_pair = (forms[i], forms[j])
            _try___eq__(
                method_name, name_pair, form_pair, expected_values[i][j])
    print_test_done(method_name)

def _try___eq__(method_name, try_name, form_pair, expected_value):
    f1, f2 = form_pair
    actual_value = (f1 == f2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test___ne__():
    method_name = '__ne__'
    names = [
        # l: line; d: text dot; p: ref point
        '__p0', 
        '__p1', 
        '_dp0', 
        '_dp1', 
        'l_p0', 
        'l_p1', 
        'ldp0', 
        'ldp1', 
    ]
    forms, names = _make_forms_and_names_for_eq_and_ne()
    expected_values = [
        # __p0
        [False, False, True , True , True , True , True , True ], 
        # __p1
        [False, False, True , True , True , True , True , True ], 
        # _dp0
        [True , True , False, True , True , True , True , True ], 
        # _dp1
        [True , True , True , False, True , True , True , True ], 
        # l_p0
        [True , True , True , True , False, True , True , True ], 
        # l_p1
        [True , True , True , True , True , False, True , True ], 
        # ldp0
        [True , True , True , True , True , True , False, True ], 
        # ldp1
        [True , True , True , True , True , True , True , False], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            name_pair = '%s, %s' % (names[i], names[j])
            form_pair = (forms[i], forms[j])
            _try___ne__(
                method_name, name_pair, form_pair, expected_values[i][j])
    print_test_done(method_name)

def _try___ne__(method_name, try_name, form_pair, expected_value):
    f1, f2 = form_pair
    actual_value = (f1 != f2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def _make_forms_and_names_for_eq_and_ne():
    _vm.ViewMain.clear_all()
    p0 = origin_p3d
    p1 = _rs.PointAdd([50, 0, 0], p0)
    l = line = _rs.AddLine([0, 0, 0], [20, 20, 0])
    d = text_dot = _rs.AddTextDot('x', [10, 10, 0])
    __p0 = _rf.RhForm([    ], p0)
    __p1 = _rf.RhForm([    ], p1)
    _dp0 = _rf.RhForm([   d], p0)
    _dp1 = _rf.RhForm([   d], p1)
    l_p0 = _rf.RhForm([l   ], p0)
    l_p1 = _rf.RhForm([l   ], p1)
    ldp0 = _rf.RhForm([l, d], p0)
    ldp1 = _rf.RhForm([l, d], p1)
    forms = [ __p0 ,  __p1 ,  _dp0 ,  _dp1 ,  l_p0 ,  l_p1 ,  ldp0 ,  ldp1 ]
    names = ['__p0', '__p1', '_dp0', '_dp1', 'l_p0', 'l_p1', 'ldp0', 'ldp1']
    return (forms, names)

def test_guid_ordering():
    # Guids can be ordered
    obs = []
    labels = ['a', 'b', 'c', 'd']
    xyz0, xyz1, xyz2, xyz3 = [
        (0.0, 0.0, 0.0), 
        (1.1, 1.1, 1.1), 
        (2.2, 2.2, 2.2), 
        (3.3, 3.3, 3.3), 
    ]
    xyzs = [xyz0, xyz2]
    xyz_pairs = [
        (xyz0, xyz1), 
        (xyz2, xyz3), 
    ]
    n = len(xyzs)
    for i in range(n):
        p = _rs.AddTextDot(labels[i], xyzs[i])
        obs.append(p)
        (p0, p1) = xyz_pairs[i]
        l = _rs.AddLine(p0, p1)
        obs.append(l)
    ordered_obs = sorted(obs)
    print("Done: test_guid_ordering")

test___init__()
test_is_a_form()
test_get_form_pair_from_rule_objects()
test_find_least_bounding_box()
test_array()
test_get_all()
test_enframe()
test_move_to()
test_get_frame_position()
test___eq__()
test___ne__()

# test_guid_ordering()


