from view import frame as                       _fr
from view import layer as                       _l
from view import py_rule as                     _pr
from view import rh_form as                     _rf
from view import rh_rule as                     _rr
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm
from view import view_utilities as              _vu

print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message
default_layer_name = _s.Settings.default_layer_name
d = _tu.make_hash_suite_dict()
e = _tu.make_element_dict()

def test___init__():
    method_name = '__init__'
    try_names = [
        'bad_type_a', 
        'bad_type_b', 
        'bad_type_layer_name', 
        'bad_value_layer_name', 
        'empty_empty', 
        'empty_point', 
        'empty_xyz', 
        'point_empty', 
        'point_point', 
        'point_xyz', 
        'xyz_empty', 
        'xyz_point', 
        'xyz_xyz', 
    ]
    rf_empty, rf_point, rf_xyz = _set_up_for___init__()
    rh_form_triples = [
        (37, rf_xyz, default_layer_name), 
        (rf_xyz, 37, default_layer_name), 
        (rf_empty, rf_empty, 37), 
        (rf_empty, rf_empty, 'kilroy'), 
        (rf_empty, rf_empty, default_layer_name), 
        (rf_empty, rf_point, default_layer_name), 
        (rf_empty, rf_xyz, default_layer_name), 
        (rf_point, rf_empty, default_layer_name), 
        (rf_point, rf_point, default_layer_name), 
        (rf_point, rf_xyz, default_layer_name), 
        (rf_xyz, rf_empty, default_layer_name), 
        (rf_xyz, rf_point, default_layer_name), 
        (rf_xyz, rf_xyz, default_layer_name), 
    ]
    expected_values = [
        False, 
        False, 
        False, 
        False, 
        True, 
        True, 
        True, 
        True, 
        True, 
        True, 
        True, 
        True, 
        True, 
    ]
    n = len(try_names)
    for i in range(n):
        _try___init__(
            method_name, 
            try_names[i], 
            rh_form_triples[i], 
            expected_values[i])
    print_test_done(method_name)

def _set_up_for___init__():
    _vm.ViewMain.clear_all()
    pf_point = e['pf_point']
    pf_xyz = e['pf_xyz']
    rf_point = pf_point.draw(default_layer_name)
    rf_xyz = pf_xyz.draw(default_layer_name)
    rf_empty = _rf.RhForm()
    return (rf_empty, rf_point, rf_xyz)

def _try___init__(method_name, try_name, rh_form_triple, expected_value):
    rh_form_a, rh_form_b, layer_name = rh_form_triple
    actual_output = _rr.RhRule(rh_form_a, rh_form_b, layer_name)
    actual_value = actual_output.is_well_formed
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__find_common_bounding_points():
    method_name = '_find_common_bounding_points'
    try_names = [
        'empty_empty', 
        'point_empty', 
        'point_point', 
        'xy_empty', 
        'xy_point', 
        'xy_xy', 
        'xz_empty', 
        'xz_point', 
        'xz_xy', 
        'xz_xz', 
        'yz_empty', 
        'yz_point', 
        'yz_xy', 
        'yz_xz', 
        'yz_yz', 
        'xyz_empty', 
        'xyz_point', 
        'xyz_xy', 
        'xyz_xz', 
        'xyz_yz', 
        'xyz_xyz', 
    ]
    _vm.ViewMain.clear_all()
    rf_empty, rf_point, rf_xy, rf_xz, rf_yz, rf_xyz = (
        _make_rh_forms_for__find_common_bounding_points())
    rh_form_pairs = [
        (rf_empty, rf_empty), 
        (rf_point, rf_empty), 
        (rf_point, rf_point), 
        (rf_xy, rf_empty), 
        (rf_xy, rf_point), 
        (rf_xy, rf_xy), 
        (rf_xz, rf_empty), 
        (rf_xz, rf_point), 
        (rf_xz, rf_xy), 
        (rf_xz, rf_xz), 
        (rf_yz, rf_empty), 
        (rf_yz, rf_point), 
        (rf_yz, rf_xy), 
        (rf_yz, rf_xz), 
        (rf_yz, rf_yz), 
        (rf_xyz, rf_empty), 
        (rf_xyz, rf_point), 
        (rf_xyz, rf_xy), 
        (rf_xyz, rf_xz), 
        (rf_xyz, rf_yz), 
        (rf_xyz, rf_xyz), 
    ]
    bpp_empty, bpp_point, bpp_xy, bpp_xz, bpp_yz, bpp_xyz = (
        _set_up_for__find_common_bounding_points())
    expected_values = [
        bpp_empty, 
        bpp_point, 
        bpp_point, 
        bpp_xy, 
        bpp_xyz, 
        bpp_xy, 
        bpp_xz, 
        bpp_xyz, 
        bpp_xyz, 
        bpp_xz, 
        bpp_yz, 
        bpp_xyz, 
        bpp_xyz, 
        bpp_xyz, 
        bpp_yz, 
        bpp_xyz, 
        bpp_xyz, 
        bpp_xyz, 
        bpp_xyz, 
        bpp_xyz, 
        bpp_xyz, 
    ]
    n = len(try_names)
    for i in range(n):
        _try__find_common_bounding_points(
            method_name, try_names[i], rh_form_pairs[i], expected_values[i])
    print_test_done(method_name)

def _make_rh_forms_for__find_common_bounding_points():
    pf_names = [
        'pf_empty', 
        'pf_point', 
        'pf_xy', 
        'pf_xz', 
        'pf_yz', 
        'pf_xyz', 
    ]
    rh_forms = []
    for pf_name in pf_names:
        py_form = e[pf_name]
        rh_form = py_form.draw(default_layer_name)
        rh_forms.append(rh_form)
    return rh_forms

def _set_up_for__find_common_bounding_points():
    xyz_names = [
        'xyz_000', 
        'xyz_001', 
        'xyz_010', 
        'xyz_011', 
        'xyz_100', 
        'xyz_101', 
        'xyz_110', 
        'xyz_111', 
    ]
    p3d_obs = []
    for xyz_name in xyz_names:
        xyz = e[xyz_name]
        p3d_ob = _vu.get_point3d(xyz)
        p3d_obs.append(p3d_ob)
    p3d_000, p3d_001, p3d_010, p3d_011, p3d_100, p3d_101, p3d_110, p3d_111 = (
        p3d_obs)
    bpp_empty = []
    bpp_point = [
        p3d_111, p3d_111, p3d_111, p3d_111, 
        p3d_111, p3d_111, p3d_111, p3d_111, ]
    bpp_xy = [
        p3d_000, p3d_100, p3d_110, p3d_010, 
        p3d_000, p3d_100, p3d_110, p3d_010, ]
    bpp_xz = [
        p3d_000, p3d_100, p3d_100, p3d_000, 
        p3d_001, p3d_101, p3d_101, p3d_001, ]
    bpp_yz = [
        p3d_000, p3d_000, p3d_010, p3d_010, 
        p3d_001, p3d_001, p3d_011, p3d_011, ]
    bpp_xyz = [
        p3d_000, p3d_100, p3d_110, p3d_010, 
        p3d_001, p3d_101, p3d_111, p3d_011, ]
    return (bpp_empty, bpp_point, bpp_xy, bpp_xz, bpp_yz, bpp_xyz)

def _try__find_common_bounding_points(
    method_name, try_name, rh_form_pair, expected_value
):
    rh_form_a, rh_form_b = rh_form_pair
    actual_value = _rr.RhRule._find_common_bounding_points(
        rh_form_a, rh_form_b)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__find_padded_frame_specs():
    method_name = '_find_padded_frame_specs'
    try_names = [
        'empty', 
        'point', 
        'x', 
        'y', 
        'z', 
        'xy', 
        'xz', 
        'yz', 
        'xyz', 
    ]
    xyz_000, xyz_001, xyz_010, xyz_011, xyz_100, xyz_101, xyz_110, xyz_111 = [
        e[point_name] for point_name in [
            'xyz_000', 'xyz_001', 'xyz_010', 'xyz_011', 
            'xyz_100', 'xyz_101', 'xyz_110', 'xyz_111', ]]
    bounding_point_lists = [
        [], 
        [   xyz_111, xyz_111, xyz_111, xyz_111, 
            xyz_111, xyz_111, xyz_111, xyz_111, ], #  'point', 
        [   xyz_000, xyz_100, xyz_100, xyz_000, 
            xyz_000, xyz_100, xyz_100, xyz_000, ], #  'x', 
        [   xyz_000, xyz_000, xyz_010, xyz_010, 
            xyz_000, xyz_000, xyz_010, xyz_010, ], #  'y', 
        [   xyz_000, xyz_000, xyz_000, xyz_000, 
            xyz_001, xyz_001, xyz_001, xyz_001, ], #  'z', 
        [   xyz_000, xyz_100, xyz_110, xyz_010, 
            xyz_000, xyz_100, xyz_110, xyz_010, ], #  'xy', 
        [   xyz_000, xyz_100, xyz_100, xyz_000, 
            xyz_001, xyz_101, xyz_101, xyz_001, ], #  'xz', 
        [   xyz_000, xyz_000, xyz_010, xyz_010, 
            xyz_001, xyz_001, xyz_011, xyz_011, ], #  'yz', 
        [   xyz_000, xyz_100, xyz_110, xyz_010, 
            xyz_001, xyz_101, xyz_111, xyz_011, ], #  'xyz', 
    ]
    w0, w1, dw, p0, p1 = _make_coords_for__find_padded_frame_specs()
    expected_values = [
        # (point, diagonal), 
        (   (   ( 0 - p0), 
                ( 0 - p0), 
                ( 0 -  0)), 
            (   ( 0 + p0 + p0), 
                ( 0 + p0 + p0), 
                ( 0 +  0 + p0))), # 'empty', 
        (   (   (w1 - p0), 
                (w1 - p0), 
                (w1 -  0)), 
            (   ( 0 + p0 + p0), 
                ( 0 + p0 + p0), 
                ( 0 +  0 + p0))), # 'point', 
        (   (   (w0 - p1), 
                (w0 - p0), 
                (w0 -  0)), 
            (   (dw + p1 + p1), 
                ( 0 + p0 + p0), 
                ( 0 +  0 + p0))), # 'x', 
        (   (   (w0 - p0), 
                (w0 - p1), 
                (w0 -  0)), 
            (   ( 0 + p0 + p0), 
                (dw + p1 + p1), 
                ( 0 +  0 + p0))), # 'y', 
        (   (   (w0 - p0), 
                (w0 - p0), 
                (w0 -  0)), 
            (   ( 0 + p0 + p0), 
                ( 0 + p0 + p0), 
                (dw +  0 + p1))), # 'z', 
        (   (   (w0 - p1), 
                (w0 - p1), 
                (w0 -  0)), 
            (   (dw + p1 + p1), 
                (dw + p1 + p1), 
                ( 0 +  0 + p0))), # 'xy', 
        (   (   (w0 - p1), 
                (w0 - p0), 
                (w0 -  0)), 
            (   (dw + p1 + p1), 
                ( 0 + p0 + p0), 
                (dw +  0 + p1))), # 'xz', 
        (   (   (w0 - p0), 
                (w0 - p1), 
                (w0 -  0)), 
            (   ( 0 + p0 + p0), 
                (dw + p1 + p1), 
                (dw +  0 + p1))), # 'yz', 
        (   (   (w0 - p1), 
                (w0 - p1), 
                (w0 -  0)), 
            (   (dw + p1 + p1), 
                (dw + p1 + p1), 
                (dw +  0 + p1))), # 'xyz', 
    ]
    n = len(try_names)
    for i in range(n):
        _try__find_padded_frame_specs(
            method_name, 
            try_names[i], 
            bounding_point_lists[i], 
            expected_values[i], 
        )
    print_test_done(method_name)

def _make_coords_for__find_padded_frame_specs():
    w0 = e['w0']
    w1 = e['w1']
    dw = w1 - w0
    pf = _s.Settings.frame_padding_factor
    p0 = _s.Settings.frame_pad_minimum
    p1 = dw * pf
    return (w0, w1, dw, p0, p1)

def _try__find_padded_frame_specs(
    method_name, try_name, bounding_points, expected_value
):
    actual_point_p3d, actual_diag_p3d = (
        _rr.RhRule._find_padded_frame_specs(bounding_points))
    actual_point_xyz = tuple(actual_point_p3d)
    actual_diag_xyz = tuple(actual_diag_p3d)
    actual_value = (actual_point_xyz, actual_diag_xyz)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_new_from_object_lists():
    method_name = 'new_from_object_lists'
    try_names = [
        'bad_type_a', 
        'bad_type_b', 
        'bad_type_ab', 
        'empty_empty', 
        'empty_point', 
        'empty_xyz', 
        'point_empty', 
        'point_point', 
        'point_xyz', 
        'xyz_empty', 
        'xyz_point', 
        'xyz_xyz', 
    ]
    point, xyz = _set_up_for_new_from_object_lists()
    input_triples = [
        (37, [point], default_layer_name), 
        ([point], 37, default_layer_name), 
        (37, 37, default_layer_name), 
        ([], [], default_layer_name), 
        ([], [point], default_layer_name), 
        ([], [xyz], default_layer_name), 
        ([point], [], default_layer_name), 
        ([point], [point], default_layer_name), 
        ([point], [xyz], default_layer_name), 
        ([xyz], [], default_layer_name), 
        ([xyz], [point], default_layer_name), 
        ([xyz], [xyz], default_layer_name), 
    ]
    expected_values = [
        None, 
        None, 
        None, 
        True, 
        True, 
        True, 
        True, 
        True, 
        True, 
        True, 
        True, 
        True, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_new_from_object_lists(
            method_name, try_names[i], input_triples[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_new_from_object_lists():
    _vm.ViewMain.clear_all()
    pf_point = e['pf_point']
    pf_xyz = e['pf_xyz']
    rf_point = pf_point.draw(default_layer_name)
    rf_xyz = pf_xyz.draw(default_layer_name)
    return (rf_point.els, rf_xyz.els)

def _try_new_from_object_lists(
    method_name, try_name, input_triple, expected_value
):
    obs_a, obs_b, layer_name = input_triple
    actual_output = _rr.RhRule.new_from_object_lists(obs_a, obs_b, layer_name)
    if type(actual_output) == _rr.RhRule:
        actual_value = actual_output.is_well_formed
    else:
        actual_value = None
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_is_a_rule():
    method_name = 'is_a_rule'
    try_names = [
        # 'bad_state_no_group', 
        '0_obs_0_frames', 
        '0_obs_1_frame', 
        '0_obs_2_frames_overlapping', 
        '0_obs_2_frames_disjoint', 
        '0_obs_3_frames', 

        '1_ob_0_frames', 
        '1_ob_1_frame', 
        '1_ob_2_frames_overlapping', 
        '1_ob_2_frames_disjoint', 
        '1_ob_3_frames', 

        '2_obs_0_frames', 
        '2_obs_1_frame', 
        '2_obs_2_frames_overlapping', 
        '2_obs_2_frames_disjoint', 
        '2_obs_3_frames', 
    ]
    ob1, ob2, fr1, fr2a, fr2b, fr3 = _set_up_for_is_a_rule()
    input_tuples = [
        [], 
        [fr1], 
        [fr2a, fr2b], 
        [fr1, fr2a], 
        [fr1, fr2a, fr3], 
        
        [ob1, ], 
        [ob1, fr1], 
        [ob1, fr2a, fr2b], 
        [ob1, fr1, fr2a], 
        [ob1, fr1, fr2a, fr3], 
        
        [ob1, ob2, ], 
        [ob1, ob2, fr1], 
        [ob1, ob2, fr2a, fr2b], 
        [ob1, ob2, fr1, fr2a], 
        [ob1, ob2, fr1, fr2a, fr3], 
    ]
    expected_values = [
        False, 
        False, 
        False, 
        True, 
        False, 

        False, 
        False, 
        False, 
        True, 
        False, 

        False, 
        False, 
        False, 
        True, 
        False, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_is_a_rule(
            method_name, try_names[i], input_tuples[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_is_a_rule():
    p000, p110, p220, p300, p310, p330, p600 = [
        ( 0,  0, 0),
        (10, 10, 0),
        (20, 20, 0),
        (30,  0, 0),
        (30, 10, 0),
        (30, 30, 0),
        (60,  0, 0),
    ]
    scale = (20, 20, 1)
    ob1 = point_1 = _rs.AddTextDot('a', p110)
    ob2 = line_2 = _rs.AddLine(p220, p330)
    # default_layer_name = _s.Settings.default_layer_name
    fr1 = _fr.Frame.new_instance(default_layer_name, p000, scale)
    fr2a = _fr.Frame.new_instance(default_layer_name, p300, scale)
    fr2b = _fr.Frame.new_instance(default_layer_name, p310, scale)
    fr3 = _fr.Frame.new_instance(default_layer_name, p600, scale)
    obs_and_frames = (ob1, ob2, fr1, fr2a, fr2b, fr3)
    _rs.AddGroup(default_layer_name)
    _rs.AddObjectsToGroup(obs_and_frames, default_layer_name)
    return obs_and_frames

def _try_is_a_rule(method_name, try_name, obs, expected_value):
    actual_value = _rr.RhRule.is_a_rule(obs)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_get_objects():
    method_name = 'get_objects'
    try_names = [
        'no_rule_layers', 
        'empty_rule_layer', 
        'above_plus_points_to_empty', 
        'above_plus_square_hash', 
    ]
    setups = [
        '_vm.ViewMain.clear_all()', 
        '_l.Layer.new("11111")', 
        '_pr.PyRule.draw(d["pr_points_to_empty"])', 
        '_pr.PyRule.draw(d["pr_square_to_hash"])', 
    ]
    message = 'Select rules'
    n = len(try_names)
    for i in range(n):
        _try_get_objects(method_name, try_names[i], setups[i], message)
    print_test_done(method_name)

def _try_get_objects(method_name, try_name, setup, message):
    eval(setup)
    actual_objects = _rr.RhRule.get_objects()
    actual_value = len(actual_objects)
    expected_objects = _rs.GetObjects(message)
    if expected_objects == None:
        expected_objects = []
    expected_value = len(expected_objects)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_find_frame_dy():
    method_name = 'find_frame_dy'
    try_names = [
        'rr_ill_formed', 
        'rr_point_empty', 
        'rr_point_point', 
        'rr_hash_hash', 
        'rr_xyz_xyz', 
    ]
    rh_rules = _set_up_for_find_frame_dy()
    expected_values = [
        None, 
        20, 
        20, 
        140, 
        112, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_find_frame_y(
            method_name, try_names[i], rh_rules[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_find_frame_dy():
    _vm.ViewMain.clear_all()
    rr_ill_formed = _rr.RhRule(17, 19, 'ill_formed')
    rr_point_empty = e['pr_point_empty'].draw()
    rr_point_point = e['pr_point_point'].draw()
    rr_hash_hash = d['pr_hash_to_hash'].draw()
    rr_xyz_xyz = e['pr_xyz_xyz'].draw()
    rh_rules = [
        rr_ill_formed, 
        rr_point_empty, 
        rr_point_point, 
        rr_hash_hash, 
        rr_xyz_xyz, 
    ]
    return rh_rules

def _try_find_frame_y(method_name, try_name, rh_rule, expected_value):
    actual_value = rh_rule.find_frame_dy()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_arrange():
    method_name = 'arrange'
    try_names = [
        'bad_value_ill_formed', 
        'empty_empty', 
        'empty_point', 
        'empty_xyz', 
        'point_empty', 
        'point_point', 
        'point_xyz', 
        'xyz_empty', 
        'xyz_point', 
        'xyz_xyz', 
    ]
    rh_rules = _make_rh_rules_for_arrange()
    rule_display_point = _s.Settings.rule_display_point
    padded_frame_empty_diag = [20, 20, 10]
    padded_frame_point_diag = [20, 20, 10]
    padded_frame_xyz_diag = [112, 112, 96]
    expected_values = [
        None, 
        _vu.get_point3d((- 40,  20, 0)), 
        _vu.get_point3d((- 40,  20, 0)), 
        _vu.get_point3d((-224, 112, 0)), 
        _vu.get_point3d((- 40,  20, 0)), 
        _vu.get_point3d((- 40,  20, 0)), 
        _vu.get_point3d((-224, 112, 0)), 
        _vu.get_point3d((-224, 112, 0)), 
        _vu.get_point3d((-224, 112, 0)), 
        _vu.get_point3d((-224, 112, 0)), 
    ]
    n = len(try_names)
    for i in range(n):
        _try_arrange(
            method_name, 
            try_names[i], 
            rh_rules[i], 
            expected_values[i])
    print_test_done(method_name)

def _make_rh_rules_for_arrange():
    _vm.ViewMain.clear_all()
    pf_point = e['pf_point']
    pf_xyz = e['pf_xyz']
    rf_point = pf_point.draw(default_layer_name)
    rf_xyz = pf_xyz.draw(default_layer_name)
    rf_empty = _rf.RhForm.new_empty()

    text_dots, xyz_lines = _make_dots_and_lines_for_arrange()
    layer_names = [
        'ill_formed', 
        'empty_empty', 
        'empty_point', 
        'empty_xyz', 
        'point_empty', 
        'point_point', 
        'point_xyz', 
        'xyz_empty', 
        'xyz_point', 
        'xyz_xyz', 
    ]
    for name in layer_names:
        _rs.AddLayer(name)
    rr_ill_formed = _rr.RhRule(17, 19, 'ill_formed')
    rr_empty_empty = _rr.RhRule.new_from_object_lists(
        [], [], 'empty_empty')
    rr_empty_point = _rr.RhRule.new_from_object_lists(
        [], [text_dots[0]], 'empty_point')
    rr_empty_xyz = _rr.RhRule.new_from_object_lists(
        [], [xyz_lines[0]], 'empty_xyz')
    rr_point_empty = _rr.RhRule.new_from_object_lists(
        [text_dots[1]], [], 'point_empty')
    rr_point_point = _rr.RhRule.new_from_object_lists(
        [text_dots[2]], [text_dots[3]], 'point_point')
    rr_point_xyz = _rr.RhRule.new_from_object_lists(
        [text_dots[4]], [xyz_lines[1]], 'point_xyz')
    rr_xyz_empty = _rr.RhRule.new_from_object_lists(
        [xyz_lines[2]], [], 'xyz_empty')
    rr_xyz_point = _rr.RhRule.new_from_object_lists(
        [xyz_lines[3]], [text_dots[5]], 'xyz_point')
    rr_xyz_xyz = _rr.RhRule.new_from_object_lists(
        [xyz_lines[4]], [xyz_lines[5]], 'xyz_xyz')

    rh_rules = [
        rr_ill_formed, 
        rr_empty_empty, 
        rr_empty_point, 
        rr_empty_xyz, 
        rr_point_empty, 
        rr_point_point, 
        rr_point_xyz, 
        rr_xyz_empty, 
        rr_xyz_point, 
        rr_xyz_xyz, 
    ]
    return rh_rules

def _make_dots_and_lines_for_arrange():
    _rs.CurrentLayer(default_layer_name)
    text_dots = []
    for i in range(6):
        text_dot = _rs.AddTextDot('a', (90, 90, 90))
        text_dots.append(text_dot)
    xyz_lines = []
    for i in range(6):
        xyz_line = _rs.AddLine((10, 10, 10), (90, 90, 90))
        xyz_lines.append(xyz_line)
    return (text_dots, xyz_lines)

def _try_arrange(method_name, try_name, rh_rule, expected_value):
    actual_value = rh_rule.arrange()
    if rh_rule.is_well_formed:
        rh_rule.select()
    message = "[Test: %s. %s %s]" % (
        try_name, 
        "Check the location of the rule.", 
        "Select the rule group.", 
    )
    _rs.GetObjects(message, 0)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)
    selected_obs = _rs.SelectedObjects()
    _rs.UnselectObjects(selected_obs)

def test__find_rule_part_positions():
    method_name = '_find_rule_part_positions'
    try_names = [
        'bad_value_ill_formed', 
        'empty_empty', 
        'empty_point', 
        'empty_xyz', 
        'point_empty', 
        'point_point', 
        'point_xyz', 
        'xyz_empty', 
        'xyz_point', 
        'xyz_xyz', 
    ]
    _vm.ViewMain.clear_all()
    rh_rules = _make_rh_rules_for_arrange()
    f0_dx = small_frame_dx = 10.0 * (1 + 0 + 1)       #  20
    f1_dx = large_frame_dx = 80.0 * (0.2 + 1 + 0.2)   # 112
    f0_dy = small_frame_dy = f0_dx
    f1_dy = small_frame_dy = f1_dx
    half_f0_dy = half_small_frame_dy = f0_dx / 2
    half_f1_dy = half_large_frame_dy = f1_dx / 2
    expected_values = [
        None,                                   ##  'bad_value_ill_formed', 
        (   _vu.get_point3d((
                f0_dx * -4, 
                f0_dy, 
                0)), 
            _vu.get_point3d((
                f0_dx * -2.5, 
                f0_dy * 1.5, 
                0)), 
            _vu.get_point3d((
                f0_dx * -2, 
                f0_dy, 
                0)), 
        ),                                      ##  'empty_empty', 
        (   _vu.get_point3d((
                f0_dx * -4, 
                f0_dy, 
                0)), 
            _vu.get_point3d((
                f0_dx * -2.5, 
                f0_dy * 1.5, 
                0)), 
            _vu.get_point3d((
                f0_dx * -2, 
                f0_dy, 
                0)), 
        ),                                      ##  'empty_point', 
        (   _vu.get_point3d((
                f1_dx * -4, 
                f1_dy, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2.5, 
                f1_dy * 1.5, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2, 
                f1_dy, 
                0)), 
        ),                                      ##  'empty_xyz', 
        (   _vu.get_point3d((
                f0_dx * -4, 
                f0_dy, 
                0)), 
            _vu.get_point3d((
                f0_dx * -2.5, 
                f0_dy * 1.5, 
                0)), 
            _vu.get_point3d((
                f0_dx * -2, 
                f0_dy, 
                0)), 
        ),                                      ##  'point_empty', 
        (   _vu.get_point3d((
                f0_dx * -4, 
                f0_dy, 
                0)), 
            _vu.get_point3d((
                f0_dx * -2.5, 
                f0_dy * 1.5, 
                0)), 
            _vu.get_point3d((
                f0_dx * -2, 
                f0_dy, 
                0)), 
        ),                                      ##  'point_point', 
        (   _vu.get_point3d((
                f1_dx * -4, 
                f1_dy, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2.5, 
                f1_dy * 1.5, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2, 
                f1_dy, 
                0)), 
        ),                                      ##  'point_xyz', 
        (   _vu.get_point3d((
                f1_dx * -4, 
                f1_dy, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2.5, 
                f1_dy * 1.5, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2, 
                f1_dy, 
                0)), 
        ),                                      ##  'xyz_empty', 
        (   _vu.get_point3d((
                f1_dx * -4, 
                f1_dy, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2.5, 
                f1_dy * 1.5, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2, 
                f1_dy, 
                0)), 
        ),                                      ##  'xyz_point', 
        (   _vu.get_point3d((
                f1_dx * -4, 
                f1_dy, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2.5, 
                f1_dy * 1.5, 
                0)), 
            _vu.get_point3d((
                f1_dx * -2, 
                f1_dy, 
                0)), 
        ),                                      ##  'xyz_xyz', 
    ]
    n = len(try_names)
    for i in range(n):
        _try__find_rule_part_positions(
            method_name, try_names[i], rh_rules[i], expected_values[i])
    print_test_done(method_name)

def _try__find_rule_part_positions(
    method_name, try_name, rh_rule, expected_value
):
    actual_value = rh_rule._find_rule_part_positions()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

"""
    _find_rule_part_positions: empty_empty:
        expected (
            <Rhino.Geometry.Point3d object at 0x0000000000000171 [-80,20,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000172 [-50,30,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000173 [-40,20,0]>)
        got (
            <Rhino.Geometry.Point3d object at 0x0000000000000174 [-80,0,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000175 [-50,10,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000176 [-40,0,0]>)'
    _find_rule_part_positions: empty_point:
        expected (
            <Rhino.Geometry.Point3d object at 0x0000000000000177 [-80,20,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000178 [-50,30,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000179 [-40,20,0]>)
        got (
            <Rhino.Geometry.Point3d object at 0x000000000000017A [-80,0,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000017B [-50,10,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000017C [-40,0,0]>)'
    _find_rule_part_positions: empty_xyz:
        expected (
            <Rhino.Geometry.Point3d object at 0x000000000000017D [-448,112,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000017E [-280,168,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000017F [-224,112,0]>)
        got (
            <Rhino.Geometry.Point3d object at 0x0000000000000180 [-448,0,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000181 [-280,56,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000182 [-224,0,0]>)'
    _find_rule_part_positions: point_empty:
        expected (
            <Rhino.Geometry.Point3d object at 0x0000000000000183 [-80,20,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000184 [-50,30,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000185 [-40,20,0]>)
        got (
            <Rhino.Geometry.Point3d object at 0x0000000000000186 [-80,0,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000187 [-50,10,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000188 [-40,0,0]>)'
    _find_rule_part_positions: point_point:
        expected (
            <Rhino.Geometry.Point3d object at 0x0000000000000189 [-80,20,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000018A [-50,30,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000018B [-40,20,0]>)
        got (
            <Rhino.Geometry.Point3d object at 0x000000000000018C [-80,0,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000018D [-50,10,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000018E [-40,0,0]>)'
    _find_rule_part_positions: point_xyz:
        expected (
            <Rhino.Geometry.Point3d object at 0x000000000000018F [-448,112,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000190 [-280,168,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000191 [-224,112,0]>)
        got (
            <Rhino.Geometry.Point3d object at 0x0000000000000192 [-448,0,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000193 [-280,56,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000194 [-224,0,0]>)'
    _find_rule_part_positions: xyz_empty:
        expected (
            <Rhino.Geometry.Point3d object at 0x0000000000000195 [-448,112,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000196 [-280,168,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000197 [-224,112,0]>)
        got (
            <Rhino.Geometry.Point3d object at 0x0000000000000198 [-448,0,0]>, 
            <Rhino.Geometry.Point3d object at 0x0000000000000199 [-280,56,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000019A [-224,0,0]>)'
    _find_rule_part_positions: xyz_point:
        expected (
            <Rhino.Geometry.Point3d object at 0x000000000000019B [-448,112,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000019C [-280,168,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000019D [-224,112,0]>)
        got (
            <Rhino.Geometry.Point3d object at 0x000000000000019E [-448,0,0]>, 
            <Rhino.Geometry.Point3d object at 0x000000000000019F [-280,56,0]>, 
            <Rhino.Geometry.Point3d object at 0x00000000000001A0 [-224,0,0]>)'
    _find_rule_part_positions: xyz_xyz:
        expected (
            <Rhino.Geometry.Point3d object at 0x00000000000001A1 [-448,112,0]>, 
            <Rhino.Geometry.Point3d object at 0x00000000000001A2 [-280,168,0]>, 
            <Rhino.Geometry.Point3d object at 0x00000000000001A3 [-224,112,0]>)
        got (
            <Rhino.Geometry.Point3d object at 0x00000000000001A4 [-448,0,0]>, 
            <Rhino.Geometry.Point3d object at 0x00000000000001A5 [-280,56,0]>, 
            <Rhino.Geometry.Point3d object at 0x00000000000001A6 [-224,0,0]>)'
    Done: test__find_rule_part_positions
"""

# def test__make_rh_rules_for_arrange():
    # method_name = '_make_rh_rules_for_arrange'
    # rh_rules = _make_rh_rules_for_arrange()
    # n = len(rh_rules)
    # for i in range(n):
    #     print('%i: %s' % (i, rh_rules[i]))
    # print_test_done(method_name)

# def test__make_dots_and_lines_for_arrange():
    # _make_dots_and_lines_for_arrange()

# test___init__()
# test__find_common_bounding_points()
# test__find_padded_frame_specs()

# test_new_from_object_lists()
# test_is_a_rule()
# test_get_objects()
# test_find_frame_dy()
# test_arrange()
test__find_rule_part_positions()

# test__make_rh_rules_for_arrange()
# test__make_dots_and_lines_for_arrange()
