from view import py_form as                     _pf
from view import py_rule as                     _pr
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm

d = _tu.make_hash_suite_dict()
default_layer_name = _s.Settings.default_layer_name
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message

def test_make_hash_suite_dict():
    method_name = 'make_hash_suite_dict'
    try_names = [
        'ag_empty', 
        'ag_points', 
        'ag_square', 
        'ag_ells', 
        'ag_L11', 
        'ag_L12', 
        'ag_L21', 
        'ag_L22', 
        'ag_hash', 
        'pf_empty', 
        'pf_points', 
        'pf_square', 
        'pf_ells', 
        'pf_hash', 
        'pf_hash_minus_L11', 
        'pf_hash_minus_L12', 
        'pf_hash_minus_L21', 
        'pf_hash_minus_L22', 
        'pr_points_to_empty', 
        'pr_points_to_points', 
        'pr_points_to_square', 
        'pr_points_to_ells', 
        'pr_points_to_hash', 
        'pr_square_to_empty', 
        'pr_square_to_points', 
        'pr_square_to_square', 
        'pr_square_to_ells', 
        'pr_square_to_hash', 
        'pr_ells_to_empty', 
        'pr_ells_to_points', 
        'pr_ells_to_square', 
        'pr_ells_to_ells', 
        'pr_ells_to_hash', 
        'pr_hash_to_empty', 
        'pr_hash_to_points', 
        'pr_hash_to_square', 
        'pr_hash_to_ells', 
        'pr_hash_to_hash', 
    ]
    expected_values = [
        dict, 
        dict, 
        dict, 
        dict, 
        dict, 
        dict, 
        dict, 
        dict, 
        dict, 
        _pf.PyForm, 
        _pf.PyForm, 
        _pf.PyForm, 
        _pf.PyForm, 
        _pf.PyForm, 
        _pf.PyForm, 
        _pf.PyForm, 
        _pf.PyForm, 
        _pf.PyForm, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
        _pr.PyRule, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_make_hash_suite_dict(
            method_name, try_names[i], expected_values[i])
    print_test_done(method_name)

def _try_make_hash_suite_dict(method_name, try_name, expected_value):
    return_value = d[try_name]
    actual_value = type(return_value)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def print_hash_suite_dict():
    names = d.keys()
    for name in sorted(names):
        print('%s: %s' % (name, d[name]))

def test_make_modular_py_form():
    method_name = 'test_make_modular_py_form'
    try_names = [
        'empty',                                                    # 0
        'square',                                                   # 1
        'hash',                                                     # 2
        'ells',                                                     # 3
        'pt_ring_0',                                                # 4
        'pt_ring_1',                                                # 5
        'pt_ring_2',                                                # 6
        'pt_ring_3',                                                # 7
        'square_pt_e',                                              # 8
        'hash_pt_ends',                                             # 9
        'ells_pt_ends',                                             # 10
    ]
    arg_lists = [
        [],                                                         # 0
        ['square'],                                                 # 1
        ['hash'],                                                   # 2
        ['ells'],                                                   # 3
        [   '', 
            ['33'], 
            '0'],                                                   # 4
        [   '', 
            [   '22', '32', '42', 
                '23',       '43', 
                '24', '34', '44', ], 
            '1'],                                                   # 5
        [   '', 
            [   '11', '21', '31', '41', '51', 
                '12',                   '52', 
                '13',                   '53', 
                '14',                   '54', 
                '15', '25', '35', '45', '55', ], 
            '2'],                                                   # 6
        [   '', 
            [   '00', '10', '20', '30', '40', '50', '60', 
                '01',                               '61', 
                '02',                               '62', 
                '03',                               '63', 
                '04',                               '64', 
                '05',                               '65', 
                '06', '16', '26', '36', '46', '56', '66', ], 
            '3'],                                                   # 7
        [   'square', 
            ['43']],                                                # 8
        [   'hash', 
            ['01', '05', '10', '16', '50', '56', '61', '65'], ],    # 9
        [   'ells', 
            ['01', '05', '10', '16', '50', '56', '61', '65'], ],    # 10
    ]
    n = len(try_names)
    for i in range(n):
        _try_make_modular_py_form(
            method_name, try_names[i], arg_lists[i])
    print_test_done(method_name)

def _try_make_modular_py_form(method_name, try_name, args):
    _vm.ViewMain.clear_all()
    n = len(args)
    if n == 0:
        py_form = _tu.make_modular_py_form()
    elif n == 1:
        line_shape_name = args[0]
        py_form = _tu.make_modular_py_form(line_shape_name)
    elif n == 2:
        line_shape_name, point_names = args
        py_form = _tu.make_modular_py_form(line_shape_name, point_names)
    elif n == 3:
        line_shape_name, point_names, label = args
        py_form = _tu.make_modular_py_form(line_shape_name, point_names, label)
    else:
        message = "Too many args"
    py_form.draw(default_layer_name)
    message = "Check the py_form: %s" % try_name
    _rs.GetString(message)

test_make_hash_suite_dict()
print_hash_suite_dict()
test_make_modular_py_form()


