import                                          copy
from view import arrow as                       _a
from view import draw as                        _d
from view import frame as                       _fr
from view import grammar as                     _g
from view import layer as                       _l
from view import py_form as                     _pf
from view import py_rule as                     _pr
import                                          random
import rhinoscriptsyntax as                     _rs
from view import settings as                    _s
import sortalgi as                              _sgi
from view.tests import test_utilities as        _tu
from view import view_main as                   _vm
from view import view_utilities as              _vu

d = _tu.make_hash_suite_dict()
e = _tu.make_element_dict()
default_layer_name = _s.Settings.default_layer_name
origin = _s.Settings.origin
origin_p3d = _s.Settings.origin_p3d
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message
red = [255, 0, 0]
green = [0, 255, 0]
vm = _vm.ViewMain()

### request (new)

def test_request_form_and_subform():
    method_name = 'request_form_and_subform'
    try_names = [
        # 0: select none; 1: select all; x: cancel
        'bad_state_empty_grammar', 
        '0x', 
        '01_0', 
        '01_1', 
        '1_0', 
        '1_1', 
    ]
    _vm.ViewMain.clear_all()
    pf_hash = d['pf_hash']
    instructions = [
        "[Test: Do nothing]", 
        "[Test: None; cancel]", 
        "[Test: None, all. None]", 
        "[Test: None, all. All]", 
        "[Test: All. None]", 
        "[Test: All. All]", 
    ]
    expected_values = [
        None,                   # 'bad_state_empty_grammar', 
        None,                   # '0x', 
        (pf_hash, pf_hash),     # '01_0', 
        (pf_hash, pf_hash),     # '01_1', 
        (pf_hash, pf_hash),     # '1_0', 
        (pf_hash, pf_hash),     # '1_1', 
    ]
    n = len(try_names)
    for i in range(n):
        if i == 1:
            pf_hash.draw(default_layer_name)
        _try_request_form_and_subform(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _try_request_form_and_subform(
    method_name, try_name, instruction, expected_value
):
    print(instruction)
    actual_value = vm.request_form_and_subform()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_request_new_rule():
    method_name = 'request_new_rule'
    try_names = [
        'ill_formed', 
        'new_unique', 
    ]
    _set_up_for_request_new_rule()
    instructions = [
        'Select empty, square, origin', 
        'Select square, square, origin', 
    ]
    pr_square_to_square = d['pr_square_to_square']
    expected_values = [
        None, 
        pr_square_to_square, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_request_new_rule(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_request_new_rule():
    vm.clear_all()
    pf_square = d['pf_square']
    pf_square.draw(default_layer_name)

def _try_request_new_rule(method_name, try_name, instruction, expected_value):
    print('[Test: %s] %s' % (try_name, instruction))
    actual_value = vm.request_new_rule()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_request_rule():
    method_name = 'request_rule'
    try_names = [
        'bad_state_empty_grammar', 
        '0x', 
        '0rule', 
        'rule', 
    ]
    _vm.ViewMain.clear_all()
    instructions = [
        "[Test: None]", 
        "[Test: None; cancel]", 
        "[Test: None, rule]", 
        "[Test: Rule]", 
    ]
    pr_hash_to_ells = d['pr_hash_to_ells']
    pr_hash_to_ells_name = str(pr_hash_to_ells.id_)
    expected_values = [
        None, 
        None, 
        pr_hash_to_ells_name,
        pr_hash_to_ells_name,
    ]
    n = len(try_names)
    for i in range(n):
        if i == 1:
            _set_up_request_rule()
        _try_request_rule(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _set_up_request_rule():
    d['pr_hash_to_ells'].draw()
    d['pf_square'].draw('Default')
    d['pf_ells'].draw('Default')

def _try_request_rule(method_name, try_name, instruction, expected_value):
    print(instruction)
    actual_value = vm.request_rule()
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__request_form():
    method_name = '_request_form'
    vm.clear_all()
    ref_point_xyz = (50, 0, 0)
    ref_point = _vu.get_point3d(ref_point_xyz)
    obs = _d.Draw._draw_framed_object_group(ref_point)
    try_names = [
        'none', 
        'all', 
    ]
    instructions = [
        "[Test: Select none]", 
        "[Test: Select all]", 
    ]
    empty_py_form = _pf.PyForm.new_empty()
    py_form = _pf.PyForm.new_from_objects_and_ref_point(obs, origin_p3d)
    expected_values = [
        empty_py_form, 
        py_form, 
    ]
    n = len(try_names)
    for i in range(n):
        _try__request_form(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _try__request_form(method_name, try_name, instruction, expected_value):
    print(instruction)
    message = "Select a shape"
    repr_str = "vm.%s(message)" % method_name
    actual_value = eval(repr_str)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__request_form_with_ref_point():
    method_name = '_request_form_with_ref_point'
    _vm.ViewMain.clear_all()
    ref_point_xyz = (50, 0, 0)
    ref_point = _vu.get_point3d(ref_point_xyz)
    obs = _d.Draw._draw_framed_object_group(ref_point)
    try_names = [
        'none', 
        'all', 
    ]
    instructions = [
        "[Test: Select none]", 
        "[Test: Select all. Select (50, 0, 0)]", 
    ]
    empty_py_form = _pf.PyForm.new_empty()
    py_form = _pf.PyForm.new_from_objects_and_ref_point(obs, ref_point)
    expected_values = [
        empty_py_form, 
        py_form, 
    ]
    n = len(try_names)
    for i in range(n):
        _try__request_form_with_ref_point(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _try__request_form_with_ref_point(
    method_name, try_name, instruction, expected_value
):
    print(instruction)
    message = "Select a shape and reference point"
    repr_str = 'vm.%s(message)' % method_name
    actual_value = eval(repr_str)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_request_non_empty_form():
    method_name = 'request_non_empty_form'
    _vm.ViewMain.clear_all()
    obs = _d.Draw._draw_framed_object_group()
    try_names = [
        'none_cancel', 
        'none_retry_all', 
        'all', 
    ]
    instructions = [
        '[Test: Select nothing. Cancel]', 
        '[Test: Select nothing. Retry. Select all]', 
        '[Test: Select all]', 
    ]
    py_form = _pf.PyForm.new_from_objects_and_ref_point(obs, origin)
    expected_values = [
        None, 
        py_form, 
        py_form, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_request_non_empty_form(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _try_request_non_empty_form(
    method_name, try_name, instruction, expected_value
):
    print(instruction)
    message = 'Select a non-empty shape'
    repr_str = 'vm.%s(message)' % method_name
    actual_value = eval(repr_str)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_request_non_empty_form_with_ref_point():
    method_name = 'request_non_empty_form_with_ref_point'
    vm.clear_all()
    ref_point_xyz = (50, 0, 0)
    ref_point = _vu.get_point3d(ref_point_xyz)
    obs = _d.Draw._draw_framed_object_group(ref_point)
    try_names = [
        'none_cancel', 
        'none_retry_all', 
        'all', 
    ]
    instructions = [
        '[Test: Select nothing. Cancel]', 
        '[Test: Select nothing. Retry. Select all. Select (50, 0, 0)]', 
        '[Test: Select all. Select (50, 0, 0)]', 
    ]
    py_form = _pf.PyForm.new_from_objects_and_ref_point(obs, ref_point)
    expected_values = [
        None, 
        py_form, 
        py_form, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_request_non_empty_form_with_ref_point(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _try_request_non_empty_form_with_ref_point(
    method_name, try_name, instruction, expected_value
):
    print(instruction)
    message = "Select a non-empty shape"
    repr_str = 'vm.%s(message)' % (method_name)
    actual_value = eval(repr_str)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test__request_objects():
    method_name = '_request_objects'
    _vm.ViewMain.clear_all()
    obs = _d.Draw._draw_framed_object_group()
    try_names = [
        'select_none', 
        'select_all', 
    ]
    instructions = [
        '[Test: Select none]', 
        '[Test: Select all]', 
    ]
    expected_values = [
        [], 
        obs, 
    ]
    n = len(try_names)
    for i in range(n):
        _try__request_objects(
            method_name, try_names[i], instructions[i], expected_values[i])
    print_test_done(method_name)

def _try__request_objects(method_name, try_name, instruction, expected_value):
    print(instruction)
    message = 'Select form'
    # vm = _vm.ViewMain()
    repr_str = 'vm.%s(message)' % method_name
    actual_value = eval(repr_str)
    if not set(actual_value) == set(expected_value):
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

### get

def test_get_rule_layer_names():
    method_name = 'get_rule_layer_names'
    try_names = [
        'no_layers', 
        'shape_layers_no_rule_layers', 
        'rule_layers_no_shape_layers', 
        'shape_and_rule_layers', 
    ]
    py_forms = [
        d['pf_points'], 
        d['pf_square'], 
    ]
    py_rule_1, py_rule_2 = py_rules = [
        d['pr_points_to_square'], 
        d['pr_ells_to_hash'], 
    ]
    py_rule_ids = sorted([py_rule_1.id, py_rule_2.id])
    setup_list_pairs = [
        ([], []), 
        (py_forms, []), 
        ([], py_rules), 
        (py_forms, py_rules), 
    ]
    expected_values = [
        [], 
        [], 
        py_rule_ids, 
        py_rule_ids, 
    ]
    n = len(try_names)
    for i in range(n):
        _set_up_for_get_rule_layer_names(setup_list_pairs[i])
        _try_get_rule_layer_names(
            method_name, 
            try_names[i], 
            expected_values[i])
    print_test_done(method_name)

def _set_up_for_get_rule_layer_names(lists):
    _sgi.sortal_setup()
    _vm.ViewMain.clear_all()
    py_forms, py_rules = lists
    for pf in py_forms:
        pf_hash = hash(pf)
        shape_layer_name = _l.Layer.new(pf_hash)
        pf.draw(shape_layer_name)
    n = len(py_rules)
    if n > 0:
        for i in range(n):
            pr = py_rules[i]
            pr.draw((0, 200 * i, 0))

def _try_get_rule_layer_names(method_name, try_name, expected_value):
    received_value = _vm.ViewMain.get_rule_layer_names()
    actual_value = sorted(received_value)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_rh_rule_exists():
    method_name = 'rh_rule_exists'
    try_names = [
        'no_layer_name', 
        'hash_empty_no_blocks_no_obs', 
        'ells_empty_obs_no_blocks', 
        'square_empty_blocks_no_obs', 
        'points_empty_blocks_obs', 
        'points_empty_blocks_obs_1', 
    ]
    py_rules = _set_up_for_rh_rule_exists()
    expected_values = [
        False, 
        False, 
        False, 
        True, 
        True, 
        True, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_rh_rule_exists(
            method_name, try_names[i], py_rules[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_rh_rule_exists():
    _vm.ViewMain.clear_all()
    _set_up_non_rh_layers_for_rh_rule_exists()
    py_rules = _set_up_py_rules_for_rh_rule_exists()
    return py_rules

def _set_up_non_rh_layers_for_rh_rule_exists():
    def _draw_obs_on_ells_empty():
        py_rule = d['pr_ells_to_empty']
        layer_name = str(py_rule.id_)
        _l.Layer.new(layer_name)
        py_rule.a.draw(layer_name)
        
    def _draw_blocks_on_square_empty():
        py_rule = d['pr_square_to_empty']
        layer_name = str(py_rule.id_)
        _l.Layer.new(layer_name)
        p1 = (10, 10, 10)
        p2 = (30, 30, 30)
        p3 = (50, 50, 50)
        _fr.Frame.new_instance(layer_name, p1)
        _fr.Frame.new_instance(layer_name, p2)
        _a.Arrow.new_instance(layer_name, p3)
        
    def _draw_blocks_and_obs_on_points_empty():
        py_rule = d['pr_points_to_empty']
        rh_rule = py_rule.draw()
        rh_rule_1 = py_rule.draw()
        py_rule_id_str = str(py_rule.id_)
        _rs.PurgeLayer(py_rule_id_str)

    _draw_obs_on_ells_empty()
    _draw_blocks_on_square_empty()
    _draw_blocks_and_obs_on_points_empty()

def _set_up_py_rules_for_rh_rule_exists():
    py_rule_names = [
        'pr_xyz_empty', 
        'pr_hash_to_empty', 
        'pr_ells_to_empty', 
        'pr_square_to_empty', 
        'pr_points_to_empty', 
        'pr_points_to_empty', 
    ]
    py_rules = []
    for name in py_rule_names:
        if 'to' in name:
            dic = d
        else:
            dic = e
        py_rule = dic[name]
        py_rules.append(py_rule)
    return py_rules

def _try_rh_rule_exists(method_name, try_name, py_rule, expected_value):
    actual_value = vm.rh_rule_exists(py_rule)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_arrange_rh_rules():
    method_name = 'arrange_rh_rules'
    try_names = [
        '0_new_0_old', 
        '1_new_0_old', 
        '0_new_1_old', 
        '1_new_1_old', 
    ]
    _set_up_for_arrange_rh_rules()
    setups = [
        'None', 
        'd["pr_hash_to_square"].draw()', 
        'None', 
        'd["pr_points_to_square"].draw()', 
    ]
    expected_values = [
        None, 
        0, 
        None, 
        19, 
    ]
    n = len(try_names)
    for i in range(n):
        _try_arrange_rh_rules(
            method_name, try_names[i], setups[i], expected_values[i])
    print_test_done(method_name)

def _set_up_for_arrange_rh_rules():
    _vm.ViewMain.clear_all()
    for pf_name in ['pf_hash', 'pf_ells']:
        layer_name = _l.Layer.new(pf_name)
        py_form = d[pf_name]
        py_form.draw(layer_name)

def _try_arrange_rh_rules(method_name, try_name, setup, expected_value):
    new_rule = eval(setup)
    actual_value = vm.arrange_rh_rules(new_rule)
    message = "Confirm objects moved: old rules, not new rule, not forms"
    _rs.GetString(message)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

### draw

### setup for tests

def _draw_rule_application_forms():
    n = n_forms = 4
    py_forms = []
    for i in range(n):
        x = 50 * i
        ref_point = (x, 0, 0)
        obs = _d.Draw._draw_framed_object_group(ref_point, 'fi')
        if i == 1:
            py_form = _pf.PyForm.new_from_objects_and_ref_point(
                obs, origin)
        else:
            py_form = _pf.PyForm.new_from_objects_and_ref_point(
                obs, ref_point)
        py_forms.append(py_form)
    pf1, pf2, pf3, pf4 = py_forms
    return (pf1, pf2, pf3, pf4)

def _test__draw_rule_application_forms():
    _vm.ViewMain.clear_all()
    py_forms = _draw_rule_application_forms()
    for py_form in py_forms:
        print(py_form)

def _draw_setup_rule(args):
    """Receives:
        args                [l_description, r_description, ref_point]
            l_description   str. Contains 'e', 'f', 'o' for elements (lines 
                            and text dots), frames, and other objects
            r_description   str. As above
            ref_point       point3d
    Draws left and right located forms as specified on a dedicated layer. 
    Returns:
        py_rule             PyRule
        # left_located_form   (form, left_ref_point)
        # right_located_form  (form, right_ref_point)
        #     form            set([guid, ...]). Lines and text dots
        #     ref_point       point3d
    """
    previous_layer = _rs.CurrentLayer()
    test_layer = 'test_rule_layer'
    if not _rs.IsLayer(test_layer):
        _rs.AddLayer(test_layer)
    _rs.CurrentLayer(test_layer)
    l_description, r_description, ref_point = args
    left_point = ref_point
    offset = (50, 0, 0)
    right_xyz = [left_point[i] + offset[i] for i in range(len(left_point))]
    right_point_obj = _rs.AddPoint(right_xyz)
    right_point = _rs.PointCoordinates(right_point_obj)

    left_py_form = _draw_located_objects(l_description, left_point)
    right_py_form = _draw_located_objects(r_description, right_point)
    py_rule = _pr.PyRule(left_py_form, right_py_form)
    _rs.CurrentLayer(previous_layer)
    return py_rule

def _draw_located_objects(description, ref_point):
    """Receives:
        description         str. Contains 'e', 'f', 'o' for elements (lines 
                            and text dots), frames, and other objects
        ref_point           (x, y, z). The reference point
            x, y, z         num
    Draws located objects as specified at the reference point. Returns:
        py_form             PyForm
        # located_form        set([guid, ...]). Elements, i.e., lines and text 
        #                     dots
        # ref_point     (x, y, z)
        #     x, y, z         num
    """
    els = []
    if 'e' in description:
        els = _draw_located_elements(ref_point)
    if 'f' in description:
        located_frame = _draw_located_frame(ref_point)
    if 'o' in description:
        located_other_set = _draw_located_others(ref_point)
    py_form = _pf.PyForm(els, ref_point, 'r')
    return py_form

def _draw_located_elements(ref_point):
    """Receives:
        ref_point           (x, y, z). The reference point
            x, y, z         num
    Draws elements (lines and text dots) at the reference point. Returns:
        located_element_set set([guid, ...]). Lines and text dots
    """
    n_lines = 2
    n_text_dots = 2

    line_set = _draw_lines(n_lines)
    text_dot_set = _draw_text_dots(n_text_dots)
    located_line_set = _move_object_set(line_set, ref_point)
    located_text_dot_set = _move_object_set(text_dot_set, ref_point)
    located_element_set = located_line_set.union(located_text_dot_set)
    return located_element_set

def _draw_located_frame(reference_point):
    layer_name = _rs.CurrentLayer()
    located_frame = _fr.Frame.new_instance(layer_name, reference_point)
    return located_frame

def _draw_located_others(reference_point):
    located_other_set = _draw_other_objects()
    _move_object_set(located_other_set, reference_point)
    return located_other_set

def _move_object_set(object_set, reference_point):
    for object_ in object_set:
        _rs.MoveObject(object_, reference_point)
    # object_set = _rs.MoveObjects(object_set, reference_point)
    return object_set

def _draw_forms(forms_arg):
    """Receives:
        forms_arg           str in ['l', 'r', 'lr', '']
    Draws left form, right form, both, or neither. Returns:
        left_form           set([guid, ...]). Lines and text dots
        right_form          set([guid, ...]). Lines and text dots
    """
    empty_form = (set([]), origin)
    if forms_arg == 'l':
        left_form = _draw_form('l')
        right_form = empty_form
    elif forms_arg == 'r':
        left_form = empty_form
        right_form = _draw_form('r')
    elif forms_arg == 'lr':
        left_form = _draw_form('l')
        right_form = _draw_form('r')
    elif forms_arg == '':
        left_form = empty_form
        right_form = empty_form
    else:
        error_message = "We shouldn't have gotten here"
        print(error_message)
        return None
    return (left_form, right_form)

def _draw_form(l_or_r):
    """Receives:
        l_or_r              str in ['l', 'r']
    Draws a left or right form (lines and text dots). Returns:
        form                set([guid, ...]). Lines and text dots
    """
    lines = _draw_lines()
    text_dots = _draw_text_dots()
    return form

def _draw_setup_objects(types, n=2):
    """Receives:
        n                   int. The number of each object type to draw
    Returns:
        objects             {<type>: [guid, ...], ...}
            <type>          str. 'lines' | 'frames' | 'text_dots' | 'others'
            guid
    What about layers?
    """
    _vm.ViewMain.clear_all()
    objects = {}
    # if 'points' in types:
    #     points = _draw_points(n)
        # objects['points'] = points
    if 'lines' in types:
        lines = _draw_lines(n)
        objects['lines'] = lines
    # if 'curves' in types:
    #     curves = _draw_curves(n)
    #     objects['curves'] = curves
    # if 'planes' in types:
    #     planes = _draw_planes(n)
    #     objects['planes'] = planes
    # if 'surfaces' in types:
    #     surfaces = _draw_surfaces(n)
    #     objects['surfaces'] = surfaces
    if 'frames' in types:
        frames = _draw_frames(n)
        objects['frames'] = frames
    # if 'instances' in types:
    #     instances = _draw_instances(n)
    #     objects['instances'] = instances
    if 'text_dots' in types:
        text_dots = _draw_text_dots(n)
        objects['text_dots'] = text_dots
    if 'others' in types:
        others = _draw_other_objects()
        objects['others'] = others
    return objects

def _draw_objects():
    # draw surfaces, annotations, etc.
    # frame = _draw_frames(2)
    frame = _draw_frame()
    lines = _draw_lines(3)
    dots = _draw_text_dots(3)
    others = _draw_other_objects()
    return (frame, lines, dots, others)

def _draw_frames(n):
    frames = []
    for i in range(n):
        x = 50 * i
        y, z = 0, 0
        frame = _draw_frame((x, y, z))
        frames.append(frame)
    return frames

def _draw_frame(p=(0, 0, 0)):
    frame = _fr.Frame.new_instance(default_layer_name, p)
    return frame

def _draw_random_lines_and_text_dots():
    n = 3
    line_set = _draw_lines(n)
    text_dot_set = _draw_text_dots(n)
    line_and_text_dot_set = line_set.union(text_dot_set)
    return line_and_text_dot_set

def _draw_lines(n):
    line_set = set([])
    for i in range(n):
        xyz_1 = _make_random_xyz()
        xyz_2 = _make_random_xyz()
        line = _rs.AddLine(xyz_1, xyz_2)
        line_set.add(line)
    return line_set
    
def _draw_text_dots(n):
    text_dot_set = set([])
    for i in range(n):
        label = _make_random_label()
        xyz = _make_random_xyz()
        text_dot = _rs.AddTextDot(label, xyz)
        text_dot_set.add(text_dot)
    return text_dot_set

def _make_random_label():
    abc = 'abcdefghijklmnopqrstuvwxyz'
    sample = random.sample(abc, 1)
    label = sample.pop()
    return label

def _draw_other_objects():
    curve = _draw_random_curve()
    rect = _draw_random_rectangle()
    light = _draw_random_light()
    other_set = set([curve, rect, light])
    return other_set

def _draw_random_curve():
    n_xyzs = 3
    xyzs = [_make_random_xyz() for i in range(n_xyzs)]
    curve = _rs.AddCurve(xyzs)
    return curve

def _draw_random_rectangle():
    plane = _rs.WorldXYPlane()
    xyz = _make_random_xyz()
    w, h, z = xyz
    rect = _rs.AddRectangle(plane, w, h)
    return rect

def _draw_random_light():
    xyz = _make_random_xyz()
    light = _rs.AddPointLight(xyz)
    return light

def _make_random_xyz():
    min_ = 0
    max_ = 32
    x = random.uniform(min_, max_)
    y = random.uniform(min_, max_)
    z = random.uniform(min_, max_)
    return (x, y, z)

### experiment

def test_object_description():
    message = 'Select an object'
    obj = _rs.GetObject(message)
    descr = _rs.ObjectDescription(obj)
    print('descr: %s' % descr)                  ##  Curve

test_request_form_and_subform()
test_request_new_rule()
test_request_rule()

test__request_form()
test__request_form_with_ref_point()
test_request_non_empty_form()
test_request_non_empty_form_with_ref_point()
test__request_objects()

# test_rh_rule_exists()
# test_arrange_rh_rules()


### test experiment

# _vm.ViewMain.clear_all()
# test_object_description()

