import                                          math
import rhinoscriptsyntax as                     _rs
from view.tests import test_utilities as        _tu
from view import view_utilities as              _vu

cos, sin = math.cos, math.sin
TAU = math.pi * 2
c = cos30 = cos(TAU / 12)
s = sin60 = sin(TAU /  6)
print_test_done = _tu.print_test_done
print_test_error_message = _tu.print_test_error_message


print('cos30: %s' % cos30)                      ##  0.866025403784
print('sin60: %s' % sin60)                      ##  0.866025403784
print('cos30 >  sin60: %s' % cos30 >  sin60)    ##  True
print('cos30 == sin60: %s' % cos30 == sin60)    ##  False
print('cos30 <  sin60: %s' % cos30 <  sin60)    ##  False

def test_eeq_xyz():
    method_name = 'eeq_xyz'
    xyzs, names = _make_xyzs_and_names()
    expected_values = [
        # ooo
        [   True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # ooc
        [   False, True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oos
        [   False, True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oco
        [   False, False, False, 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # occ
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # ocs
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oso
        [   False, False, False, 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # osc
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oss
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # coo
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # coc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # cos
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # cco
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            True , False, False, 
            True , False, False, 
        ], 
        # ccc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # ccs
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # cso
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            True , False, False, 
            True , False, False, 
        ], 
        # csc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # css
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # soo
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # soc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # sos
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # sco
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            True , False, False, 
            True , False, False, 
        ], 
        # scc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # scs
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # sso
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            True , False, False, 
            True , False, False, 
        ], 
        # ssc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # sss
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            try_name = '%s %s %s' % (names[i], method_name, names[j])
            _try_eeq_xyz(
                method_name, 
                try_name, 
                xyzs[i], 
                xyzs[j], 
                expected_values[i][j])
    print_test_done(method_name)

def _try_eeq_xyz(method_name, try_name, xyz1, xyz2, expected_value):
    actual_value = _vu.eeq_xyz(xyz1, xyz2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_ege_xyz():
    method_name = 'ege_xyz'
    xyzs, names = _make_xyzs_and_names()
    expected_values = [
        # ooo
        [   True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # ooc
        [   True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oos
        [   True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oco
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # occ
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # ocs
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oso
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # osc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oss
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # coo
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # coc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # cos
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # cco
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # ccc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # ccs
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # cso
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # csc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # css
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # soo
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # soc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # sos
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # sco
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # scc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # scs
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # sso
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # ssc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # sss
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            try_name = '%s %s %s' % (names[i], method_name, names[j])
            _try_ege_xyz(
                method_name, 
                try_name, 
                xyzs[i], 
                xyzs[j], 
                expected_values[i][j])
    print_test_done(method_name)

def _try_ege_xyz(method_name, try_name, xyz1, xyz2, expected_value):
    actual_value = _vu.ege_xyz(xyz1, xyz2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_egt_xyz():
    method_name = 'egt_xyz'
    xyzs, names = _make_xyzs_and_names()
    expected_values = [
        # ooo
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # ooc
        [   True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oos
        [   True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oco
        [   True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # occ
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # ocs
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oso
        [   True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # osc
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # oss
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # coo
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # coc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # cos
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # cco
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # ccc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # ccs
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # cso
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # csc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # css
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # soo
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # soc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # sos
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            False, False, False, 
            False, False, False, 
            True , False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # sco
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # scc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # scs
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # sso
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            False, False, False, 
            False, False, False, 
        ], 
        # ssc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # sss
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            try_name = '%s %s %s' % (names[i], method_name, names[j])
            _try_egt_xyz(
                method_name, 
                try_name, 
                xyzs[i], 
                xyzs[j], 
                expected_values[i][j])
    print_test_done(method_name)

def _try_egt_xyz(method_name, try_name, xyz1, xyz2, expected_value):
    actual_value = _vu.egt_xyz(xyz1, xyz2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_ele_xyz():
    method_name = 'ele_xyz'
    xyzs, names = _make_xyzs_and_names()
    expected_values = [
        # ooo
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # ooc
        [   False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oos
        [   False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oco
        [   False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # occ
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # ocs
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oso
        [   False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # osc
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oss
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # coo
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # coc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # cos
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # cco
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # ccc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # ccs
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # cso
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # csc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # css
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # soo
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # soc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # sos
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # sco
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # scc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # scs
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # sso
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # ssc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # sss
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            try_name = '%s %s %s' % (names[i], method_name, names[j])
            _try_ele_xyz(
                method_name, 
                try_name, 
                xyzs[i], 
                xyzs[j], 
                expected_values[i][j])
    print_test_done(method_name)

def _try_ele_xyz(method_name, try_name, xyz1, xyz2, expected_value):
    actual_value = _vu.ele_xyz(xyz1, xyz2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_elt_xyz():
    method_name = 'elt_xyz'
    xyzs, names = _make_xyzs_and_names()
    expected_values = [
        # ooo
        [   False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # ooc
        [   False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oos
        [   False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oco
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # occ
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # ocs
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oso
        [   False, False, False, 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # osc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oss
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # coo
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # coc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # cos
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # cco
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # ccc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # ccs
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # cso
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # csc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # css
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # soo
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # soc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # sos
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            True , True , True , 
            True , True , True , 
            False, False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # sco
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # scc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # scs
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # sso
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, True , True , 
            False, True , True , 
            False, False, False, 
            False, True , True , 
            False, True , True , 
        ], 
        # ssc
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
        # sss
        [   False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
            False, False, False, 
        ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            try_name = '%s %s %s' % (names[i], method_name, names[j])
            _try_elt_xyz(
                method_name, 
                try_name, 
                xyzs[i], 
                xyzs[j], 
                expected_values[i][j])
    print_test_done(method_name)

def _try_elt_xyz(method_name, try_name, xyz1, xyz2, expected_value):
    actual_value = _vu.elt_xyz(xyz1, xyz2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def test_ene_xyz():
    method_name = 'ene_xyz'
    xyzs, names = _make_xyzs_and_names()
    expected_values = [
        # ooo
        [   False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # ooc
        [   True , False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oos
        [   True , False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oco
        [   True , True , True , 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # occ
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # ocs
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oso
        [   True , True , True , 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # osc
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # oss
        [   True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # coo
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # coc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # cos
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # cco
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            False, True , True , 
            False, True , True , 
        ], 
        # ccc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # ccs
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # cso
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            False, True , True , 
            False, True , True , 
        ], 
        # csc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # css
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # soo
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            True , True , True , 
            True , True , True , 
        ], 
        # soc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # sos
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , True , True , 
            True , True , True , 
        ], 
        # sco
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            False, True , True , 
            False, True , True , 
        ], 
        # scc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # scs
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # sso
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            False, True , True , 
            False, True , True , 
            True , True , True , 
            False, True , True , 
            False, True , True , 
        ], 
        # ssc
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
        # sss
        [   True , True , True , 
            True , True , True , 
            True , True , True , 
            True , True , True , 
            True , False, False, 
            True , False, False, 
            True , True , True , 
            True , False, False, 
            True , False, False, 
        ], 
    ]
    n = len(names)
    for i in range(n):
        for j in range(n):
            try_name = '%s %s %s' % (names[i], method_name, names[j])
            _try_ene_xyz(
                method_name, 
                try_name, 
                xyzs[i], 
                xyzs[j], 
                expected_values[i][j])
    print_test_done(method_name)

def _try_ene_xyz(method_name, try_name, xyz1, xyz2, expected_value):
    actual_value = _vu.ene_xyz(xyz1, xyz2)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

def _make_xyzs_and_names():
    xyzs = [
        (0, 0, 0), (0, 0, c), (0, 0, s), 
        (0, c, 0), (0, c, c), (0, c, s), 
        (0, s, 0), (0, s, c), (0, s, s), 
        (c, 0, 0), (c, 0, c), (c, 0, s), 
        (c, c, 0), (c, c, c), (c, c, s), 
        (c, s, 0), (c, s, c), (c, s, s), 
        (s, 0, 0), (s, 0, c), (s, 0, s), 
        (s, c, 0), (s, c, c), (s, c, s), 
        (s, s, 0), (s, s, c), (s, s, s), 
    ]
    names = [
        'ooo', 'ooc', 'oos', 
        'oco', 'occ', 'ocs', 
        'oso', 'osc', 'oss', 
        'coo', 'coc', 'cos', 
        'cco', 'ccc', 'ccs', 
        'cso', 'csc', 'css', 
        'soo', 'soc', 'sos', 
        'sco', 'scc', 'scs', 
        'sso', 'ssc', 'sss', 
    ]
    return (xyzs, names)

def test_is_a_list_of_strings():
    method_name = 'is_a_list_of_strings'
    input_values = [
        'a', 
        123, 
        [        ], 
        [     123], 
        [     'a'], 
        [123, 123], 
        [123, 'a'], 
        ['a', 123], 
        ['a', 'b'], 
    ]
    expected_values = [
        False, 
        False, 
        True, 
        False, 
        True, 
        False, 
        False, 
        False, 
        True, 
    ]
    n = len(input_values)
    for i in range(n):
        input_value = input_values[i]
        try_name = str(input_value)
        expected_value = expected_values[i]
        _try_is_a_list_of_strings(
            method_name, try_name, input_value, expected_value)
    print_test_done(method_name)

def _try_is_a_list_of_strings(
    method_name, try_name, input_value, expected_value
):
    actual_value = _vu.is_a_list_of_strings(input_value)
    if not actual_value == expected_value:
        print_test_error_message(
            method_name, try_name, expected_value, actual_value)

test_eeq_xyz()
test_ege_xyz()
test_egt_xyz()
test_ele_xyz()
test_elt_xyz()
test_ene_xyz()
test_is_a_list_of_strings()
