import                                  random
from view import draw as                _d
from view import py_point as            _pp
import rhinoscriptsyntax as             _rs
from view import settings as            _s
from view import view_utilities as      _vu

origin = _s.Settings.origin

class TextDot(object):
    def __init__(self):
        pass

    @classmethod
    def is_a_num_triple(cls, item):
        """Receives:
            item            guid
        Returns:
            value           boolean. True if the item is a point3d. False 
                            otherwise
        """
        value = False
        if (len(item) == 3 and
            _vu.is_a_number(item[0]) and
            _vu.is_a_number(item[1]) and
            _vu.is_a_number(item[2])
        ):
            value = True
        return value

    @classmethod
    def draw_random_text_dots(cls, n=2, ref_point=origin, rel_location='i'):
        text_dots = []
        for i in range(n):
            label = cls._make_random_label()
            xyz = _d.Draw._make_random_xyz(ref_point, rel_location)
            text_dot = cls.draw(label, xyz)
            text_dots.append(text_dot)
        return text_dots

    @classmethod
    def draw_random(cls, ref_point=origin, rel_location='i'):
        label = cls._make_random_label()
        xyz = _d.Draw._make_random_xyz(ref_point, rel_location)
        text_dot = cls.draw(label, xyz)
        return text_dot

    @classmethod
    def _make_random_label(cls):
        abc = 'abcdefghijklmnopqrstuvwxyz'
        sample = random.sample(abc, 1)
        label = sample.pop()
        return label

    @classmethod
    def draw(cls, label, xyz):
        text_dot = _rs.AddTextDot(label, xyz)
        return text_dot

    @classmethod
    def is_text_dot(cls, object_):
        """Receives:
            object_         guid
        Returns:
            value           boolean. True if object_ is a text dot. False 
                            otherwise
        """
        value = False
        if _rs.ObjectType(object_) == _s.Settings.text_dot_type:
            value = True
        return value

    @classmethod
    def get_agnostic(cls, text_dot):
        """Receives:
            text_dot        guid
        Returns:
            point_ag        (xyz, attribs)
                xyz         (num, num, num). The coordinates of the point
                attribs     [attrib, ...]. A list of attributes
                attrib      'attrib=<attrib>'
        """
        td = text_dot
        abs_p3d = _rs.TextDotPoint(td)
        rel_p3d = _rs.PointSubtract(abs_p3d, ref_point3d)
        rel_xyz = tuple(rel_p3d)
        label = _rs.TextDotText(td)
        label_attrib = 'label=%s' % label
        attribs = [label_attrib]
        point_ag = (rel_xyz, attribs)

        point_ag = (xyz, attribs)
        return point_ag