import rhinoscriptsyntax as             _rs
from view import settings as            _s

def eeq_xyz(xyz1, xyz2):
    """Receives:
        xyz1, xyz2      (num, num, num)
    Determines whether xyz1 is effectively equal to xyz2. Returns:
        value           boolean. True if xyz1 == xyz2, within Rhino's 
                        tolerance. False otherwise
    """
    value = _rs.PointCompare(xyz1, xyz2)
    return value

def ege_xyz(xyz1, xyz2):
    """Receives:
        xyz1, xyz2      (num, num, num)
    Determines whether xyz1 is effectively greater than or equal to xyz2. 
    Returns:
        value           boolean. True if xyz1 >= xyz2, within Rhino's 
                        tolerance. False otherwise
    """
    value = False
    if egt(xyz1[0], xyz2[0]):
        value = True
    elif eeq(xyz1[0], xyz2[0]):
        if egt(xyz1[1], xyz2[1]):
            value = True
        elif eeq(xyz1[1], xyz2[1]):
            if ege(xyz1[2], xyz2[2]):
                value = True
    return value

def egt_xyz(xyz1, xyz2):
    """Receives:
        xyz1, xyz2      (num, num, num)
    Determines whether xyz1 is effectively greater than xyz2. Returns:
        value           boolean. True if xyz1 > xyz2, within Rhino's 
                        tolerance. False otherwise
    """
    value = False
    if egt(xyz1[0], xyz2[0]):
        value = True
    elif eeq(xyz1[0], xyz2[0]):
        if egt(xyz1[1], xyz2[1]):
            value = True
        elif eeq(xyz1[1], xyz2[1]):
            if egt(xyz1[2], xyz2[2]):
                value = True
    return value

def ele_xyz(xyz1, xyz2):
    """Receives:
        xyz1, xyz2      (num, num, num)
    Determines whether xyz1 is effective less than or equal to xyz2. 
    Returns:
        value           boolean. True if xyz1 <= xyz2, within Rhino's 
                        tolerance. False otherwise
    """
    value = False
    if elt(xyz1[0], xyz2[0]):
        value = True
    elif eeq(xyz1[0], xyz2[0]):
        if elt(xyz1[1], xyz2[1]):
            value = True
        elif eeq(xyz1[1], xyz2[1]):
            if ele(xyz1[2], xyz2[2]):
                value = True
    return value

def elt_xyz(xyz1, xyz2):
    """Receives:
        xyz1, xyz2      (num, num, num)
    Determines whether xyz1 is effective less than xyz2. Returns:
        value           boolean. True if xyz1 < xyz2, within Rhino's 
                        tolerance. False otherwise
    """
    value = False
    if elt(xyz1[0], xyz2[0]):
        value = True
    elif eeq(xyz1[0], xyz2[0]):
        if elt(xyz1[1], xyz2[1]):
            value = True
        elif eeq(xyz1[1], xyz2[1]):
            if elt(xyz1[2], xyz2[2]):
                value = True
    return value

def ene_xyz(xyz1, xyz2):
    """Receives:
        xyz1, xyz2      (num, num, num)
    Determines whether xyz1 is effectively not equal to xyz2. Returns:
        value           boolean. True if xyz1 != xyz2, within Rhino's 
                        tolerance. False otherwise
    """
    value = False
    if not _rs.PointCompare(xyz1, xyz2):
        value = True
    return value

def eeq(x, y):
    """Receives:
        x, y            num
    Determines whether x and y are effectively equal. Returns:
        value           boolean. True if x == y, within Rhino's tolerance. 
                        False otherwise
    """
    value = _rs.PointCompare((x, 1, 1), (y, 1, 1))
    return value

def ege(x, y):
    """Receives:
        x, y            num
    Determines whether x is effectively greater than or equal to y. 
    Returns: 
        value           boolean. True if x >= y, within Rhino's tolerance. 
                        False otherwise
    """
    value = False
    if (_rs.PointCompare((x, 1, 1), (y, 1, 1)) or
        x > y
    ):
        value = True
    return value

def egt(x, y):
    """Receives:
        x, y            num
    Determines whether x is effectively greater than y. Returns:
        value           boolean. True if x > y, within Rhino's tolerance. 
                        False otherwise
    """
    value = False
    if (not _rs.PointCompare((x, 1, 1), (y, 1, 1)) and
        x > y
    ):
        value = True
    return value

def ele(x, y):
    """Receives:
        x, y            num
    Determines whether x is effectively less than or equal to y. Returns: 
        value           boolean. True if x < y, within Rhino's tolerance. 
                        False otherwise
    """
    value = False
    if (_rs.PointCompare((x, 1, 1), (y, 1, 1)) or
        x < y
    ):
        value = True
    return value

def elt(x, y):
    """Receives:
        x, y            num
    Determines whether x is effectively less than y. Returns:
        value           boolean. True if x < y, within Rhino's tolerance. 
                        False otherwise
    """
    value = False
    if (not _rs.PointCompare((x, 1, 1), (y, 1, 1)) and
        x < y
    ):
        value = True
    return value

def ene(x, y):
    """Receives:
        x, y            num
    Determines whether x is effectively not equal to y. Returns:
        value           boolean. True if x != y, within Rhino's 
                        tolerance. False otherwise. 
    """
    value = False
    if not _rs.PointCompare(x, y):
        value = True
    return value

def get_point3d(xyz):
    """Receives:
        xyz             (num, num, num) 
    Returns:
        p3d             Point3D
    """
    p3d = _rs.PointAdd(xyz, _s.Settings.origin)
    return p3d

def is_a_list_of_strings(item):
    """Receives:
        item            any type
    Returns:
        value           boolean. True if item is a list (possibly empty) 
                        of strings. False otherwise
    """
    value = True
    if not type(item) == list:
        value = False
    else:
        elements = item
        for element in elements:
            if not type(element) == str:
                value = False
                break
    return value

def is_a_number(item):                 ##  copies elsewhere
    """Receives:
        item            Any type
    Returns:
        value           boolean. True if item is an int or a float. False 
                        otherwise
    """
    value = False
    if (type(item) == int or
        type(item) == float
    ):
        value = True
    return value

def _print_error_message(class_name, method_name, message):
    print '%s.%s:\n    %s' % (class_name, method_name, message)
